#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Wikidata... (ノಠ益ಠ)ノ

import re

RE_WIKIDATA_PREDICATE = re.compile('.*/(?P<wdtp>P[0-9]+)\s*$')

def translate_to_legible_wikidata_properties(wikidata_property):
    """Translates a wikidata property code into a legible property.

    >>>print translate_to_legible_wikidata_properties(u'P214')
    >>>'VIAF_ID_(P214)'

    """
    contains_wikidata_prop = re.match(RE_WIKIDATA_PREDICATE, wikidata_property)
    if contains_wikidata_prop:
        wikidata_predicate_code = contains_wikidata_prop.group('wdtp')
        return re.sub(
            wikidata_predicate_code,
            u'%s' % wdtprop.get(wikidata_predicate_code, wikidata_property),
            wikidata_property
        )
    return wikidata_property

# TODO: extract owl:sameAs links

wdtprop= {
    u'P6': u'head_of_government_(P6)',
    u'P10': u'video_(P10)',
    u'P14': u'graphic_symbol_of_thoroughfare_(P14)',
    u'P15': u'route_map_(P15)',
    u'P16': u'highway_system_(P16)',
    u'P17': u'country_(P17)',
    u'P18': u'image_(P18)',
    u'P19': u'place_of_birth_(P19)',
    u'P20': u'place_of_death_(P20)',
    u'P21': u'sex_or_gender_(P21)',
    u'P22': u'father_(P22)',
    u'P25': u'mother_(P25)',
    u'P26': u'spouse_(P26)',
    u'P27': u'country_of_citizenship_(P27)',
    u'P30': u'continent_(P30)',
    u'P31': u'instance_of_(P31)',
    u'P35': u'head_of_state_(P35)',
    u'P36': u'capital_(P36)',
    u'P37': u'official_language_(P37)',
    u'P38': u'currency_(P38)',
    u'P39': u'position_held_(P39)',
    u'P40': u'child_(P40)',
    u'P41': u'flag_image_(P41)',
    u'P47': u'shares_border_with_(P47)',
    u'P50': u'author_(P50)',
    u'P51': u'audio_(P51)',
    u'P53': u'family_(P53)',
    u'P54': u'member_of_sports_team_(P54)',
    u'P57': u'director_(P57)',
    u'P58': u'screenwriter_(P58)',
    u'P59': u'constellation_(P59)',
    u'P61': u'discoverer_or_inventor_(P61)',
    u'P65': u'site_of_astronomical_discovery_(P65)',
    u'P66': u'ancestral_home_(P66)',
    u'P69': u'educated_at_(P69)',
    u'P78': u'op-level_internet_domain_(P78)',
    u'P81': u'connecting_line_(P81)',
    u'P84': u'architect_(P84)',
    u'P85': u'anthem_(P85)',
    u'P86': u'composer_(P86)',
    u'P87': u'librettist_(P87)',
    u'P88': u'commissioned_by_(P88)',
    u'P91': u'sexual_orientation_(P91)',
    u'P92': u'main_regulatory_text_(P92)',
    u'P94': u'coat_of_arms_image_(P94)',
    u'P97': u'noble_title_(P97)',
    u'P98': u'editor_(P98)',
    u'P101': u'field_of_work_(P101)',
    u'P102': u'member_of_political_party_(P102)',
    u'P103': u'native_language_(P103)',
    u'P105': u'axon_rank_(P105)',
    u'P106': u'occupation_(P106)',
    u'P108': u'employer_(P108)',
    u'P109': u'signature_(P109)',
    u'P110': u'illustrator_(P110)',
    u'P111': u'measured_physical_quantity_(P111)',
    u'P112': u'founded_by_(P112)',
    u'P113': u'airline_hub_(P113)',
    u'P114': u'airline_alliance_(P114)',
    u'P115': u'home_venue_(P115)',
    u'P117': u'chemical_structure_(P117)',
    u'P118': u'league_(P118)',
    u'P119': u'place_of_interment_(P119)',
    u'P121': u'item_operated_(P121)',
    u'P122': u'basic_form_of_government_(P122)',
    u'P123': u'publisher_(P123)',
    u'P126': u'maintained_by_(P126)',
    u'P127': u'owned_by_(P127)',
    u'P128': u'regulates_(molecular_biology)_(P128)',
    u'P129': u'physically_interacts_with_(P129)',
    u'P131': u'located_in_the_administrative_territorial_entity_(P131)',
    u'P134': u'has_dialect_(P134)',
    u'P135': u'movement_(P135)',
    u'P136': u'genre_(P136)',
    u'P137': u'operator_(P137)',
    u'P138': u'named_after_(P138)',
    u'P140': u'religion_(P140)',
    u'P141': u'IUCN_conservation_status_(P141)',
    u'P143': u'imported_from_(P143)',
    u'P144': u'based_on_(P144)',
    u'P149': u'architectural_style_(P149)',
    u'P150': u'contains_administrative_territorial_entity_(P150)',
    u'P154': u'logo_image_(P154)',
    u'P155': u'follows_(P155)',
    u'P156': u'followed_by_(P156)',
    u'P157': u'killed_by_(P157)',
    u'P158': u'seal_image_(P158)',
    u'P159': u'headquarters_location_(P159)',
    u'P161': u'cast_member_(P161)',
    u'P162': u'producer_(P162)',
    u'P163': u'flag_(P163)',
    u'P166': u'award_received_(P166)',
    u'P167': u'structure_replaced_by_(P167)',
    u'P169': u'chief_executive_officer_(P169)',
    u'P170': u'creator_(P170)',
    u'P171': u'parent_taxon_(P171)',
    u'P172': u'ethnic_group_(P172)',
    u'P175': u'performer_(P175)',
    u'P176': u'manufacturer_(P176)',
    u'P177': u'crosses_(P177)',
    u'P178': u'developer_(P178)',
    u'P179': u'series_(P179)',
    u'P180': u'depicts_(P180)',
    u'P181': u'axon_range_map_image_(P181)',
    u'P183': u'endemic_to_(P183)',
    u'P184': u'doctoral_advisor_(P184)',
    u'P185': u'doctoral_student_(P185)',
    u'P186': u'material_used_(P186)',
    u'P189': u'location_of_discovery_(P189)',
    u'P190': u'sister_city_(P190)',
    u'P193': u'main_building_contractor_(P193)',
    u'P194': u'legislative_body_(P194)',
    u'P195': u'collection_(P195)',
    u'P196': u'minor_planet_group_(P196)',
    u'P197': u'adjacent_station_(P197)',
    u'P199': u'business_division_(P199)',
    u'P200': u'lake_inflows_(P200)',
    u'P201': u'lake_outflow_(P201)',
    u'P205': u'basin_country_(P205)',
    u'P206': u'located_in_or_next_to_body_of_water_(P206)',
    u'P207': u'bathymetry_image_(P207)',
    u'P208': u'executive_body_(P208)',
    u'P209': u'highest_judicial_authority_(P209)',
    u'P210': u'party_chief_representative_(P210)',
    u'P212': u'ISBN-13_(P212)',
    u'P213': u'ISNI_(P213)',
    u'P214': u'VIAF_ID_(P214)',
    u'P215': u'spectral_class_(P215)',
    u'P217': u'inventory_number_(P217)',
    u'P218': u'ISO_639-1_code_(P218)',
    u'P219': u'ISO_639-2_code_(P219)',
    u'P220': u'ISO_639-3_code_(P220)',
    u'P221': u'ISO_639-6_code_(P221)',
    u'P223': u'galaxy_morphological_type_(P223)',
    u'P225': u'axon_name_(P225)',
    u'P227': u'GND_ID_(P227)',
    u'P229': u'IATA_airline_designator_(P229)',
    u'P230': u'ICAO_airline_designator_(P230)',
    u'P231': u'CAS_Registry_Number_(P231)',
    u'P232': u'EC_no._(P232)',
    u'P233': u'canonical_SMILES_(P233)',
    u'P234': u'InChI_(P234)',
    u'P235': u'InChIKey_(P235)',
    u'P236': u'ISSN_(P236)',
    u'P237': u'coat_of_arms_(P237)',
    u'P238': u'IATA_airport_code_(P238)',
    u'P239': u'ICAO_airport_code_(P239)',
    u'P240': u'FAA_airport_code_(P240)',
    u'P241': u'military_branch_(P241)',
    u'P242': u'locator_map_image_(P242)',
    u'P243': u'OCLC_control_number_(P243)',
    u'P244': u'Library_of_Congress_authority_ID_(P244)',
    u'P245': u'ULAN_ID_(P245)',
    u'P246': u'element_symbol_(P246)',
    u'P247': u'COSPAR_ID_(P247)',
    u'P248': u'stated_in_(P248)',
    u'P249': u'icker_symbol_(P249)',
    u'P263': u'official_residence_(P263)',
    u'P264': u'record_label_(P264)',
    u'P267': u'ATC_code_(P267)',
    u'P268': u'BnF_ID_(P268)',
    u'P269': u'SUDOC_authorities_(P269)',
    u'P270': u'CALIS_ID_(P270)',
    u'P271': u'CiNii_author_ID_(P271)',
    u'P272': u'production_company_(P272)',
    u'P274': u'chemical_formula_(P274)',
    u'P275': u'license_(P275)',
    u'P276': u'location_(P276)',
    u'P277': u'programming_language_(P277)',
    u'P278': u'GOST_7.75-97_code_(P278)',
    u'P279': u'subclass_of_(P279)',
    u'P281': u'postal_code_(P281)',
    u'P282': u'writing_system_(P282)',
    u'P286': u'head_coach_(P286)',
    u'P287': u'designed_by_(P287)',
    u'P289': u'vessel_class_(P289)',
    u'P291': u'place_of_publication_(P291)',
    u'P296': u'station_code_(P296)',
    u'P297': u'ISO_3166-1_alpha-2_code_(P297)',
    u'P298': u'ISO_3166-1_alpha-3_code_(P298)',
    u'P299': u'ISO_3166-1_numeric_code_(P299)',
    u'P300': u'ISO_3166-2_code_(P300)',
    u'P301': u'category\'s_main_topic_(P301)',
    u'P303': u'EE_breed_number_(P303)',
    u'P304': u'page(s)_(P304)',
    u'P305': u'IETF_language_tag_(P305)',
    u'P306': u'operating_system_(P306)',
    u'P344': u'director_of_photography_(P344)',
    u'P345': u'IMDb_ID_(P345)',
    u'P347': u'Joconde_ID_(P347)',
    u'P348': u'software_version_(P348)',
    u'P349': u'NDL_Auth_ID_(P349)',
    u'P350': u'RKDimages_(P350)',
    u'P351': u'Entrez_Gene_ID_(P351)',
    u'P352': u'UniProt_protein_ID_(P352)',
    u'P353': u'HGNC_gene_symbol_(P353)',
    u'P354': u'HGNC_ID_(P354)',
    u'P355': u'subsidiary_(P355)',
    u'P356': u'DOI_(P356)',
    u'P358': u'discography_(P358)',
    u'P359': u'Rijksmonument_ID_(P359)',
    u'P360': u'is_a_list_of_(P360)',
    u'P361': u'part_of_(P361)',
    u'P364': u'original_language_of_work_(P364)',
    u'P366': u'use_(P366)',
    u'P367': u'astronomic_symbol_image_(P367)',
    u'P368': u'Sandbox-CommonsMediaFile_(P368)',
    u'P369': u'Sandbox-Item_(P369)',
    u'P370': u'Sandbox-String_(P370)',
    u'P371': u'presenter_(P371)',
    u'P373': u'Commons_category_(P373)',
    u'P374': u'INSEE_municipality_code_(P374)',
    u'P375': u'space_launch_vehicle_(P375)',
    u'P376': u'located_on_astronomical_body_(P376)',
    u'P377': u'SCN_(P377)',
    u'P380': u'Mérimée_ID_(P380)',
    u'P381': u'PCP_reference_number_(P381)',
    u'P382': u'CBS_municipality_code_(P382)',
    u'P393': u'edition_number_(P393)',
    u'P395': u'licence_plate_code_(P395)',
    u'P396': u'SBN_ID_(P396)',
    u'P397': u'parent_astronomical_body_(P397)',
    u'P398': u'child_astronomical_body_(P398)',
    u'P399': u'companion_of_(P399)',
    u'P400': u'platform_(P400)',
    u'P402': u'OpenStreetMap_Relation_identifier_(P402)',
    u'P403': u'mouth_of_the_watercourse_(P403)',
    u'P404': u'game_mode_(P404)',
    u'P405': u'axon_author_(P405)',
    u'P406': u'soundtrack_album_(P406)',
    u'P407': u'language_of_work_or_name_(P407)',
    u'P408': u'software_engine_(P408)',
    u'P409': u'NLA_(Australia)_ID_(P409)',
    u'P410': u'military_rank_(P410)',
    u'P411': u'canonization_status_(P411)',
    u'P412': u'voice_type_(P412)',
    u'P413': u'position_played_on_team_/_speciality_(P413)',
    u'P414': u'stock_exchange_(P414)',
    u'P415': u'radio_format_(P415)',
    u'P416': u'quantity_symbol_(P416)',
    u'P417': u'patron_saint_(P417)',
    u'P418': u'seal_description_(P418)',
    u'P421': u'located_in_time_zone_(P421)',
    u'P423': u'shooting_handedness_(P423)',
    u'P424': u'Wikimedia_language_code_(P424)',
    u'P425': u'field_of_this_occupation_(P425)',
    u'P426': u'aircraft_registration_(P426)',
    u'P427': u'axonomic_type_(P427)',
    u'P428': u'botanist_author_abbreviation_(P428)',
    u'P429': u'dantai_code_(P429)',
    u'P432': u'callsign_of_airline_(P432)',
    u'P433': u'issue_(P433)',
    u'P434': u'MusicBrainz_artist_ID_(P434)',
    u'P435': u'MusicBrainz_work_ID_(P435)',
    u'P436': u'MusicBrainz_release_group_ID_(P436)',
    u'P437': u'distribution_(P437)',
    u'P439': u'German_municipality_key_(P439)',
    u'P440': u'German_district_key_(P440)',
    u'P442': u'China_administrative_division_code_(P442)',
    u'P443': u'pronunciation_audio_(P443)',
    u'P444': u'review_score_(P444)',
    u'P447': u'score_by_(P447)',
    u'P449': u'original_network_(P449)',
    u'P450': u'astronaut_mission_(P450)',
    u'P451': u'partner_(P451)',
    u'P452': u'industry_(P452)',
    u'P453': u'character_role_(P453)',
    u'P454': u'Structurae_ID_(structure)_(P454)',
    u'P455': u'Emporis_building_ID_(P455)',
    u'P457': u'foundational_text_(P457)',
    u'P458': u'IMO_ship_number_(P458)',
    u'P459': u'determination_method_(P459)',
    u'P460': u'said_to_be_the_same_as_(P460)',
    u'P461': u'opposite_of_(P461)',
    u'P462': u'color_(P462)',
    u'P463': u'member_of_(P463)',
    u'P464': u'NOR_(P464)',
    u'P465': u'sRGB_color_hex_triplet_(P465)',
    u'P466': u'occupant_(P466)',
    u'P467': u'legislated_by_(P467)',
    u'P468': u'dan/kyu_rank_(P468)',
    u'P469': u'lakes_on_river_(P469)',
    u'P470': u'Eight_Banner_register_(P470)',
    u'P473': u'local_dialing_code_(P473)',
    u'P474': u'country_calling_code_(P474)',
    u'P476': u'CELEX_number_(P476)',
    u'P477': u'Canadian_Register_of_Historic_Places_ID_(P477)',
    u'P478': u'volume_(P478)',
    u'P479': u'input_method_(P479)',
    u'P480': u'FilmAffinity_ID_(P480)',
    u'P481': u'Palissy_ID_(P481)',
    u'P483': u'recorded_at_(P483)',
    u'P484': u'IMA_Number,_broad_sense_(P484)',
    u'P485': u'archives_at_(P485)',
    u'P486': u'MeSH_ID_(P486)',
    u'P487': u'Unicode_character_(P487)',
    u'P488': u'leader_(P488)',
    u'P489': u'currency_symbol_description_(P489)',
    u'P490': u'provisional_designation_(P490)',
    u'P491': u'orbit_diagram_(P491)',
    u'P492': u'OMIM_ID_(P492)',
    u'P493': u'ICD-9_(P493)',
    u'P494': u'ICD-10_(P494)',
    u'P495': u'country_of_origin_(P495)',
    u'P496': u'ORCID_iD_(P496)',
    u'P497': u'CBDB_ID_(P497)',
    u'P498': u'ISO_4217_code_(P498)',
    u'P500': u'exclave_of_(P500)',
    u'P501': u'enclave_within_(P501)',
    u'P502': u'HURDAT_identifier_(P502)',
    u'P503': u'ISO_standard_(P503)',
    u'P504': u'home_port_(P504)',
    u'P505': u'general_manager_(P505)',
    u'P506': u'ISO_15924_alpha-4_code_(P506)',
    u'P507': u'Swedish_county_code_(P507)',
    u'P508': u'BNCF_Thesaurus_(P508)',
    u'P509': u'cause_of_death_(P509)',
    u'P511': u'honorific_prefix_(P511)',
    u'P512': u'academic_degree_(P512)',
    u'P514': u'interleaves_with_(P514)',
    u'P515': u'phase_of_matter_(P515)',
    u'P516': u'powerplant_(P516)',
    u'P517': u'interaction_(P517)',
    u'P518': u'applies_to_part_(P518)',
    u'P520': u'armament_(P520)',
    u'P521': u'scheduled_service_destination_(P521)',
    u'P522': u'ype_of_orbit_(P522)',
    u'P523': u'emporal_range_start_(P523)',
    u'P524': u'emporal_range_end_(P524)',
    u'P525': u'Swedish_municipality_code_(P525)',
    u'P527': u'has_part_(P527)',
    u'P528': u'catalog_code_(P528)',
    u'P529': u'runway_(P529)',
    u'P530': u'diplomatic_relation_(P530)',
    u'P531': u'diplomatic_mission_sent_(P531)',
    u'P532': u'port_of_registry_(P532)',
    u'P533': u'arget_(P533)',
    u'P534': u'streak_color_(P534)',
    u'P535': u'Find_a_Grave_memorial_ID_(P535)',
    u'P536': u'ATP_player_ID_(P536)',
    u'P537': u'winning_(P537)',
    u'P538': u'fracturing_(P538)',
    u'P539': u'Museofile_(P539)',
    u'P541': u'office_contested_(P541)',
    u'P542': u'officially_opened_by_(P542)',
    u'P543': u'oath_made_by_(P543)',
    u'P545': u'orch_lit_by_(P545)',
    u'P546': u'docking_port_(P546)',
    u'P547': u'commemorates_(P547)',
    u'P548': u'version_type_(P548)',
    u'P549': u'Mathematics_Genealogy_Project_ID_(P549)',
    u'P550': u'chivalric_order_(P550)',
    u'P551': u'residence_(P551)',
    u'P552': u'handedness_(P552)',
    u'P553': u'website_account_on_(P553)',
    u'P554': u'website_username_(P554)',
    u'P555': u'doubles_record_(P555)',
    u'P556': u'crystal_system_(P556)',
    u'P557': u'DiseasesDB_(P557)',
    u'P558': u'unit_symbol_(P558)',
    u'P559': u'erminus_(P559)',
    u'P560': u'direction_(P560)',
    u'P561': u'NATO_reporting_name_(P561)',
    u'P562': u'central_bank/issuer_(P562)',
    u'P563': u'ICD-O_(P563)',
    u'P564': u'singles_record_(P564)',
    u'P565': u'crystal_habit_(P565)',
    u'P566': u'basionym_(P566)',
    u'P567': u'underlies_(P567)',
    u'P568': u'overlies_(P568)',
    u'P569': u'date_of_birth_(P569)',
    u'P570': u'date_of_death_(P570)',
    u'P571': u'inception_(P571)',
    u'P574': u'date_of_taxon_name_publication_(P574)',
    u'P575': u'ime_of_discovery_(P575)',
    u'P576': u'dissolved,_abolished_or_demolished_(P576)',
    u'P577': u'publication_date_(P577)',
    u'P578': u'Sandbox-TimeValue_(P578)',
    u'P579': u'IMA_status_and/or_rank_(P579)',
    u'P580': u'start_time_(P580)',
    u'P582': u'end_time_(P582)',
    u'P585': u'point_in_time_(P585)',
    u'P586': u'IPNI_author_ID_(P586)',
    u'P587': u'MMSI_(P587)',
    u'P588': u'coolant_(P588)',
    u'P589': u'point_group_(P589)',
    u'P590': u'GNIS_ID_(P590)',
    u'P591': u'EC_number_(P591)',
    u'P592': u'ChEMBL_ID_(P592)',
    u'P593': u'HomoloGene_ID_(P593)',
    u'P594': u'Ensembl_Gene_ID_(P594)',
    u'P595': u'Guide_to_Pharmacology_Ligand_ID_(P595)',
    u'P597': u'WTA_player_ID_(P597)',
    u'P598': u'commander_of_(P598)',
    u'P599': u'ITF_ID_(P599)',
    u'P600': u'Wine_AppDB-ID_(P600)',
    u'P604': u'MedlinePlus_ID_(P604)',
    u'P605': u'NUTS_code_(P605)',
    u'P606': u'first_flight_(P606)',
    u'P607': u'conflict_(P607)',
    u'P608': u'exhibition_history_(P608)',
    u'P609': u'erminus_location_(P609)',
    u'P610': u'highest_point_(P610)',
    u'P611': u'religious_order_(P611)',
    u'P612': u'mother_house_(P612)',
    u'P613': u'OS_grid_reference_(P613)',
    u'P617': u'yard_number_(P617)',
    u'P618': u'source_of_energy_(P618)',
    u'P619': u'ime_of_spacecraft_launch_(P619)',
    u'P620': u'ime_of_spacecraft_landing_(P620)',
    u'P621': u'ime_of_spacecraft_orbit_decay_(P621)',
    u'P622': u'spacecraft_docking/undocking_date_(P622)',
    u'P624': u'guidance_system_(P624)',
    u'P625': u'coordinate_location_(P625)',
    u'P626': u'Sandbox-GeoCoordinateValue_(P626)',
    u'P627': u'IUCN_taxon_ID_(P627)',
    u'P628': u'E_number_(P628)',
    u'P629': u'edition_or_translation_of_(P629)',
    u'P630': u'Paris_city_digital_code_(P630)',
    u'P631': u'structural_engineer_(P631)',
    u'P632': u'cultural_properties_of_Belarus_reference_number_(P632)',
    u'P633': u'Quebec_cultural_heritage_directory_ID_(P633)',
    u'P634': u'captain_(P634)',
    u'P635': u'ISTAT_ID_(P635)',
    u'P636': u'route_of_administration_(P636)',
    u'P637': u'RefSeq_Protein_ID_(P637)',
    u'P638': u'PDB_ID_(P638)',
    u'P639': u'RefSeq_RNA_ID_(P639)',
    u'P640': u'Léonore_ID_(P640)',
    u'P641': u'sport_(P641)',
    u'P642': u'of_(P642)',
    u'P644': u'genomic_start_(P644)',
    u'P645': u'genomic_end_(P645)',
    u'P646': u'Freebase_ID_(P646)',
    u'P647': u'drafted_by_(P647)',
    u'P648': u'Open_Library_ID_(P648)',
    u'P649': u'NRHP_reference_number_(P649)',
    u'P650': u'RKDartists_ID_(P650)',
    u'P651': u'Biografisch_Portaal_number_(P651)',
    u'P652': u'UNII_(P652)',
    u'P653': u'PubMed_Health_(P653)',
    u'P654': u'direction_relative_to_location_(P654)',
    u'P655': u'ranslator_(P655)',
    u'P656': u'RefSeq_(P656)',
    u'P657': u'RTECS_number_(P657)',
    u'P658': u'racklist_(P658)',
    u'P659': u'genomic_assembly_(P659)',
    u'P660': u'EC_classification_(P660)',
    u'P661': u'ChemSpider_ID_(P661)',
    u'P662': u'PubChem_CID_(P662)',
    u'P663': u'DSM-IV_(P663)',
    u'P664': u'organizer_(P664)',
    u'P665': u'KEGG_ID_(P665)',
    u'P667': u'ICPC_2_ID_(P667)',
    u'P668': u'GeneReviews_ID_(P668)',
    u'P669': u'located_on_street_(P669)',
    u'P670': u'street_number_(P670)',
    u'P671': u'Mouse_Genome_Informatics_ID_(P671)',
    u'P672': u'MeSH_Code_(P672)',
    u'P673': u'eMedicine_(P673)',
    u'P674': u'characters_(P674)',
    u'P675': u'Google_Books_ID_(P675)',
    u'P676': u'lyrics_by_(P676)',
    u'P677': u'ÚSOP_code_(P677)',
    u'P678': u'incertae_sedis_(P678)',
    u'P679': u'ZVG_number_(P679)',
    u'P680': u'molecular_function_(P680)',
    u'P681': u'cell_component_(P681)',
    u'P682': u'biological_process_(P682)',
    u'P683': u'ChEBI_ID_(P683)',
    u'P684': u'ortholog_(P684)',
    u'P685': u'NCBI_Taxonomy_ID_(P685)',
    u'P686': u'Gene_Ontology_ID_(P686)',
    u'P687': u'BHL_Page_ID_(P687)',
    u'P688': u'encodes_(P688)',
    u'P689': u'afflicts_(P689)',
    u'P690': u'space_group_(P690)',
    u'P691': u'NKCR_AUT_ID_(P691)',
    u'P692': u'Gene_Atlas_Image_(P692)',
    u'P693': u'cleavage_(P693)',
    u'P694': u'replaced_synonym_(for_nom._nov.)_(P694)',
    u'P695': u'UN_number_(P695)',
    u'P696': u'Neurolex_ID_(P696)',
    u'P697': u'ex_taxon_author_(P697)',
    u'P698': u'PubMed_ID_(P698)',
    u'P699': u'Disease_Ontology_ID_(P699)',
    u'P700': u'Kemler_code_(P700)',
    u'P701': u'Dodis_ID_(P701)',
    u'P702': u'encoded_by_(P702)',
    u'P703': u'found_in_taxon_(P703)',
    u'P704': u'Ensembl_Transcript_ID_(P704)',
    u'P705': u'Ensembl_Protein_ID_(P705)',
    u'P706': u'located_on_terrain_feature_(P706)',
    u'P707': u'satellite_bus_(P707)',
    u'P708': u'diocese_(P708)',
    u'P709': u'Historic_Scotland_ID_(P709)',
    u'P710': u'participant_(P710)',
    u'P711': u'Strunz_8th_edition_(series_ID,_updated)_(P711)',
    u'P712': u'Nickel-Strunz_9th_edition_(updated_2009)_(P712)',
    u'P713': u'Nickel-Strunz_10th_(pending)_edition_(P713)',
    u'P714': u'Dana_8th_edition_(P714)',
    u'P715': u'Drugbank_ID_(P715)',
    u'P716': u'JPL_Small-Body_Database_ID_(P716)',
    u'P717': u'Minor_Planet_Center_observatory_code_(P717)',
    u'P718': u'Canmore_ID_(P718)',
    u'P720': u'asteroid_spectral_type_(P720)',
    u'P721': u'OKATO_ID_(P721)',
    u'P722': u'UIC_station_code_(P722)',
    u'P723': u'DBNL_author_ID_(P723)',
    u'P724': u'Internet_Archive_ID_(P724)',
    u'P725': u'voice_actor_(P725)',
    u'P726': u'candidate_(P726)',
    u'P727': u'Europeana_ID_(P727)',
    u'P728': u'GHS_hazard_statement_(P728)',
    u'P729': u'service_entry_(P729)',
    u'P730': u'service_retirement_(P730)',
    u'P731': u'Litholex_ID_(P731)',
    u'P732': u'BGS_Lexicon_ID_(P732)',
    u'P733': u'DINOloket_(P733)',
    u'P734': u'family_name_(P734)',
    u'P735': u'given_name_(P735)',
    u'P736': u'cover_artist_(P736)',
    u'P737': u'influenced_by_(P737)',
    u'P739': u'ammunition_(P739)',
    u'P740': u'location_of_formation_(P740)',
    u'P741': u'playing_hand_(P741)',
    u'P742': u'pseudonym_(P742)',
    u'P744': u'asteroid_family_(P744)',
    u'P745': u'Low_German_Bibliography_and_Biography_ID_(P745)',
    u'P746': u'date_of_disappearance_(P746)',
    u'P747': u'edition_(P747)',
    u'P748': u'appointed_by_(P748)',
    u'P749': u'parent_organization_(P749)',
    u'P750': u'distributor_(P750)',
    u'P751': u'introduced_feature_(P751)',
    u'P756': u'removed_feature_(P756)',
    u'P757': u'World_Heritage_Site_ID_(P757)',
    u'P758': u'Kulturminne_ID_(P758)',
    u'P759': u'Alberta_Register_of_Historic_Places_ID_(P759)',
    u'P760': u'DPLA_ID_(P760)',
    u'P761': u'Lake_ID_(Sweden)_(P761)',
    u'P762': u'Czech_cultural_heritage_ID_(P762)',
    u'P763': u'PEI_Register_of_Historic_Places_ID_(P763)',
    u'P764': u'OKTMO_ID_(P764)',
    u'P765': u'surface_played_on_(P765)',
    u'P767': u'contributor(s)_to_the_creative_work_(P767)',
    u'P768': u'electoral_district_(P768)',
    u'P769': u'significant_drug_interaction_(P769)',
    u'P770': u'cause_of_destruction_(P770)',
    u'P771': u'Swiss_municipality_code_(P771)',
    u'P772': u'INE_municipality_code_(P772)',
    u'P773': u'ISO_3166-3_(P773)',
    u'P774': u'FIPS_55-3_(locations_in_the_US)_(P774)',
    u'P775': u'Swedish_urban_area_code_(P775)',
    u'P776': u'Swedish_minor_urban_area_code_(P776)',
    u'P777': u'Swedish_civil_parish_code/ATA_code_(P777)',
    u'P778': u'Church_of_Sweden_parish_code_(P778)',
    u'P779': u'Church_of_Sweden_Pastoratskod_(P779)',
    u'P780': u'symptoms_(P780)',
    u'P781': u'SIKART_ID_(P781)',
    u'P782': u'LAU_(P782)',
    u'P783': u'hymenium_type_(P783)',
    u'P784': u'mushroom_cap_shape_(P784)',
    u'P785': u'hymenium_attachment_(P785)',
    u'P786': u'stipe_character_(P786)',
    u'P787': u'spore_print_color_(P787)',
    u'P788': u'mushroom_ecological_type_(P788)',
    u'P789': u'edibility_(P789)',
    u'P790': u'approved_by_(P790)',
    u'P791': u'International_Standard_Identifier_for_Libraries_(P791)',
    u'P792': u'chapter_(P792)',
    u'P793': u'significant_event_(P793)',
    u'P795': u'located_on_linear_feature_(P795)',
    u'P797': u'authority_(P797)',
    u'P798': u'military_designation_(P798)',
    u'P799': u'Air_Ministry_specification_ID_(P799)',
    u'P800': u'notable_work_(P800)',
    u'P802': u'student_(P802)',
    u'P803': u'professorship_(P803)',
    u'P804': u'GNIS_Antarctica_ID_(P804)',
    u'P805': u'statement_is_subject_of_(P805)',
    u'P806': u'Italian_cadastre_code_(P806)',
    u'P807': u'separated_from_(P807)',
    u'P808': u'code_Bien_de_Interés_Cultural_(P808)',
    u'P809': u'WDPA_ID_(P809)',
    u'P811': u'academic_minor_(P811)',
    u'P812': u'academic_major_(P812)',
    u'P813': u'retrieved_(P813)',
    u'P814': u'IUCN_protected_areas_category_(P814)',
    u'P815': u'ITIS_TSN_(P815)',
    u'P816': u'decays_to_(P816)',
    u'P817': u'decay_mode_(P817)',
    u'P818': u'arXiv_ID_(P818)',
    u'P819': u'ADS_bibcode_(P819)',
    u'P820': u'arXiv_classification_(P820)',
    u'P821': u'CGNDB_Unique_ID_(P821)',
    u'P822': u'mascot_(P822)',
    u'P823': u'speaker_(P823)',
    u'P824': u'Meteoritical_Bulletin_Database_ID_(P824)',
    u'P825': u'dedicated_to_(P825)',
    u'P826': u'onality_(P826)',
    u'P827': u'BBC_programme_ID_(P827)',
    u'P828': u'has_cause_(P828)',
    u'P829': u'OEIS_ID_(P829)',
    u'P830': u'Encyclopedia_of_Life_ID_(P830)',
    u'P831': u'parent_club_(P831)',
    u'P832': u'public_holiday_(P832)',
    u'P833': u'interchange_station_(P833)',
    u'P834': u'rain_depot_(P834)',
    u'P835': u'author_citation_(zoology)_(P835)',
    u'P836': u'GSS_code_(2011)_(P836)',
    u'P837': u'day_in_year_for_periodic_occurrence_(P837)',
    u'P838': u'BioLib_ID_(P838)',
    u'P839': u'IMSLP_ID_(P839)',
    u'P840': u'narrative_location_(P840)',
    u'P841': u'feast_day_(P841)',
    u'P842': u'Fossilworks_ID_(P842)',
    u'P843': u'SIRUTA_code_(P843)',
    u'P844': u'UBIGEO_code_(P844)',
    u'P845': u'Saskatchewan_Register_of_Heritage_Property_ID_(P845)',
    u'P846': u'Global_Biodiversity_Information_Facility_ID_(P846)',
    u'P847': u'United_States_Navy_aircraft_designation_(P847)',
    u'P849': u'Japanese_military_aircraft_designation_(P849)',
    u'P850': u'WoRMS-ID_(P850)',
    u'P852': u'ESRB_rating_(P852)',
    u'P853': u'CERO_rating_(P853)',
    u'P854': u'reference_URL_(P854)',
    u'P855': u'Sandbox-URL_(P855)',
    u'P856': u'official_website_(P856)',
    u'P858': u'ESPNscrum_player_ID_(P858)',
    u'P859': u'sponsor_(P859)',
    u'P860': u'e-archiv.li_ID_(P860)',
    u'P861': u'PremiershipRugby.com_ID_(P861)',
    u'P862': u'Operational_Requirement_of_the_UK_Air_Ministry_(P862)',
    u'P863': u'InPhO_ID_(P863)',
    u'P864': u'ACM_Digital_Library_author_ID_(P864)',
    u'P865': u'BMLO_(P865)',
    u'P866': u'Perlentaucher_ID_(P866)',
    u'P867': u'ROME_Occupation_Code_(v3)_(P867)',
    u'P868': u'foods_traditionally_associated_(P868)',
    u'P870': u'instrumentation_(P870)',
    u'P872': u'printed_by_(P872)',
    u'P873': u'phase_point_(P873)',
    u'P874': u'UN_class_(P874)',
    u'P875': u'UN_code_classification_(P875)',
    u'P876': u'UN_packaging_group_(P876)',
    u'P877': u'NFPA_Other_(P877)',
    u'P878': u'avionics_(P878)',
    u'P879': u'pennant_number_(P879)',
    u'P880': u'CPU_(P880)',
    u'P881': u'ype_of_variable_star_(P881)',
    u'P882': u'FIPS_6-4_(US_counties)_(P882)',
    u'P883': u'FIPS_5-2_(code_for_US_states)_(P883)',
    u'P884': u'State_Water_Register_Code_(Russia)_(P884)',
    u'P885': u'origin_of_the_watercourse_(P885)',
    u'P886': u'LIR_(P886)',
    u'P887': u'based_on_heuristic_(P887)',
    u'P888': u'JSTOR_article_ID_(P888)',
    u'P889': u'Mathematical_Reviews_ID_(P889)',
    u'P892': u'RfC_ID_(P892)',
    u'P893': u'Social_Science_Research_Network_ID_(P893)',
    u'P894': u'zbMATH_work_ID_(P894)',
    u'P897': u'United_States_Army_and_Air_Force_aircraft_designation_(P897)',
    u'P898': u'IPA_transcription_(P898)',
    u'P901': u'FIPS_10-4_(countries_and_regions)_(P901)',
    u'P902': u'HDS_ID_(P902)',
    u'P905': u'PORT_film_ID_(P905)',
    u'P906': u'SELIBR_(P906)',
    u'P908': u'PEGI_rating_(P908)',
    u'P909': u'Nova_Scotia_Register_of_Historic_Places_ID_(P909)',
    u'P910': u'opic\'s_main_category_(P910)',
    u'P911': u'South_African_municipality_code_(P911)',
    u'P912': u'has_facility_(P912)',
    u'P913': u'notation_(P913)',
    u'P914': u'USK_rating_(P914)',
    u'P915': u'filming_location_(P915)',
    u'P916': u'GSRR_rating_(P916)',
    u'P917': u'GRAU_index_(P917)',
    u'P918': u'NOC_Occupation_Code_(P918)',
    u'P919': u'SOC_Code_(2010)_(P919)',
    u'P920': u'LEM_ID_(P920)',
    u'P921': u'main_subject_(P921)',
    u'P922': u'magnetic_ordering_(P922)',
    u'P923': u'medical_examinations_(P923)',
    u'P924': u'medical_treatment_(P924)',
    u'P925': u'presynaptic_connection_(P925)',
    u'P926': u'postsynaptic_connection_(P926)',
    u'P927': u'anatomical_location_(P927)',
    u'P928': u'activating_neurotransmitter_(P928)',
    u'P929': u'color_space_(P929)',
    u'P930': u'ype_of_electrification_(P930)',
    u'P931': u'place_served_by_transport_hub_(P931)',
    u'P932': u'PMCID_(P932)',
    u'P933': u'heritagefoundation.ca_ID_(P933)',
    u'P935': u'Commons_gallery_(P935)',
    u'P937': u'work_location_(P937)',
    u'P938': u'FishBase_species_ID_(P938)',
    u'P939': u'KSH_code_(P939)',
    u'P940': u'GHS_precautionary_statements_(P940)',
    u'P941': u'inspired_by_(P941)',
    u'P942': u'heme_music_(P942)',
    u'P943': u'programmer_(P943)',
    u'P944': u'Code_of_nomenclature_(P944)',
    u'P945': u'allegiance_(P945)',
    u'P946': u'ISIN_(P946)',
    u'P947': u'RSL_ID_(person)_(P947)',
    u'P948': u'page_banner_(P948)',
    u'P949': u'National_Library_of_Israel_ID_(P949)',
    u'P950': u'BNE_ID_(P950)',
    u'P951': u'NSZL_ID_(P951)',
    u'P952': u'ISCO_occupation_code_(P952)',
    u'P953': u'full_work_available_at_(P953)',
    u'P954': u'IBNR_ID_(P954)',
    u'P957': u'ISBN-10_(P957)',
    u'P958': u'section,_verse,_or_paragraph_(P958)',
    u'P959': u'MSW_ID_(P959)',
    u'P960': u'Tropicos_ID_(P960)',
    u'P961': u'IPNI_plant_ID_(P961)',
    u'P962': u'MycoBank_taxon_name_ID_(P962)',
    u'P963': u'streaming_media_URL_(P963)',
    u'P964': u'Austrian_municipality_key_(P964)',
    u'P965': u'burial_plot_reference_(P965)',
    u'P966': u'MusicBrainz_label_ID_(P966)',
    u'P967': u'guest_of_honor_(P967)',
    u'P968': u'e-mail_(P968)',
    u'P969': u'located_at_street_address_(P969)',
    u'P970': u'neurological_function_(P970)',
    u'P971': u'category_combines_topics_(P971)',
    u'P972': u'catalog_(P972)',
    u'P973': u'described_at_URL_(P973)',
    u'P974': u'ributary_(P974)',
    u'P980': u'code_for_weekend_and_holiday_homes_(Sweden)_(P980)',
    u'P981': u'BAG-code_for_Dutch_locations_(P981)',
    u'P982': u'MusicBrainz_area_ID_(P982)',
    u'P984': u'IOC_country_code_(P984)',
    u'P988': u'Philippine_Standard_Geographic_Code_(P988)',
    u'P989': u'spoken_text_audio_(P989)',
    u'P990': u'audio_recording_of_the_subject\'s_spoken_voice_(P990)',
    u'P991': u'successful_candidate_(P991)',
    u'P993': u'NFPA_Health_(P993)',
    u'P994': u'NFPA_Fire_(P994)',
    u'P995': u'NFPA_Reactivity_(P995)',
    u'P996': u'scanned_file_on_Wikimedia_Commons_(P996)',
    u'P998': u'DMOZ_ID_(P998)',
    u'P999': u'ARICNS_(P999)',
    u'P1000': u'record_held_(P1000)',
    u'P1001': u'applies_to_jurisdiction_(P1001)',
    u'P1002': u'engine_configuration_(P1002)',
    u'P1003': u'NLR_(Romania)_ID_(P1003)',
    u'P1004': u'MusicBrainz_place_ID_(P1004)',
    u'P1005': u'PTBNP_ID_(P1005)',
    u'P1006': u'National_Thesaurus_for_Author_Names_ID_(P1006)',
    u'P1007': u'Lattes_Platform_number_(P1007)',
    u'P1010': u'Iran_statistics_ID_(P1010)',
    u'P1011': u'excluding_(P1011)',
    u'P1012': u'including_(P1012)',
    u'P1013': u'criterion_used_(P1013)',
    u'P1014': u'AAT_ID_(P1014)',
    u'P1015': u'BIBSYS_ID_(P1015)',
    u'P1016': u'asteroid_taxonomy_(P1016)',
    u'P1017': u'BAV_ID_(P1017)',
    u'P1018': u'language_regulatory_body_(P1018)',
    u'P1019': u'feed_URL_(P1019)',
    u'P1021': u'KldB-2010_occupation_code_(P1021)',
    u'P1022': u'CNO-11_occupation_code_(P1022)',
    u'P1023': u'SBC-2010_occupation_code_(P1023)',
    u'P1024': u'SBFI_occupation_code_(P1024)',
    u'P1025': u'SUDOC_editions_(P1025)',
    u'P1026': u'doctoral_thesis_(P1026)',
    u'P1027': u'conferred_by_(P1027)',
    u'P1028': u'donated_by_(P1028)',
    u'P1029': u'crew_member_(P1029)',
    u'P1030': u'light_characteristic_of_lighthouse_(P1030)',
    u'P1031': u'legal_citation_of_this_text_(P1031)',
    u'P1032': u'Digital_Rights_Management_system_(P1032)',
    u'P1033': u'GHS_signal_word_(P1033)',
    u'P1034': u'main_food_source_(P1034)',
    u'P1035': u'honorific_suffix_(P1035)',
    u'P1036': u'Dewey_Decimal_Classification_(P1036)',
    u'P1037': u'manager/director_(P1037)',
    u'P1038': u'relative_(P1038)',
    u'P1039': u'ype_of_kinship_(P1039)',
    u'P1040': u'film_editor_(P1040)',
    u'P1041': u'socket_supported_(P1041)',
    u'P1042': u'ZDB_ID_(P1042)',
    u'P1043': u'IDEO_Job_ID_(P1043)',
    u'P1044': u'SWB_editions_(P1044)',
    u'P1045': u'Sycomore_ID_(P1045)',
    u'P1046': u'discovery_method_(P1046)',
    u'P1047': u'Catholic_Hierarchy_person_ID_(P1047)',
    u'P1048': u'NCL_ID_(P1048)',
    u'P1049': u'worshipped_by_(P1049)',
    u'P1050': u'medical_condition_(P1050)',
    u'P1051': u'PSH_ID_(P1051)',
    u'P1052': u'Portuguese_Job_Code_CPP-2010_(P1052)',
    u'P1053': u'ResearcherID_(P1053)',
    u'P1054': u'NDL_bib_ID_(P1054)',
    u'P1055': u'NLM_Unique_ID_(P1055)',
    u'P1056': u'product_or_material_produced_(P1056)',
    u'P1057': u'chromosome_(P1057)',
    u'P1058': u'ERA_Journal_ID_(P1058)',
    u'P1059': u'CVR_number_(P1059)',
    u'P1060': u'pathogen_transmission_process_(P1060)',
    u'P1064': u'rack_gauge_(P1064)',
    u'P1065': u'archive_URL_(P1065)',
    u'P1066': u'student_of_(P1066)',
    u'P1067': u'Thailand_central_administrative_unit_code_(P1067)',
    u'P1068': u'instruction_set_(P1068)',
    u'P1069': u'Statistics_Denmarks_classification_of_occupation_(DISCO-08)_(P1069)',
    u'P1070': u'PlantList-ID_(P1070)',
    u'P1071': u'location_of_final_assembly_(P1071)',
    u'P1072': u'readable_file_format_(P1072)',
    u'P1073': u'writable_file_format_(P1073)',
    u'P1074': u'fictional_analog_of_(P1074)',
    u'P1075': u'rector_(P1075)',
    u'P1076': u'ICTV_virus_ID_(P1076)',
    u'P1077': u'KOATUU_identifier_(P1077)',
    u'P1078': u'valvetrain_configuration_(P1078)',
    u'P1079': u'launch_contractor_(P1079)',
    u'P1080': u'from_fictional_universe_(P1080)',
    u'P1081': u'Human_Development_Index_(P1081)',
    u'P1082': u'population_(P1082)',
    u'P1083': u'maximum_capacity_(P1083)',
    u'P1084': u'EUL_editions_(P1084)',
    u'P1085': u'LibraryThing_work_ID_(P1085)',
    u'P1086': u'atomic_number_(P1086)',
    u'P1087': u'Elo_rating_(P1087)',
    u'P1088': u'Mohs\'_hardness_(P1088)',
    u'P1090': u'redshift_(P1090)',
    u'P1092': u'otal_produced_(P1092)',
    u'P1093': u'gross_tonnage_(P1093)',
    u'P1096': u'orbital_eccentricity_(P1096)',
    u'P1097': u'g-factor_(P1097)',
    u'P1098': u'number_of_speakers_(P1098)',
    u'P1099': u'number_of_masts_(P1099)',
    u'P1100': u'number_of_cylinders_(P1100)',
    u'P1101': u'floors_above_ground_(P1101)',
    u'P1102': u'flattening_(P1102)',
    u'P1103': u'number_of_platform_tracks_(P1103)',
    u'P1104': u'number_of_pages_(P1104)',
    u'P1106': u'Sandbox-Quantity_(P1106)',
    u'P1107': u'proportion_(P1107)',
    u'P1108': u'electronegativity_(P1108)',
    u'P1109': u'refractive_index_(P1109)',
    u'P1110': u'attendance_(P1110)',
    u'P1111': u'votes_received_(P1111)',
    u'P1112': u'Pokédex_number_(P1112)',
    u'P1113': u'number_of_episodes_(P1113)',
    u'P1114': u'quantity_(P1114)',
    u'P1115': u'ATVK_ID_(P1115)',
    u'P1116': u'ELSTAT_geographical_code_(P1116)',
    u'P1117': u'pKa_(P1117)',
    u'P1120': u'number_of_deaths_(P1120)',
    u'P1121': u'oxidation_state_(P1121)',
    u'P1122': u'spin_quantum_number_(P1122)',
    u'P1123': u'parity_(P1123)',
    u'P1124': u'TEU_(P1124)',
    u'P1125': u'Gini_coefficient_(P1125)',
    u'P1126': u'isospin_quantum_number_(P1126)',
    u'P1127': u'isospin_z-component_(P1127)',
    u'P1128': u'employees_(P1128)',
    u'P1129': u'national_team_caps_(P1129)',
    u'P1132': u'number_of_participants_(P1132)',
    u'P1133': u'DGO4_identifier_(P1133)',
    u'P1135': u'nomenclatural_status_(P1135)',
    u'P1136': u'solved_by_(P1136)',
    u'P1137': u'fossil_found_in_this_unit_(P1137)',
    u'P1138': u'Kunstindeks_Danmark_Artist_ID_(P1138)',
    u'P1139': u'floors_below_ground_(P1139)',
    u'P1140': u'EHAK_id_(P1140)',
    u'P1141': u'number_of_processor_cores_(P1141)',
    u'P1142': u'political_ideology_(P1142)',
    u'P1143': u'BN_(Argentine)_editions_(P1143)',
    u'P1144': u'LCOC_LCCN_(bibliographic)_(P1144)',
    u'P1145': u'Lagrangian_point_(P1145)',
    u'P1146': u'IAAF_ID_(P1146)',
    u'P1148': u'neutron_number_(P1148)',
    u'P1149': u'Library_of_Congress_Classification_(P1149)',
    u'P1150': u'Regensburg_Classification_(P1150)',
    u'P1151': u'opic\'s_main_Wikimedia_portal_(P1151)',
    u'P1153': u'Scopus_Author_ID_(P1153)',
    u'P1154': u'Scopus_EID_(P1154)',
    u'P1155': u'Scopus_Affiliation_ID_(P1155)',
    u'P1156': u'Scopus_Source_ID_(P1156)',
    u'P1157': u'US_Congress_Bio_ID_(P1157)',
    u'P1158': u'location_of_landing_(P1158)',
    u'P1159': u'CODEN_(P1159)',
    u'P1160': u'ISO_4_abbreviation_(P1160)',
    u'P1161': u'Z39.5_abbreviation_(P1161)',
    u'P1162': u'Bluebook_abbreviation_(P1162)',
    u'P1163': u'media_type_(P1163)',
    u'P1164': u'cardinality_of_the_group_(P1164)',
    u'P1165': u'home_world_(P1165)',
    u'P1167': u'USB_vendor_ID_(P1167)',
    u'P1168': u'municipality_code_(Denmark)_(P1168)',
    u'P1170': u'ransmitted_signal_type_(P1170)',
    u'P1171': u'approximation_algorithm_(P1171)',
    u'P1172': u'Geokod_(P1172)',
    u'P1174': u'visitors_per_year_(P1174)',
    u'P1181': u'numeric_value_(P1181)',
    u'P1182': u'LIBRIS_editions_(P1182)',
    u'P1183': u'Gewässerkennzahl_(P1183)',
    u'P1184': u'handle_(P1184)',
    u'P1185': u'Rodovid_ID_(P1185)',
    u'P1186': u'MEP_directory_ID_(P1186)',
    u'P1187': u'Dharma_Drum_Buddhist_College_person_ID_(P1187)',
    u'P1188': u'Dharma_Drum_Buddhist_College_place_ID_(P1188)',
    u'P1189': u'Chinese_Library_Classification_(P1189)',
    u'P1190': u'Universal_Decimal_Classification_(P1190)',
    u'P1191': u'date_of_first_performance_(P1191)',
    u'P1192': u'connecting_service_(P1192)',
    u'P1193': u'prevalence_(P1193)',
    u'P1194': u'received_signal_type_(P1194)',
    u'P1195': u'file_extension_(P1195)',
    u'P1196': u'manner_of_death_(P1196)',
    u'P1198': u'unemployment_rate_(P1198)',
    u'P1199': u'mode_of_inheritance_(P1199)',
    u'P1200': u'bodies_of_water_basin_category_(P1200)',
    u'P1201': u'space_tug_(P1201)',
    u'P1202': u'carries_scientific_instrument_(P1202)',
    u'P1203': u'Finnish_municipality_number_(P1203)',
    u'P1204': u'Wikimedia_portal\'s_main_topic_(P1204)',
    u'P1207': u'NUKAT_(WarsawU)_authorities_(P1207)',
    u'P1208': u'ISMN_(P1208)',
    u'P1209': u'CN_(P1209)',
    u'P1210': u'supercharger_(P1210)',
    u'P1211': u'fuel_system_(P1211)',
    u'P1212': u'Atlas_ID_(P1212)',
    u'P1213': u'NLC_authorities_(P1213)',
    u'P1214': u'Riksdagen_person-id_(P1214)',
    u'P1215': u'apparent_magnitude_(P1215)',
    u'P1216': u'National_Heritage_List_for_England_number_(P1216)',
    u'P1217': u'Internet_Broadway_Database_venue_ID_(P1217)',
    u'P1218': u'Internet_Broadway_Database_production_ID_(P1218)',
    u'P1219': u'Internet_Broadway_Database_show_ID_(P1219)',
    u'P1220': u'Internet_Broadway_Database_person_ID_(P1220)',
    u'P1221': u'compressor_type_(P1221)',
    u'P1222': u'(deprecated)_NARA_person_ID_(P1222)',
    u'P1225': u'US_National_Archives_Identifier_(P1225)',
    u'P1227': u'astronomical_filter_(P1227)',
    u'P1229': u'Openpolis_ID_(P1229)',
    u'P1230': u'JSTOR_journal_code_(P1230)',
    u'P1232': u'Linguist_list_code_(P1232)',
    u'P1233': u'ISFDB_author_ID_(P1233)',
    u'P1234': u'ISFDB_publication_ID_(P1234)',
    u'P1235': u'ISFDB_series_ID_(P1235)',
    u'P1236': u'Parsons_code_(P1236)',
    u'P1237': u'Box_Office_Mojo_film_ID_(P1237)',
    u'P1238': u'Swedish_Football_Association_ID_(P1238)',
    u'P1239': u'ISFDB_publisher_ID_(P1239)',
    u'P1240': u'Danish_Bibliometric_Research_Indicator_level_(P1240)',
    u'P1241': u'Swiss_Football_Association_Club_Number_(P1241)',
    u'P1242': u'Theatricalia_play_ID_(P1242)',
    u'P1243': u'International_Standard_Recording_Code_(P1243)',
    u'P1245': u'OmegaWiki_Defined_Meaning_(P1245)',
    u'P1246': u'patent_number_(P1246)',
    u'P1247': u'compression_ratio_(P1247)',
    u'P1248': u'KulturNav-id_(P1248)',
    u'P1249': u'ime_of_earliest_written_record_(P1249)',
    u'P1250': u'Danish_Bibliometric_Research_Indicator_(BFI)_SNO/CNO_(P1250)',
    u'P1251': u'ABS_ASCL_code_(P1251)',
    u'P1252': u'AUSTLANG_code_(P1252)',
    u'P1253': u'BCU_Ecrivainsvd_(P1253)',
    u'P1254': u'Slovenska_biografija_ID_(P1254)',
    u'P1255': u'Helveticarchives_ID_(P1255)',
    u'P1256': u'Iconclass_notation_(P1256)',
    u'P1257': u'depicts_Iconclass_notation_(P1257)',
    u'P1258': u'Rotten_Tomatoes_ID_(P1258)',
    u'P1259': u'coordinates_of_the_point_of_view_(P1259)',
    u'P1260': u'Cultural_heritage_database_in_Sweden_ID_(P1260)',
    u'P1261': u'Rundata_(P1261)',
    u'P1262': u'RAÄ_number_(P1262)',
    u'P1263': u'NNDB_people_ID_(P1263)',
    u'P1264': u'valid_in_period_(P1264)',
    u'P1265': u'AlloCiné_film_ID_(P1265)',
    u'P1266': u'AlloCiné_person_ID_(P1266)',
    u'P1267': u'AlloCiné_series_ID_(P1267)',
    u'P1268': u'represents_(P1268)',
    u'P1269': u'facet_of_(P1269)',
    u'P1270': u'Norwegian_Register_journal_ID_(P1270)',
    u'P1271': u'Norway_Database_for_Statistics_on_Higher_education_publisher_ID_(P1271)',
    u'P1272': u'Norway_Import_Service_and_Registration_Authority_periodical_code_(P1272)',
    u'P1273': u'CANTIC-ID_(P1273)',
    u'P1274': u'ISFDB_title_ID_(P1274)',
    u'P1275': u'Norway_Import_Service_and_Registration_Authority_publisher_code_(P1275)',
    u'P1276': u'Dictionnaire_du_Jura_ID_(P1276)',
    u'P1277': u'Jufo_ID_(P1277)',
    u'P1278': u'Legal_Entity_ID_(P1278)',
    u'P1279': u'inflation_rate_(P1279)',
    u'P1280': u'CONOR_ID_(P1280)',
    u'P1281': u'WOEID_(P1281)',
    u'P1282': u'OpenStreetMap_tag_or_key_(P1282)',
    u'P1283': u'filmography_(P1283)',
    u'P1284': u'Munzinger_IBA_(P1284)',
    u'P1285': u'Munzinger_Sport_number_(P1285)',
    u'P1286': u'Munzinger_Pop_ID_(P1286)',
    u'P1287': u'KDG_Komponisten_der_Gegenwart_(P1287)',
    u'P1288': u'KLG_Kritisches_Lexikon_der_Gegenwartsliteratur_(P1288)',
    u'P1289': u'KLfG_Critical_Dictionary_of_foreign_contemporary_literature_(P1289)',
    u'P1290': u'godparent_(P1290)',
    u'P1291': u'Association_Authors_of_Switzerland_ID_(P1291)',
    u'P1292': u'DNB_editions_(P1292)',
    u'P1293': u'Royal_Aero_Club_Aviator\'s_Certificate_ID_(P1293)',
    u'P1294': u'WWF_ecoregion_code_(P1294)',
    u'P1295': u'emissivity_(P1295)',
    u'P1296': u'Gran_Enciclopèdia_Catalana_ID_(P1296)',
    u'P1297': u'IRS_Employer_Identification_Number_(P1297)',
    u'P1299': u'depicted_by_(P1299)',
    u'P1300': u'bibcode_(P1300)',
    u'P1301': u'number_of_elevators_(P1301)',
    u'P1302': u'primary_destinations_(P1302)',
    u'P1303': u'instrument_(P1303)',
    u'P1304': u'central_bank_(P1304)',
    u'P1305': u'Skyscraper_Center_ID_(P1305)',
    u'P1307': u'Swiss_parliament_ID_(P1307)',
    u'P1308': u'officeholder_(P1308)',
    u'P1309': u'EGAXA_ID_(P1309)',
    u'P1310': u'statement_disputed_by_(P1310)',
    u'P1311': u'lostbridges.org_ID_(P1311)',
    u'P1312': u'has_facet_polytope_(P1312)',
    u'P1313': u'office_held_by_head_of_government_(P1313)',
    u'P1314': u'number_of_spans_(P1314)',
    u'P1315': u'People_Australia_ID_(P1315)',
    u'P1316': u'SMDB_ID_(P1316)',
    u'P1317': u'floruit_(P1317)',
    u'P1318': u'proved_by_(P1318)',
    u'P1319': u'earliest_date_(P1319)',
    u'P1320': u'OpenCorporates_ID_(P1320)',
    u'P1321': u'place_of_origin_(Switzerland)_(P1321)',
    u'P1322': u'dual_to_(P1322)',
    u'P1323': u'Terminologia_Anatomica_98_ID_(P1323)',
    u'P1324': u'source_code_repository_(P1324)',
    u'P1325': u'external_data_available_at_(P1325)',
    u'P1326': u'latest_date_(P1326)',
    u'P1327': u'professional_or_sports_partner_(P1327)',
    u'P1329': u'phone_number_(P1329)',
    u'P1330': u'MusicBrainz_instrument_ID_(P1330)',
    u'P1331': u'PACE_member_ID_(P1331)',
    u'P1332': u'coordinate_of_northernmost_point_(P1332)',
    u'P1333': u'coordinate_of_southernmost_point_(P1333)',
    u'P1334': u'coordinate_of_easternmost_point_(P1334)',
    u'P1335': u'coordinate_of_westernmost_point_(P1335)',
    u'P1336': u'erritory_claimed_by_(P1336)',
    u'P1338': u'EPSG_ID_(P1338)',
    u'P1339': u'number_of_injured_(P1339)',
    u'P1340': u'eye_color_(P1340)',
    u'P1341': u'Italian_Chamber_of_Deputies_dati_ID_(P1341)',
    u'P1342': u'number_of_seats_(P1342)',
    u'P1343': u'described_by_source_(P1343)',
    u'P1344': u'participant_of_(P1344)',
    u'P1345': u'number_of_victims_made_by_killer_(P1345)',
    u'P1346': u'winner_(P1346)',
    u'P1347': u'military_casualty_classification_(P1347)',
    u'P1348': u'AlgaeBase_URL_(P1348)',
    u'P1349': u'ploidy_(P1349)',
    u'P1350': u'number_of_matches_played_(P1350)',
    u'P1351': u'number_of_points/goals/set_scored_(P1351)',
    u'P1352': u'ranking_(P1352)',
    u'P1353': u'original_spelling_(P1353)',
    u'P1354': u'shown_with_features_(P1354)',
    u'P1355': u'number_of_wins_(P1355)',
    u'P1356': u'number_of_losses_(P1356)',
    u'P1357': u'matches/games_drawn/tied_(P1357)',
    u'P1358': u'points_for_(P1358)',
    u'P1359': u'number_of_points/goals_conceded_(P1359)',
    u'P1360': u'Monte_Carlo_Particle_Number_(P1360)',
    u'P1362': u'Theaterlexikon_der_Schweiz_online_ID_(P1362)',
    u'P1363': u'points/goal_scored_by_(P1363)',
    u'P1364': u'ITTF_table_tennis_player_ID_(P1364)',
    u'P1365': u'replaces_(P1365)',
    u'P1366': u'replaced_by_(P1366)',
    u'P1367': u'Art_UK_artist_ID_(P1367)',
    u'P1368': u'LNB_ID_(P1368)',
    u'P1369': u'Iranian_National_Heritage_registration_number_(P1369)',
    u'P1370': u'IHSI_ID_(P1370)',
    u'P1371': u'ASI_Monument_ID_(P1371)',
    u'P1372': u'binding_of_software_library_(P1372)',
    u'P1373': u'daily_patronage_(P1373)',
    u'P1375': u'NSK_ID_(P1375)',
    u'P1376': u'capital_of_(P1376)',
    u'P1377': u'MTR_station_code_(P1377)',
    u'P1378': u'China_railway_TMIS_station_code_(P1378)',
    u'P1380': u'UglyBridges.com_ID_(P1380)',
    u'P1381': u'bridgehunter.com_ID_(P1381)',
    u'P1382': u'partially_coincident_with_(P1382)',
    u'P1383': u'contains_settlement_(P1383)',
    u'P1385': u'Enciclopédia_Açoriana_ID_(P1385)',
    u'P1386': u'Japanese_High_School_Code_(P1386)',
    u'P1387': u'political_alignment_(P1387)',
    u'P1388': u'German_regional_key_(P1388)',
    u'P1389': u'product_certification_(P1389)',
    u'P1390': u'match_time_of_event_(P1390)',
    u'P1391': u'Index_Fungorum_ID_(P1391)',
    u'P1392': u'ComicBookDB_ID_(P1392)',
    u'P1393': u'proxy_(P1393)',
    u'P1394': u'Glottolog_code_(P1394)',
    u'P1395': u'National_Cancer_Institute_ID_(P1395)',
    u'P1396': u'Linguasphere_code_(P1396)',
    u'P1397': u'State_Catalogue_of_Geographical_Names_(Russia)_ID_(P1397)',
    u'P1398': u'structure_replaces_(P1398)',
    u'P1399': u'convicted_of_(P1399)',
    u'P1400': u'FCC_Facility_ID_(P1400)',
    u'P1401': u'bug_tracking_system_(P1401)',
    u'P1402': u'Foundational_Model_of_Anatomy_ID_(P1402)',
    u'P1403': u'original_combination_(P1403)',
    u'P1404': u'World_Glacier_Inventory_ID_(P1404)',
    u'P1406': u'script_directionality_(P1406)',
    u'P1407': u'MusicBrainz_series_ID_(P1407)',
    u'P1408': u'licensed_to_broadcast_to_(P1408)',
    u'P1409': u'Cycling_Archives_cyclist_ID_(P1409)',
    u'P1410': u'number_of_representatives_in_an_organization/legislature_(P1410)',
    u'P1411': u'nominated_for_(P1411)',
    u'P1412': u'languages_spoken,_written_or_signed_(P1412)',
    u'P1414': u'GUI_toolkit_or_framework_(P1414)',
    u'P1415': u'Oxford_Dictionary_of_National_Biography_ID_(P1415)',
    u'P1416': u'affiliation_(P1416)',
    u'P1417': u'Encyclopædia_Britannica_Online_ID_(P1417)',
    u'P1418': u'orbits_completed_(P1418)',
    u'P1419': u'shape_(P1419)',
    u'P1420': u'axon_synonym_(P1420)',
    u'P1421': u'GRIN_URL_(P1421)',
    u'P1422': u'Sandrart.net_person_ID_(P1422)',
    u'P1423': u'emplate\'s_main_topic_(P1423)',
    u'P1424': u'opic\'s_main_template_(P1424)',
    u'P1425': u'ecoregion_(WWF)_(P1425)',
    u'P1427': u'start_point_(P1427)',
    u'P1428': u'Lost_Art-ID_(P1428)',
    u'P1429': u'has_pet_(P1429)',
    u'P1430': u'OpenPlaques_subject_ID_(P1430)',
    u'P1431': u'executive_producer_(P1431)',
    u'P1432': u'B-side_(P1432)',
    u'P1433': u'published_in_(P1433)',
    u'P1434': u'akes_place_in_fictional_universe_(P1434)',
    u'P1435': u'heritage_designation_(P1435)',
    u'P1436': u'collection_or_exhibition_size_(P1436)',
    u'P1437': u'plea_(P1437)',
    u'P1438': u'Jewish_Encyclopedia_ID_(Russian)_(P1438)',
    u'P1439': u'Norsk_filmografi_ID_(P1439)',
    u'P1440': u'FIDE_ID_(P1440)',
    u'P1441': u'present_in_work_(P1441)',
    u'P1442': u'image_of_grave_(P1442)',
    u'P1443': u'score_method_(P1443)',
    u'P1444': u'destination_point_(P1444)',
    u'P1445': u'fictional_universe_described_in_(P1445)',
    u'P1446': u'number_of_missing_(P1446)',
    u'P1447': u'Sports-Reference.com_Olympic_athlete_ID_(P1447)',
    u'P1448': u'official_name_(P1448)',
    u'P1449': u'nickname_(P1449)',
    u'P1450': u'Sandbox_Monolingual_text_(P1450)',
    u'P1451': u'motto_text_(P1451)',
    u'P1453': u'catholic.ru_ID_(P1453)',
    u'P1454': u'legal_form_(P1454)',
    u'P1455': u'list_of_works_(P1455)',
    u'P1456': u'list_of_monuments_(P1456)',
    u'P1457': u'absolute_magnitude_(P1457)',
    u'P1458': u'color_index_(P1458)',
    u'P1459': u'Cadw_Building_ID_(P1459)',
    u'P1460': u'NIEA_building_ID_(P1460)',
    u'P1461': u'Patientplus_ID_(P1461)',
    u'P1462': u'standards_body_(P1462)',
    u'P1463': u'PRDL_Author_ID_(P1463)',
    u'P1464': u'category_for_people_born_here_(P1464)',
    u'P1465': u'category_for_people_who_died_here_(P1465)',
    u'P1466': u'WALS_lect_code_(P1466)',
    u'P1467': u'WALS_genus_code_(P1467)',
    u'P1468': u'WALS_family_code_(P1468)',
    u'P1469': u'FIFA_player_ID_(P1469)',
    u'P1470': u'maximum_glide_ratio_(P1470)',
    u'P1471': u'reporting_mark_(P1471)',
    u'P1472': u'Commons_Creator_page_(P1472)',
    u'P1473': u'Nupill_Literatura_Digital_-_Author_(P1473)',
    u'P1474': u'Nupill_Literatura_Digital_-_Document_(P1474)',
    u'P1476': u'itle_(P1476)',
    u'P1477': u'birth_name_(P1477)',
    u'P1478': u'has_immediate_cause_(P1478)',
    u'P1479': u'has_contributing_factor_(P1479)',
    u'P1480': u'sourcing_circumstances_(P1480)',
    u'P1481': u'vici.org_ID_(P1481)',
    u'P1482': u'Stack_Exchange_tag_(P1482)',
    u'P1483': u'kulturnoe-nasledie.ru_ID_(P1483)',
    u'P1529': u'Gertrude_identifier_(P1529)',
    u'P1531': u'parent(s)_of_this_hybrid_(P1531)',
    u'P1532': u'country_for_sport_(P1532)',
    u'P1533': u'family_name_identical_to_this_given_name_(P1533)',
    u'P1534': u'end_cause_(P1534)',
    u'P1535': u'used_by_(P1535)',
    u'P1536': u'immediate_cause_of_(P1536)',
    u'P1537': u'contributing_factor_of_(P1537)',
    u'P1538': u'number_of_households_(P1538)',
    u'P1539': u'female_population_(P1539)',
    u'P1540': u'male_population_(P1540)',
    u'P1541': u'Cycling_Quotient_male_cyclist_ID_(P1541)',
    u'P1542': u'has_effect_(P1542)',
    u'P1543': u'monogram_(P1543)',
    u'P1544': u'Federal_Register_Document_Number_(P1544)',
    u'P1545': u'series_ordinal_(P1545)',
    u'P1546': u'motto_(P1546)',
    u'P1547': u'depends_on_software_(P1547)',
    u'P1548': u'maximum_Strahler_number_(P1548)',
    u'P1549': u'demonym_(P1549)',
    u'P1550': u'Orphanet_ID_(P1550)',
    u'P1551': u'Exceptional_heritage_of_Wallonia_ID_(P1551)',
    u'P1552': u'has_quality_(P1552)',
    u'P1553': u'Yandex.Music_artist_ID_(P1553)',
    u'P1554': u'UBERON_ID_(P1554)',
    u'P1555': u'Executive_Order_number_(P1555)',
    u'P1556': u'zbMATH_author_ID_(P1556)',
    u'P1557': u'manifestation_of_(P1557)',
    u'P1558': u'empo_marking_(P1558)',
    u'P1559': u'name_in_native_language_(P1559)',
    u'P1560': u'given_name_version_for_other_gender_(P1560)',
    u'P1561': u'number_of_survivors_(P1561)',
    u'P1562': u'AllMovie_movie_ID_(P1562)',
    u'P1563': u'MacTutor_id_(biographies)_(P1563)',
    u'P1564': u'At_the_Circulating_Library_ID_(P1564)',
    u'P1565': u'Enciclopedia_de_la_Literatura_en_México_ID_(P1565)',
    u'P1566': u'GeoNames_ID_(P1566)',
    u'P1567': u'NIS/INS_code_(P1567)',
    u'P1568': u'domain_(P1568)',
    u'P1571': u'codomain_(P1571)',
    u'P1573': u'BBC_Genome_ID_(P1573)',
    u'P1574': u'exemplar_of_(P1574)',
    u'P1575': u'RISS_catalog_(P1575)',
    u'P1576': u'lifestyle_(P1576)',
    u'P1577': u'Gregory-Aland-Number_(P1577)',
    u'P1578': u'Gmelin_number_(P1578)',
    u'P1579': u'Beilstein_Registry_Number_(P1579)',
    u'P1580': u'University_of_Barcelona_authority_ID_(P1580)',
    u'P1581': u'official_blog_(P1581)',
    u'P1582': u'natural_product_of_taxon_(P1582)',
    u'P1583': u'MalaCards_ID_(P1583)',
    u'P1584': u'Pleiades_ID_(P1584)',
    u'P1585': u'Brazilian_municipality_code_(P1585)',
    u'P1586': u'Catalan_object_of_cultural_interest_ID_(P1586)',
    u'P1587': u'Slovene_Cultural_Heritage_Register_ID_(P1587)',
    u'P1588': u'desa_code_of_Indonesia_(P1588)',
    u'P1589': u'deepest_point_(P1589)',
    u'P1590': u'number_of_casualties_(P1590)',
    u'P1591': u'defendant_(P1591)',
    u'P1592': u'prosecutor_(P1592)',
    u'P1593': u'defender_(P1593)',
    u'P1594': u'judge_(P1594)',
    u'P1595': u'charge_(P1595)',
    u'P1596': u'penalty_(P1596)',
    u'P1598': u'consecrator_(P1598)',
    u'P1599': u'Cambridge_Alumni_Database_ID_(P1599)',
    u'P1600': u'code_Inventari_del_Patrimoni_Arquitectònic_de_Catalunya_(P1600)',
    u'P1601': u'Esperantist_ID_(P1601)',
    u'P1602': u'Art_UK_venue_ID_(P1602)',
    u'P1603': u'number_of_cases_(P1603)',
    u'P1604': u'biosafety_level_(P1604)',
    u'P1605': u'has_natural_reservoir_(P1605)',
    u'P1606': u'natural_reservoir_of_(P1606)',
    u'P1607': u'Dialnet_author_ID_(P1607)',
    u'P1608': u'Dialnet_book_ID_(P1608)',
    u'P1609': u'Dialnet_journal_(P1609)',
    u'P1610': u'Dialnet_article_(P1610)',
    u'P1611': u'NATO_code_for_grade_(P1611)',
    u'P1612': u'Commons_Institution_page_(P1612)',
    u'P1613': u'IRC_channel_(P1613)',
    u'P1614': u'History_of_Parliament_ID_(P1614)',
    u'P1615': u'CLARA-ID_(P1615)',
    u'P1616': u'SIREN_number_(P1616)',
    u'P1617': u'BBC_Things_ID_(P1617)',
    u'P1618': u'sport_number_(P1618)',
    u'P1619': u'date_of_official_opening_(P1619)',
    u'P1620': u'plaintiff_(P1620)',
    u'P1621': u'detail_map_(P1621)',
    u'P1622': u'driving_side_(P1622)',
    u'P1624': u'MarineTraffic_Port_ID_(P1624)',
    u'P1625': u'has_melody_(P1625)',
    u'P1626': u'Thai_cultural_heritage_ID_(P1626)',
    u'P1627': u'Ethnologue.com_code_(P1627)',
    u'P1628': u'equivalent_property_(P1628)',
    u'P1629': u'subject_item_of_this_property_(P1629)',
    u'P1630': u'formatter_URL_(P1630)',
    u'P1631': u'China_Vitae_ID_(P1631)',
    u'P1632': u'Hermann-Mauguin_notation_(P1632)',
    u'P1635': u'religious_name_(P1635)',
    u'P1636': u'date_of_baptism_in_early_childhood_(P1636)',
    u'P1637': u'undercarriage_(P1637)',
    u'P1638': u'codename_(P1638)',
    u'P1639': u'pendant_of_(P1639)',
    u'P1640': u'curator_(P1640)',
    u'P1641': u'port_(P1641)',
    u'P1642': u'acquisition_transaction_(P1642)',
    u'P1643': u'departure_transaction_(P1643)',
    u'P1644': u'MessesInfo_ID_(P1644)',
    u'P1645': u'NIST/CODATA_ID_(P1645)',
    u'P1647': u'subproperty_of_(P1647)',
    u'P1648': u'Dictionary_of_Welsh_Biography_ID_(P1648)',
    u'P1649': u'KMDb_person_ID_(P1649)',
    u'P1650': u'BBF_ID_(P1650)',
    u'P1651': u'YouTube_video_ID_(P1651)',
    u'P1652': u'referee_(P1652)',
    u'P1653': u'TERYT_municipality_code_(P1653)',
    u'P1654': u'wing_configuration_(P1654)',
    u'P1656': u'unveiled_by_(P1656)',
    u'P1657': u'MPAA_film_rating_(P1657)',
    u'P1659': u'see_also_(P1659)',
    u'P1660': u'has_index_case_(P1660)',
    u'P1661': u'Alexa_rank_(P1661)',
    u'P1662': u'DOI_prefix_(P1662)',
    u'P1663': u'ProCyclingStats_cyclist_ID_(P1663)',
    u'P1664': u'Cycling_Database_ID_(P1664)',
    u'P1665': u'Chess_Games_ID_(P1665)',
    u'P1666': u'Chess_Club_ID_(P1666)',
    u'P1667': u'TGN_ID_(P1667)',
    u'P1668': u'ATCvet_(P1668)',
    u'P1669': u'CONA_ID_(P1669)',
    u'P1670': u'LAC_ID_(P1670)',
    u'P1671': u'route_number_(P1671)',
    u'P1672': u'his_taxon_is_source_of_(P1672)',
    u'P1673': u'general_formula_(P1673)',
    u'P1674': u'number_confirmed_(P1674)',
    u'P1675': u'number_probable_(P1675)',
    u'P1676': u'number_suspected_(P1676)',
    u'P1677': u'index_case_of_(P1677)',
    u'P1678': u'has_vertex_figure_(P1678)',
    u'P1679': u'Art_UK_artwork_ID_(P1679)',
    u'P1680': u'subtitle_(P1680)',
    u'P1683': u'quote_(P1683)',
    u'P1684': u'inscription_(P1684)',
    u'P1685': u'Pokémon_browser_number_(P1685)',
    u'P1686': u'for_work_(P1686)',
    u'P1687': u'Wikidata_property_(P1687)',
    u'P1688': u'AniDB_ID_(P1688)',
    u'P1689': u'central_government_debt_as_a_percent_of_GDP_(P1689)',
    u'P1690': u'ICD-10-PCS_(P1690)',
    u'P1691': u'operations_and_procedures_key_(OPS)_(P1691)',
    u'P1692': u'ICD-9-CM_(P1692)',
    u'P1693': u'Terminologia_Embryologica_(P1693)',
    u'P1694': u'Terminologia_Histologica_(P1694)',
    u'P1695': u'NLP_ID_(P1695)',
    u'P1696': u'inverse_of_(P1696)',
    u'P1697': u'otal_valid_votes_(P1697)',
    u'P1699': u'SkyscraperPage_building_id_(P1699)',
    u'P1700': u'SIPA_ID_(P1700)',
    u'P1702': u'IGESPAR_ID_(P1702)',
    u'P1703': u'is_pollinated_by_(P1703)',
    u'P1704': u'is_pollinator_of_(P1704)',
    u'P1705': u'native_label_(P1705)',
    u'P1706': u'ogether_with_(P1706)',
    u'P1707': u'DAAO_ID_(P1707)',
    u'P1708': u'LfDS_object_ID_(P1708)',
    u'P1709': u'equivalent_class_(P1709)',
    u'P1710': u'Sächsische_Biografie_(P1710)',
    u'P1711': u'British_Museum_person-institution_(P1711)',
    u'P1712': u'Metacritic_ID_(P1712)',
    u'P1713': u'biography_at_the_Bundestag_of_Germany_(P1713)',
    u'P1714': u'Journalisted_ID_(P1714)',
    u'P1715': u'RKD/ESD_(Slovenia)_ID_(P1715)',
    u'P1716': u'brand_(P1716)',
    u'P1717': u'SANDRE_ID_(P1717)',
    u'P1721': u'pinyin_transliteration_(P1721)',
    u'P1725': u'beats_per_minute_(P1725)',
    u'P1726': u'Florentine_musea_Inventario_1890__ID_(P1726)',
    u'P1727': u'Flora_of_North_America_taxon_ID_(P1727)',
    u'P1728': u'AllMusic_artist_ID_(P1728)',
    u'P1729': u'AllMusic_album_ID_(P1729)',
    u'P1730': u'AllMusic_song_ID_(P1730)',
    u'P1731': u'Fach_(P1731)',
    u'P1732': u'Naturbase_ID_(P1732)',
    u'P1733': u'Steam_Application_ID_(P1733)',
    u'P1734': u'oath_of_office_date_(P1734)',
    u'P1735': u'Comedien.ch_ID_(P1735)',
    u'P1736': u'Information_Center_for_Israeli_Art_artist_ID_(P1736)',
    u'P1738': u'Merck_Index_monograph_(P1738)',
    u'P1739': u'CiNii_book_ID_(P1739)',
    u'P1740': u'category_for_films_shot_at_this_location_(P1740)',
    u'P1741': u'GTAA_ID_(P1741)',
    u'P1743': u'Bradley_and_Fletcher_checklist_number_(P1743)',
    u'P1744': u'Agassiz_checklist_number_(P1744)',
    u'P1745': u'VASCAN_ID_(P1745)',
    u'P1746': u'ZooBank_nomenclatural_act_(P1746)',
    u'P1747': u'Flora_of_China_ID_(P1747)',
    u'P1748': u'NCI_Thesaurus_ID_(P1748)',
    u'P1749': u'Parlement_&amp;_Politiek_ID_(P1749)',
    u'P1750': u'name_day_(P1750)',
    u'P1751': u'Art_UK_collection_ID_(P1751)',
    u'P1752': u'scale_(P1752)',
    u'P1753': u'list_related_to_category_(P1753)',
    u'P1754': u'category_related_to_list_(P1754)',
    u'P1755': u'Aviation_Safety_Network_accident_ID_(P1755)',
    u'P1760': u'Aviation_Safety_Network_Wikibase_Occurrence_(P1760)',
    u'P1761': u'Watson_&amp;_Dallwitz_family_ID_(P1761)',
    u'P1762': u'Hornbostel-Sachs_classification_(P1762)',
    u'P1763': u'National_Pipe_Organ_Register_ID_(P1763)',
    u'P1764': u'FOIH_relict_ID_(P1764)',
    u'P1766': u'place_name_sign_(P1766)',
    u'P1769': u'denkXweb_identifier_(P1769)',
    u'P1770': u'Romania_LMI_code_(P1770)',
    u'P1771': u'Integrated_Postsecondary_Education_Data_System_ID_(P1771)',
    u'P1772': u'USDA_PLANTS_ID_(P1772)',
    u'P1773': u'attributed_to_(P1773)',
    u'P1774': u'workshop_of_(P1774)',
    u'P1775': u'follower_of_(P1775)',
    u'P1776': u'circle_of_(P1776)',
    u'P1777': u'manner_of_(P1777)',
    u'P1778': u'forgery_after_(P1778)',
    u'P1779': u'possible_creator_(P1779)',
    u'P1780': u'school_of_(P1780)',
    u'P1782': u'courtesy_name_(P1782)',
    u'P1785': u'emple_name_(P1785)',
    u'P1786': u'posthumous_name_(P1786)',
    u'P1787': u'art-name_(P1787)',
    u'P1788': u'DVN_ID_(P1788)',
    u'P1789': u'chief_operating_officer_(P1789)',
    u'P1791': u'category_of_people_buried_here_(P1791)',
    u'P1792': u'category_of_associated_people_(P1792)',
    u'P1793': u'format_as_a_regular_expression_(P1793)',
    u'P1794': u'bureau_du_patrimoine_de_Seine-Saint-Denis_ID_(P1794)',
    u'P1795': u'Smithsonian_American_Art_Museum:_person/institution_thesaurus_ID_(P1795)',
    u'P1796': u'International_Standard_Industrial_Classification_code_Rev.4_(P1796)',
    u'P1798': u'ISO_639-5_code_(P1798)',
    u'P1799': u'Maltese_Islands_National_Inventory_of_Cultural_Property_ID_(P1799)',
    u'P1800': u'Wikimedia_database_name_(P1800)',
    u'P1801': u'commemorative_plaque_image_(P1801)',
    u'P1802': u'EMLO_person_ID_(P1802)',
    u'P1803': u'Masaryk_University_person_ID_(P1803)',
    u'P1804': u'DNF_film_ID_(P1804)',
    u'P1806': u'ABoK_number_(P1806)',
    u'P1807': u'Great_Aragonese_Encyclopedia_ID_(P1807)',
    u'P1808': u'senat.fr_ID_(P1808)',
    u'P1809': u'choreographer_(P1809)',
    u'P1810': u'named_as_(P1810)',
    u'P1811': u'list_of_episodes_(P1811)',
    u'P1813': u'short_name_(P1813)',
    u'P1814': u'name_in_kana_(P1814)',
    u'P1815': u'RSL_scanned_book\'s_identifier_(P1815)',
    u'P1816': u'National_Portrait_Gallery_(London)_person_ID_(P1816)',
    u'P1817': u'addressee_(P1817)',
    u'P1818': u'Kaiserhof_ID_(P1818)',
    u'P1819': u'genealogics.org_person_ID_(P1819)',
    u'P1820': u'Open_Food_Facts_food_additive_id_(P1820)',
    u'P1821': u'Open_Food_Facts_food_category_ID_(P1821)',
    u'P1822': u'DSH_object_ID_(P1822)',
    u'P1823': u'BAnQ_work_ID_(P1823)',
    u'P1824': u'road_number_(P1824)',
    u'P1825': u'Baseball-Reference.com_major_league_player_ID_(P1825)',
    u'P1826': u'Baseball-Reference.com_minor_league_player_ID_(P1826)',
    u'P1827': u'ISWC_(P1827)',
    u'P1828': u'IPI_name_number_(P1828)',
    u'P1829': u'Roud_Folk_Song_Index_number_(P1829)',
    u'P1830': u'owner_of_(P1830)',
    u'P1831': u'electorate_(P1831)',
    u'P1832': u'GrassBase_ID_(P1832)',
    u'P1833': u'number_of_registered_users/contributors_(P1833)',
    u'P1836': u'draft_pick_number_(P1836)',
    u'P1837': u'Gaoloumi_ID_(P1837)',
    u'P1838': u'PSS-archi_ID_(P1838)',
    u'P1839': u'US_Federal_Election_Commission_ID_(P1839)',
    u'P1840': u'investigated_by_(P1840)',
    u'P1841': u'Swedish_district_code_(P1841)',
    u'P1842': u'Global_Anabaptist_Mennonite_Encyclopedia_Online_ID_(P1842)',
    u'P1843': u'axon_common_name_(P1843)',
    u'P1844': u'HathiTrust_ID_(P1844)',
    u'P1845': u'anti-virus_alias_(P1845)',
    u'P1846': u'distribution_map_(P1846)',
    u'P1847': u'Nasjonalbiblioteket_photographer_ID_(P1847)',
    u'P1848': u'protected_areas_INPN_Code_(P1848)',
    u'P1849': u'SSR_WrittenForm_ID_(P1849)',
    u'P1850': u'SSR_place_name_number_(P1850)',
    u'P1851': u'input_set_(P1851)',
    u'P1852': u'Perry_Index_(P1852)',
    u'P1853': u'blood_type_(P1853)',
    u'P1854': u'Kiev_street_code_(P1854)',
    u'P1855': u'Wikidata_property_example_(P1855)',
    u'P1866': u'Catholic_Hierarchy_diocese_ID_(P1866)',
    u'P1867': u'eligible_voters_(P1867)',
    u'P1868': u'ballots_cast_(P1868)',
    u'P1869': u'Hall_of_Valor_ID_(P1869)',
    u'P1870': u'Name_Assigning_Authority_Number_(P1870)',
    u'P1871': u'CERL_ID_(P1871)',
    u'P1872': u'minimum_number_of_players_(P1872)',
    u'P1873': u'maximum_number_of_players_(P1873)',
    u'P1874': u'Netflix_ID_(P1874)',
    u'P1875': u'represented_by_(P1875)',
    u'P1876': u'vessel_(P1876)',
    u'P1877': u'after_a_work_by_(P1877)',
    u'P1878': u'Vox-ATypI_classification_(P1878)',
    u'P1879': u'income_classification_(Philippines)_(P1879)',
    u'P1880': u'measured_by_(P1880)',
    u'P1881': u'list_of_characters_(P1881)',
    u'P1882': u'Web_Gallery_of_Art_ID_(P1882)',
    u'P1883': u'Declarator.org_ID_(P1883)',
    u'P1884': u'hair_color_(P1884)',
    u'P1885': u'cathedral_(P1885)',
    u'P1886': u'Smithsonian_volcano_ID_(P1886)',
    u'P1887': u'vice-county_(P1887)',
    u'P1888': u'Dictionary_of_Medieval_Names_from_European_Sources_entry_(P1888)',
    u'P1889': u'different_from_(P1889)',
    u'P1890': u'BNC_ID_(P1890)',
    u'P1891': u'signatory_(P1891)',
    u'P1893': u'OpenPlaques_plaque_ID_(P1893)',
    u'P1894': u'Danish_urban_area_code_(P1894)',
    u'P1895': u'Fauna_Europaea_ID_(P1895)',
    u'P1896': u'source_website_for_the_property_(P1896)',
    u'P1897': u'highest_note_(P1897)',
    u'P1898': u'lowest_note_(P1898)',
    u'P1899': u'LibriVox_author_ID_(P1899)',
    u'P1900': u'EAGLE_id_(P1900)',
    u'P1901': u'BALaT_person/organisation_id_(P1901)',
    u'P1902': u'Spotify_artist_ID_(P1902)',
    u'P1903': u'volcanic_explosivity_index_(P1903)',
    u'P1906': u'office_held_by_head_of_state_(P1906)',
    u'P1907': u'Australian_Dictionary_of_Biography_ID_(P1907)',
    u'P1908': u'CWGC_person_ID_(P1908)',
    u'P1909': u'side_effect_(P1909)',
    u'P1910': u'decreased_expression_in_(P1910)',
    u'P1911': u'increased_expression_in_(P1911)',
    u'P1912': u'deletion_association_with_(P1912)',
    u'P1913': u'gene_duplication_association_with_(P1913)',
    u'P1914': u'gene_insertion_association_with_(P1914)',
    u'P1915': u'gene_inversion_association_with_(P1915)',
    u'P1916': u'gene_substitution_association_with_(P1916)',
    u'P1917': u'posttranslational_modification_association_with_(P1917)',
    u'P1918': u'altered_regulation_leads_to_(P1918)',
    u'P1919': u'Ministry_of_Education_of_Chile_school_ID_(P1919)',
    u'P1920': u'CWGC_burial_ground_ID_(P1920)',
    u'P1921': u'formatter_URI_of_the_RDF_resources_(P1921)',
    u'P1922': u'first_line_(P1922)',
    u'P1923': u'participating_teams_(P1923)',
    u'P1924': u'vaccine_for_(P1924)',
    u'P1925': u'VIOLIN_ID_(P1925)',
    u'P1928': u'Vaccine_Ontology_ID_(P1928)',
    u'P1929': u'ClinVar_Variation_ID_(P1929)',
    u'P1930': u'DSM-5_(P1930)',
    u'P1931': u'NIOSH_Pocket_Guide_ID_(P1931)',
    u'P1932': u'stated_as_(P1932)',
    u'P1933': u'MobyGames_ID_(P1933)',
    u'P1934': u'Animator.ru_film_ID_(P1934)',
    u'P1935': u'Database_of_Classical_Scholars_ID_(P1935)',
    u'P1936': u'Digital_Atlas_of_the_Roman_Empire_ID_(P1936)',
    u'P1937': u'UN/LOCODE_(P1937)',
    u'P1938': u'Project_Gutenberg_author_ID_(P1938)',
    u'P1939': u'Dyntaxa_ID_(P1939)',
    u'P1940': u'conifers.org_ID_(P1940)',
    u'P1942': u'McCune-Reischauer_romanization_(P1942)',
    u'P1943': u'location_map_(P1943)',
    u'P1944': u'relief_location_map_(P1944)',
    u'P1945': u'street_key_(P1945)',
    u'P1946': u'National_Library_of_Ireland_authority_(P1946)',
    u'P1947': u'Mapillary_ID_(P1947)',
    u'P1948': u'BerlPap_identifier_(P1948)',
    u'P1949': u'CulturaItalia_ID_(P1949)',
    u'P1950': u'second_family_name_in_Spanish_name_(P1950)',
    u'P1951': u'investor_(P1951)',
    u'P1952': u'Encyclopaedia_Metallum_band_ID_(P1952)',
    u'P1953': u'Discogs_artist_ID_(P1953)',
    u'P1954': u'Discogs_master_ID_(P1954)',
    u'P1955': u'Discogs_label_ID_(P1955)',
    u'P1956': u'akeoff_and_landing_capability_(P1956)',
    u'P1957': u'Wikisource_index_page_(P1957)',
    u'P1958': u'Trismegistos_Geo_ID_(P1958)',
    u'P1959': u'Dutch_Senate_person_ID_(P1959)',
    u'P1960': u'Google_Scholar_author_ID_(P1960)',
    u'P1961': u'CTHS_society_ID_(P1961)',
    u'P1962': u'patron_(P1962)',
    u'P1963': u'properties_for_this_type_(P1963)',
    u'P1966': u'Biblioteca_Nacional_de_Chile_catalogue_number_(P1966)',
    u'P1967': u'BoxRec_ID_(P1967)',
    u'P1968': u'Foursquare_venue_ID_(P1968)',
    u'P1969': u'MovieMeter_director_ID_(P1969)',
    u'P1970': u'MovieMeter_Movie_ID_(P1970)',
    u'P1971': u'number_of_children_(P1971)',
    u'P1972': u'Open_Hub_ID_(P1972)',
    u'P1973': u'RSL_editions_(P1973)',
    u'P1976': u'INEGI_locality_ID_(P1976)',
    u'P1977': u'Les_Archives_du_Spectacle_ID_(P1977)',
    u'P1978': u'USDA_NDB_number_(P1978)',
    u'P1979': u'Righteous_Among_The_Nations_ID_(P1979)',
    u'P1980': u'PolSys_ID_(P1980)',
    u'P1981': u'FSK_film_rating_(P1981)',
    u'P1982': u'Anime_News_Network_person_ID_(P1982)',
    u'P1983': u'Anime_News_Network_company_ID_(P1983)',
    u'P1984': u'Anime_News_Network_manga_ID_(P1984)',
    u'P1985': u'Anime_News_Network_anime_ID_(P1985)',
    u'P1986': u'Dizionario_Biografico_degli_Italiani_(P1986)',
    u'P1987': u'MCN_code_(P1987)',
    u'P1988': u'Delarge_ID_(P1988)',
    u'P1989': u'Encyclopaedia_Metallum_artist_ID_(P1989)',
    u'P1990': u'species_kept_(P1990)',
    u'P1991': u'LPSN_URL_(P1991)',
    u'P1992': u'Plazi_ID_(P1992)',
    u'P1993': u'TeX_string_(P1993)',
    u'P1994': u'AllMusic_composition_ID_(P1994)',
    u'P1995': u'health_specialty_(P1995)',
    u'P1996': u'parliament.uk_ID_(P1996)',
    u'P1997': u'Facebook_Places_ID_(P1997)',
    u'P1998': u'UCI_code_(P1998)',
    u'P1999': u'UNESCO_language_status_(P1999)',
    u'P2000': u'CPDL_ID_(P2000)',
    u'P2001': u'Revised_Romanization_(P2001)',
    u'P2002': u'Twitter_username_(P2002)',
    u'P2003': u'Instagram_username_(P2003)',
    u'P2004': u'NALT_ID_(P2004)',
    u'P2005': u'Catalogus_Professorum_Halensis_(P2005)',
    u'P2006': u'ZooBank_author_ID_(P2006)',
    u'P2007': u'ZooBank_publication_ID_(P2007)',
    u'P2008': u'IPNI_publication_ID_(P2008)',
    u'P2009': u'Exif_model_(P2009)',
    u'P2010': u'Exif_make_(P2010)',
    u'P2011': u'Cooper-Hewitt_Person_ID_(P2011)',
    u'P2012': u'cuisine_(P2012)',
    u'P2013': u'Facebook_profile_ID_(P2013)',
    u'P2014': u'MoMA_artwork_id_(P2014)',
    u'P2015': u'Hansard_(1803–2005)_ID_(P2015)',
    u'P2016': u'Catalogus_Professorum_Academiae_Groninganae_id_(P2016)',
    u'P2017': u'isomeric_SMILES_(P2017)',
    u'P2018': u'Teuchos_ID_(P2018)',
    u'P2019': u'AllMovie_artist_ID_(P2019)',
    u'P2020': u'WorldFootball.net_ID_(P2020)',
    u'P2021': u'Erdős_number_(P2021)',
    u'P2024': u'German_cattle_breed_ID_(P2024)',
    u'P2025': u'Find_A_Grave_cemetery_ID_(P2025)',
    u'P2026': u'Avibase_ID_(P2026)',
    u'P2027': u'Colour_Index_International_constitution_ID_(P2027)',
    u'P2028': u'United_States_Armed_Forces_service_number_(P2028)',
    u'P2029': u'Dictionary_of_Ulster_Biography_ID_(P2029)',
    u'P2030': u'NASA_biographical_ID_(P2030)',
    u'P2031': u'work_period_(start)_(P2031)',
    u'P2032': u'work_period_(end)_(P2032)',
    u'P2033': u'Category_for_pictures_taken_with_camera_(P2033)',
    u'P2034': u'Project_Gutenberg_ebook_ID_(P2034)',
    u'P2035': u'LinkedIn_personal_profile_URL_(P2035)',
    u'P2036': u'African_Plant_Database_ID_(P2036)',
    u'P2037': u'GitHub_username_(P2037)',
    u'P2038': u'ResearchGate_person_ID_(P2038)',
    u'P2040': u'CITES_Species+_ID_(P2040)',
    u'P2041': u'National_Gallery_of_Victoria_artist_ID_(P2041)',
    u'P2042': u'Artsy_artist_ID_(P2042)',
    u'P2043': u'length_(P2043)',
    u'P2044': u'elevation_above_sea_level_(P2044)',
    u'P2045': u'orbital_inclination_(P2045)',
    u'P2046': u'area_(P2046)',
    u'P2047': u'duration_(P2047)',
    u'P2048': u'height_(P2048)',
    u'P2049': u'width_(P2049)',
    u'P2050': u'wingspan_(P2050)',
    u'P2051': u'M_sin_i_(P2051)',
    u'P2052': u'speed_(P2052)',
    u'P2053': u'watershed_area_(P2053)',
    u'P2054': u'density_(P2054)',
    u'P2055': u'electrical_conductivity_(P2055)',
    u'P2056': u'heat_capacity_(P2056)',
    u'P2057': u'HMDB_ID_(P2057)',
    u'P2058': u'depositor_(P2058)',
    u'P2060': u'luminosity_(P2060)',
    u'P2061': u'aspect_ratio_(P2061)',
    u'P2062': u'HSDB_ID_(P2062)',
    u'P2063': u'LIPID_MAPS_ID_(P2063)',
    u'P2064': u'KNApSAcK_ID_(P2064)',
    u'P2065': u'NIAID_ChemDB_ID_(P2065)',
    u'P2066': u'fusion_enthalpy_(P2066)',
    u'P2067': u'mass_(P2067)',
    u'P2068': u'hermal_conductivity_(P2068)',
    u'P2069': u'magnetic_moment_(P2069)',
    u'P2070': u'Fellow_of_the_Royal_Society_ID_(P2070)',
    u'P2071': u'Mémoire_des_hommes_ID_(P2071)',
    u'P2072': u'CDB_Chemical_ID_(P2072)',
    u'P2073': u'vehicle_range_(P2073)',
    u'P2074': u'internetmedicin.se_ID_(P2074)',
    u'P2075': u'speed_of_sound_(P2075)',
    u'P2076': u'emperature_(P2076)',
    u'P2077': u'pressure_(P2077)',
    u'P2078': u'user_manual_link_(P2078)',
    u'P2079': u'fabrication_method_(P2079)',
    u'P2080': u'AcademiaNet_ID_(P2080)',
    u'P2081': u'BLDAM_object_ID_(P2081)',
    u'P2082': u'M.49_code_(P2082)',
    u'P2083': u'Leadscope_ID_(P2083)',
    u'P2084': u'ZINC_ID_(P2084)',
    u'P2085': u'Nikkaji_(P2085)',
    u'P2086': u'CDD_Public_ID_(P2086)',
    u'P2087': u'Crunchbase_person_ID_(P2087)',
    u'P2088': u'Crunchbase_organisation_ID_(P2088)',
    u'P2089': u'Library_of_Congress_JukeBox_ID_(P2089)',
    u'P2090': u'Power_of_10_athlete_ID_(P2090)',
    u'P2091': u'FISA_rower_ID_(P2091)',
    u'P2092': u'Bildindex_der_Kunst_und_Architektur_ID_(P2092)',
    u'P2093': u'author_name_string_(P2093)',
    u'P2094': u'competition_class_(P2094)',
    u'P2095': u'co-driver_(P2095)',
    u'P2096': u'media_legend_(P2096)',
    u'P2097': u'erm_length_of_office_(P2097)',
    u'P2098': u'substitute/deputy/replacement_of_office/officeholder_(P2098)',
    u'P2099': u'BC_Geographical_Names_ID_(P2099)',
    u'P2100': u'Banque_de_noms_de_lieux_du_Québec_ID_(P2100)',
    u'P2101': u'melting_point_(P2101)',
    u'P2102': u'boiling_point_(P2102)',
    u'P2103': u'size_of_team_at_start_(P2103)',
    u'P2105': u'size_of_team_at_finish_(P2105)',
    u'P2106': u'RXNO_Ontology_(P2106)',
    u'P2107': u'decomposition_point_(P2107)',
    u'P2108': u'Kunstindeks_Danmark_artwork_ID_(P2108)',
    u'P2109': u'installed_capacity_(P2109)',
    u'P2112': u'wing_area_(P2112)',
    u'P2113': u'sublimation_temperature_(P2113)',
    u'P2114': u'half-life_(P2114)',
    u'P2115': u'NDF-RT_ID_(P2115)',
    u'P2116': u'enthalpy_of_vaporization_(P2116)',
    u'P2117': u'combustion_enthalpy_(P2117)',
    u'P2118': u'kinematic_viscosity_(P2118)',
    u'P2119': u'vapor_pressure_(P2119)',
    u'P2120': u'radius_(P2120)',
    u'P2121': u'prize_money_(P2121)',
    u'P2123': u'YerelNet_village_ID_(P2123)',
    u'P2124': u'member_count_(P2124)',
    u'P2125': u'Revised_Hepburn_romanization_(P2125)',
    u'P2126': u'Georgian_national_system_of_romanization_(P2126)',
    u'P2127': u'International_Nuclear_Event_Scale_(P2127)',
    u'P2128': u'flash_point_(P2128)',
    u'P2129': u'IDLH_(P2129)',
    u'P2130': u'cost_(P2130)',
    u'P2131': u'nominal_GDP_(P2131)',
    u'P2132': u'nominal_GDP_per_capita_(P2132)',
    u'P2133': u'otal_debt_(P2133)',
    u'P2134': u'otal_reserves_(P2134)',
    u'P2135': u'otal_exports_(P2135)',
    u'P2136': u'otal_imports_(P2136)',
    u'P2137': u'otal_equity_(P2137)',
    u'P2138': u'otal_liabilities_(P2138)',
    u'P2139': u'otal_revenue_(P2139)',
    u'P2140': u'foreign_direct_investment_net_outflow_(P2140)',
    u'P2141': u'foreign_direct_investment_net_inflow_(P2141)',
    u'P2142': u'box_office_(P2142)',
    u'P2143': u'genome_size_(P2143)',
    u'P2144': u'frequency_(P2144)',
    u'P2145': u'explosive_energy_equivalent_(P2145)',
    u'P2146': u'orbital_period_(P2146)',
    u'P2147': u'rotation_period_(P2147)',
    u'P2148': u'distance_from_river_mouth_(P2148)',
    u'P2149': u'clock_speed_(P2149)',
    u'P2150': u'FSB_speed_(P2150)',
    u'P2151': u'focal_length_(P2151)',
    u'P2152': u'antiparticle_(P2152)',
    u'P2153': u'PubChem_Substance_ID_(SID)_(P2153)',
    u'P2154': u'binding_energy_(P2154)',
    u'P2155': u'solid_solution_series_with_(P2155)',
    u'P2156': u'pseudo_crystal_habit_(P2156)',
    u'P2157': u'lithography_(P2157)',
    u'P2158': u'Cell_line_ontology_ID_(P2158)',
    u'P2159': u'solves_(P2159)',
    u'P2160': u'mass_excess_(P2160)',
    u'P2161': u'Guthrie_code_(P2161)',
    u'P2162': u'Deutsche_Ultramarathon-Vereinigung_ID_(P2162)',
    u'P2163': u'FAST_ID_(P2163)',
    u'P2164': u'SIGIC_author_ID_(P2164)',
    u'P2165': u'SIGIC_group_ID_(P2165)',
    u'P2166': u'SIGIC_institution_ID_(P2166)',
    u'P2167': u'UNSPSC_Code_(P2167)',
    u'P2168': u'SFDb_person_ID_(P2168)',
    u'P2169': u'PublicWhip_ID_(P2169)',
    u'P2170': u'Hansard_(2006–March_2016)_ID_(P2170)',
    u'P2171': u'They_Work_for_You_ID_(P2171)',
    u'P2172': u'Parliamentary_record_identifier_(P2172)',
    u'P2173': u'BBC_News_Democracy_Live_ID_(P2173)',
    u'P2174': u'MoMA_artist_id_(P2174)',
    u'P2175': u'medical_condition_treated_(P2175)',
    u'P2176': u'drug_used_for_treatment_(P2176)',
    u'P2177': u'solubility_(P2177)',
    u'P2178': u'solvent_(P2178)',
    u'P2179': u'ACM_Classification_Code_(2012)_(P2179)',
    u'P2180': u'Kansallisbiografia_ID_(P2180)',
    u'P2181': u'Finnish_MP_ID_(P2181)',
    u'P2182': u'Finnish_Ministers_database_ID_(P2182)',
    u'P2183': u'ISO_9:1995_(P2183)',
    u'P2184': u'history_of_topic_(P2184)',
    u'P2185': u'DLI_ID_(P2185)',
    u'P2186': u'Wiki_Loves_Monuments_ID_(P2186)',
    u'P2187': u'BiblioNet_publication_ID_(P2187)',
    u'P2188': u'BiblioNet_author_ID_(P2188)',
    u'P2189': u'BiblioNet_publisher_ID_(P2189)',
    u'P2190': u'C-SPAN_person_ID_(P2190)',
    u'P2191': u'NILF_author_id_(P2191)',
    u'P2192': u'endangeredlanguages.com_ID_(P2192)',
    u'P2193': u'Soccerbase_player_ID_(P2193)',
    u'P2194': u'PSS-Archi_architect_id_(P2194)',
    u'P2195': u'Soccerbase_manager_ID_(P2195)',
    u'P2196': u'students_count_(P2196)',
    u'P2197': u'production_rate_(P2197)',
    u'P2198': u'average_gradient_(P2198)',
    u'P2199': u'autoignition_temperature_(P2199)',
    u'P2200': u'electric_charge_(P2200)',
    u'P2201': u'electric_dipole_moment_(P2201)',
    u'P2202': u'lower_flammable_limit_(P2202)',
    u'P2203': u'upper_flammable_limit_(P2203)',
    u'P2204': u'minimum_explosive_concentration_(P2204)',
    u'P2205': u'Spotify_album_ID_(P2205)',
    u'P2206': u'Discogs_release_ID_(P2206)',
    u'P2207': u'Spotify_track_ID_(P2207)',
    u'P2208': u'average_shot_length_(P2208)',
    u'P2209': u'SourceForge_project_(P2209)',
    u'P2210': u'relative_to_(P2210)',
    u'P2211': u'position_angle_(P2211)',
    u'P2212': u'angular_distance_(P2212)',
    u'P2213': u'longitude_of_ascending_node_(P2213)',
    u'P2214': u'parallax_(P2214)',
    u'P2215': u'proper_motion_(P2215)',
    u'P2216': u'radial_velocity_(P2216)',
    u'P2217': u'cruise_speed_(P2217)',
    u'P2218': u'net_worth_estimate_(P2218)',
    u'P2219': u'real_gross_domestic_product_growth_rate_(P2219)',
    u'P2220': u'household_wealth_(P2220)',
    u'P2221': u'flux_(P2221)',
    u'P2222': u'gyromagnetic_ratio_(P2222)',
    u'P2223': u'decay_width_(P2223)',
    u'P2224': u'spectral_line_(P2224)',
    u'P2225': u'discharge_(P2225)',
    u'P2226': u'market_capitalization_(P2226)',
    u'P2227': u'metallicity_(P2227)',
    u'P2228': u'maximum_thrust_(P2228)',
    u'P2229': u'hermal_design_power_(P2229)',
    u'P2230': u'orque_(P2230)',
    u'P2231': u'explosive_velocity_(P2231)',
    u'P2232': u'cash_(P2232)',
    u'P2233': u'semi-major_axis_(P2233)',
    u'P2234': u'volume_as_quantity_(P2234)',
    u'P2235': u'external_superproperty_(P2235)',
    u'P2236': u'external_subproperty_(P2236)',
    u'P2237': u'units_used_for_this_property_(P2237)',
    u'P2238': u'official_symbol_(P2238)',
    u'P2239': u'first_aid_measures_(P2239)',
    u'P2240': u'median_lethal_dose_(P2240)',
    u'P2241': u'reason_for_deprecation_(P2241)',
    u'P2242': u'Florentine_musea_catalogue_ID_(P2242)',
    u'P2243': u'apoapsis_(P2243)',
    u'P2244': u'periapsis_(P2244)',
    u'P2248': u'argument_of_periapsis_(P2248)',
    u'P2249': u'Refseq_Genome_ID_(P2249)',
    u'P2250': u'life_expectancy_(P2250)',
    u'P2252': u'National_Gallery_of_Art_artist_ID_(P2252)',
    u'P2253': u'DfE_URN_(P2253)',
    u'P2254': u'maximum_operating_altitude_(P2254)',
    u'P2255': u'Debrett\'s_People_of_Today_ID_(P2255)',
    u'P2257': u'frequency_of_event_(P2257)',
    u'P2258': u'mobile_country_code_(P2258)',
    u'P2259': u'mobile_network_code_(P2259)',
    u'P2260': u'ionization_energy_(P2260)',
    u'P2261': u'beam_(P2261)',
    u'P2262': u'draft_(P2262)',
    u'P2263': u'ISOCAT_id_(P2263)',
    u'P2264': u'mix\'n\'match_catalogue_ID_(P2264)',
    u'P2266': u'Fashion_Model_Directory_model_ID_(P2266)',
    u'P2267': u'Politifact_Personality_ID_(P2267)',
    u'P2268': u'Musée_d\'Orsay_artist_ID_(P2268)',
    u'P2270': u'Emporis_building_complex_ID_(P2270)',
    u'P2271': u'Wikidata_property_example_for_properties_(P2271)',
    u'P2272': u'Hederich_encyclopedia_article_ID_(P2272)',
    u'P2273': u'Heidelberg_Academy_for_Sciences_and_Humanities_member_ID_(P2273)',
    u'P2275': u'World_Health_Organisation_International_Nonproprietary_Name_(P2275)',
    u'P2276': u'UEFA_player_ID_(P2276)',
    u'P2277': u'Magdeburger_Biographisches_Lexikon_(P2277)',
    u'P2278': u'Member_of_the_Hellenic_Parliament_ID_(P2278)',
    u'P2279': u'ambitus_(P2279)',
    u'P2280': u'Austrian_Parliament_ID_(P2280)',
    u'P2281': u'iTunes_album_ID_(P2281)',
    u'P2282': u'Groeningemuseum_work_PID_(P2282)',
    u'P2283': u'uses_(P2283)',
    u'P2284': u'price_(P2284)',
    u'P2285': u'periapsis_date_(P2285)',
    u'P2286': u'arterial_supply_(P2286)',
    u'P2287': u'CRIStin_ID_(P2287)',
    u'P2288': u'lymphatic_drainage_(P2288)',
    u'P2289': u'venous_drainage_(P2289)',
    u'P2290': u'Danish_parish_code_(P2290)',
    u'P2291': u'charted_in_(P2291)',
    u'P2292': u'consumption_rate_(P2292)',
    u'P2293': u'genetic_association_(P2293)',
    u'P2294': u'balance_of_trade_(P2294)',
    u'P2295': u'net_profit_(P2295)',
    u'P2296': u'money_supply_(P2296)',
    u'P2297': u'employment_by_economic_sector_(P2297)',
    u'P2298': u'NSDAP_membership_number_(1925–1945)_(P2298)',
    u'P2299': u'PPP_GDP_per_capita_(P2299)',
    u'P2300': u'minimal_lethal_dose_(P2300)',
    u'P2302': u'property_constraint_(P2302)',
    u'P2303': u'exception_to_constraint_(P2303)',
    u'P2304': u'group_by_(P2304)',
    u'P2305': u'item_of_property_constraint_(P2305)',
    u'P2306': u'property_(P2306)',
    u'P2307': u'namespace_(P2307)',
    u'P2308': u'class_(P2308)',
    u'P2309': u'relation_(P2309)',
    u'P2310': u'minimum_date_(property_constraint)_(P2310)',
    u'P2311': u'maximum_date_(property_constraint)_(P2311)',
    u'P2312': u'maximum_quantity_(property_constraint)_(P2312)',
    u'P2313': u'minimum_quantity_(property_constraint)_(P2313)',
    u'P2315': u'comment_(DEPRECATED)_(P2315)',
    u'P2316': u'constraint_status_(P2316)',
    u'P2317': u'call_sign_(P2317)',
    u'P2318': u'debut_participant_(P2318)',
    u'P2319': u'elector_(P2319)',
    u'P2320': u'aftershocks_(P2320)',
    u'P2321': u'general_classification_of_race_participants_(P2321)',
    u'P2322': u'article_ID_(P2322)',
    u'P2323': u'Swedish_Olympic_Committee_athlete_ID_(P2323)',
    u'P2324': u'quantity_buried_(P2324)',
    u'P2325': u'mean_anomaly_(P2325)',
    u'P2326': u'GNS_Unique_Feature_ID_(P2326)',
    u'P2327': u'ProCyclingStats_race_ID_(P2327)',
    u'P2328': u'ProCyclingStats_team_ID_(P2328)',
    u'P2329': u'antagonist_muscle_(P2329)',
    u'P2330': u'Cycling_Archives_race_ID_(P2330)',
    u'P2331': u'Cycling_Archives_team_ID_(P2331)',
    u'P2332': u'Dictionary_of_Art_Historians_ID_(P2332)',
    u'P2333': u'Norwegian_organisation_number_(P2333)',
    u'P2334': u'Swedish_Film_Database_film_ID_(P2334)',
    u'P2335': u'SFDb_company_ID_(P2335)',
    u'P2336': u'SFDb_soundtrack_ID_(P2336)',
    u'P2337': u'SFDb_group_ID_(P2337)',
    u'P2338': u'Musopen_composer_ID_(P2338)',
    u'P2339': u'BoardGameGeek_ID_(P2339)',
    u'P2340': u'CESAR_person_ID_(P2340)',
    u'P2341': u'indigenous_to_(P2341)',
    u'P2342': u'AGORHA_person/institution_ID_(P2342)',
    u'P2343': u'playing_range_image_(P2343)',
    u'P2344': u'AGORHA_work_ID_(P2344)',
    u'P2345': u'AGORHA_event_identifier_(P2345)',
    u'P2346': u'Elonet_movie_ID_(P2346)',
    u'P2347': u'YSO_ID_(P2347)',
    u'P2348': u'ime_period_(P2348)',
    u'P2349': u'Stuttgart_Database_of_Scientific_Illustrators_ID_(P2349)',
    u'P2350': u'SpeedSkatingBase.eu_ID_(P2350)',
    u'P2351': u'number_of_graves_(P2351)',
    u'P2352': u'applies_to_taxon_(P2352)',
    u'P2353': u'statistical_unit_(P2353)',
    u'P2354': u'has_list_(P2354)',
    u'P2355': u'UNESCO_Atlas_of_the_World\'s_Languages_in_Danger_ID_(P2355)',
    u'P2357': u'Classification_of_Instructional_Programs_code_(P2357)',
    u'P2358': u'Roman_praenomen_(P2358)',
    u'P2359': u'Roman_nomen_gentilicium_(P2359)',
    u'P2360': u'intended_public_(P2360)',
    u'P2361': u'online_service_(P2361)',
    u'P2362': u'ime_to_altitude_(P2362)',
    u'P2363': u'NMHH_film_rating_(P2363)',
    u'P2364': u'production_code_(P2364)',
    u'P2365': u'Roman_cognomen_(P2365)',
    u'P2366': u'Roman_agnomen_(P2366)',
    u'P2367': u'Australian_Stratigraphic_Units_Database_ID_(P2367)',
    u'P2368': u'Sandbox-Property_(P2368)',
    u'P2369': u'Soccerway_player_ID_(P2369)',
    u'P2370': u'conversion_to_SI_unit_(P2370)',
    u'P2371': u'FAO_risk_status_(P2371)',
    u'P2372': u'ODIS_ID_(P2372)',
    u'P2373': u'Genius_artist_ID_(P2373)',
    u'P2374': u'natural_abundance_(P2374)',
    u'P2375': u'has_superpartner_(P2375)',
    u'P2376': u'superpartner_of_(P2376)',
    u'P2377': u'MediaWiki_hooks_used_(P2377)',
    u'P2378': u'issued_by_(P2378)',
    u'P2379': u'deprecated_in_version_(P2379)',
    u'P2380': u'French_Sculpture_Census_artist_ID_(P2380)',
    u'P2381': u'Academic_Tree_ID_(P2381)',
    u'P2382': u'Chemins_de_mémoire_ID_(P2382)',
    u'P2383': u'CTHS_person_ID_(P2383)',
    u'P2384': u'statement_describes_(P2384)',
    u'P2385': u'French_diocesan_architects_ID_(P2385)',
    u'P2386': u'diameter_(P2386)',
    u'P2387': u'Elonet_person_ID_(P2387)',
    u'P2388': u'office_held_by_head_of_the_organisation_(P2388)',
    u'P2389': u'organisation_directed_from_the_office_(P2389)',
    u'P2390': u'Ballotpedia_ID_(P2390)',
    u'P2391': u'OKPO_ID_(P2391)',
    u'P2392': u'eaching_method_(P2392)',
    u'P2393': u'NCBI_Locus_tag_(P2393)',
    u'P2394': u'MGI_Gene_Symbol_(P2394)',
    u'P2396': u'image_of_function_(P2396)',
    u'P2397': u'YouTube_channel_ID_(P2397)',
    u'P2398': u'MLS_player_ID_(P2398)',
    u'P2399': u'British_Council_artist_ID_(P2399)',
    u'P2400': u'JMDb_film_ID_(P2400)',
    u'P2401': u'Six_Degrees_of_Francis_Bacon_ID_(P2401)',
    u'P2402': u'otal_expenditure_(P2402)',
    u'P2403': u'otal_assets_(P2403)',
    u'P2404': u'ime-weighted_average_exposure_limit_(P2404)',
    u'P2405': u'ceiling_exposure_limit_(P2405)',
    u'P2406': u'maximum_peak_exposure_limit_(P2406)',
    u'P2407': u'short-term_exposure_limit_(P2407)',
    u'P2408': u'set_in_period_(P2408)',
    u'P2409': u'CiNii_article_ID_(P2409)',
    u'P2410': u'WikiPathways_ID_(P2410)',
    u'P2411': u'Artsy_gene_(P2411)',
    u'P2412': u'Fashion_Model_Directory_designer_ID_(P2412)',
    u'P2413': u'Fashion_Model_Directory_magazine_ID_(P2413)',
    u'P2414': u'substrate_of_(P2414)',
    u'P2415': u'personal_best_(P2415)',
    u'P2416': u'sports_discipline_competed_in_(P2416)',
    u'P2417': u'stage_classification_(P2417)',
    u'P2418': u'Structurae_person_ID_(P2418)',
    u'P2421': u'Prosopographia_Attica_(P2421)',
    u'P2423': u'FIE_fencer_ID_(P2423)',
    u'P2424': u'Berlin_cultural_heritage_ID_(P2424)',
    u'P2425': u'service_ribbon_image_(P2425)',
    u'P2426': u'Xeno-canto_species_ID_(P2426)',
    u'P2427': u'GRID_ID_(P2427)',
    u'P2428': u'RePEc_Short-ID_(P2428)',
    u'P2429': u'expected_completeness_(P2429)',
    u'P2430': u'akeoff_roll_(P2430)',
    u'P2431': u'Thyssen-Bornemisza_artist_ID_(P2431)',
    u'P2432': u'J._Paul_Getty_Museum_artist_id_(P2432)',
    u'P2433': u'gender_of_a_scientific_name_of_a_genus_(P2433)',
    u'P2434': u'Panarctic_Flora_ID_(P2434)',
    u'P2435': u'PORT_person_ID_(P2435)',
    u'P2436': u'voltage_(P2436)',
    u'P2437': u'number_of_seasons_(P2437)',
    u'P2438': u'narrator_(P2438)',
    u'P2439': u'language_(P2439)',
    u'P2440': u'ransliteration_(P2440)',
    u'P2441': u'literal_translation_(P2441)',
    u'P2442': u'conversion_to_standard_unit_(P2442)',
    u'P2443': u'stage_reached_(P2443)',
    u'P2444': u'homoglyph_(P2444)',
    u'P2445': u'metasubclass_of_(P2445)',
    u'P2446': u'Transfermarkt_player_ID_(P2446)',
    u'P2447': u'Transfermarkt_manager_ID_(P2447)',
    u'P2448': u'Turkish_Football_Federation_player_ID_(P2448)',
    u'P2449': u'Turkish_Football_Federation_coach__ID_(P2449)',
    u'P2450': u'Encyclopædia_Britannica_contributor_ID_(P2450)',
    u'P2451': u'MAME_ROM_(P2451)',
    u'P2452': u'GeoNames_feature_code_(P2452)',
    u'P2453': u'nominee_(P2453)',
    u'P2454': u'KNAW_past_member_ID_(P2454)',
    u'P2455': u'Species_Profile_and_Threats_Database_ID_(P2455)',
    u'P2456': u'DBLP_ID_(P2456)',
    u'P2457': u'Australian_National_Shipwreck_ID_(P2457)',
    u'P2458': u'Mackolik.com_player_ID_(P2458)',
    u'P2459': u'IBU_biathlete_ID_(P2459)',
    u'P2460': u'Persons_of_Ancient_Athens_(P2460)',
    u'P2461': u'ComLaw_ID_(P2461)',
    u'P2462': u'member_of_the_deme_(P2462)',
    u'P2463': u'elibrary.ru_organisation_ID_(P2463)',
    u'P2464': u'BugGuide_ID_(P2464)',
    u'P2465': u'Allcinema_film_ID_(P2465)',
    u'P2467': u'Global_Geoparks_Network_ID_(P2467)',
    u'P2468': u'Theatricalia_theatre_ID_(P2468)',
    u'P2469': u'Theatricalia_person_ID_(P2469)',
    u'P2470': u'Talouselämän_vaikuttajat_ID_(P2470)',
    u'P2471': u'Models.com_person_ID_(P2471)',
    u'P2472': u'ACMA_Radiocommunications_Licence_ID_(P2472)',
    u'P2473': u'IGCPV_ID_(P2473)',
    u'P2474': u'CDLI_ID_(P2474)',
    u'P2475': u'NAVA_ID_(P2475)',
    u'P2476': u'HNI_person/institution_ID_(P2476)',
    u'P2477': u'TBRC_Resource_ID_(P2477)',
    u'P2478': u'Railways_Archive_event_ID_(P2478)',
    u'P2479': u'SPDX_ID_(P2479)',
    u'P2480': u'IHO_Hydrographic_Dictionary_(S-32)_Number_(P2480)',
    u'P2481': u'Elite_Prospects_player_ID_(P2481)',
    u'P2482': u'SABR_ID_(P2482)',
    u'P2483': u'NCES_District_ID_(P2483)',
    u'P2484': u'NCES_School_ID_(P2484)',
    u'P2485': u'Fashion_Model_Directory_photographer_ID_(P2485)',
    u'P2486': u'Fashion_Model_Directory_brand_ID_(P2486)',
    u'P2487': u'page_at_website_of_Belarus_Geocenter_(P2487)',
    u'P2488': u'page_at_Belarus_Globe_website_(P2488)',
    u'P2489': u'page_at_hram.by_(P2489)',
    u'P2490': u'page_at_OSTIS_Belarus_Wiki_(P2490)',
    u'P2491': u'Radzima.org_ID_(P2491)',
    u'P2492': u'MTMT_author_ID_(P2492)',
    u'P2493': u'OM_institution_ID_(P2493)',
    u'P2494': u'Latvian_cultural_heritage_register_ID_(P2494)',
    u'P2496': u'Latvian_toponymic_names_database_ID_(P2496)',
    u'P2497': u'Latvian_National_Address_Register_ID_(P2497)',
    u'P2498': u'Catalan_Biographical_Dictionary_of_Women_ID_(P2498)',
    u'P2499': u'league_level_above_(P2499)',
    u'P2500': u'league_level_below_(P2500)',
    u'P2501': u'results_(P2501)',
    u'P2502': u'classification_of_cycling_race_(P2502)',
    u'P2503': u'Genealogical_Gazetteer_(GOV)_ID_(P2503)',
    u'P2504': u'Norwegian_municipality_number_(P2504)',
    u'P2505': u'allows_the_passage_of_(P2505)',
    u'P2506': u'INSEE_canton_code_(P2506)',
    u'P2507': u'corrigendum_/_erratum_(P2507)',
    u'P2508': u'KINENOTE_film_ID_(P2508)',
    u'P2509': u'Movie_Walker_ID_(P2509)',
    u'P2510': u'National_Discography_of_Italian_Song_artist/group_ID_(P2510)',
    u'P2511': u'MSK_Gent_work_PID_(P2511)',
    u'P2512': u'spin-off_(P2512)',
    u'P2513': u'Jamendo_album_ID_(P2513)',
    u'P2514': u'Jamendo_artist_ID_(P2514)',
    u'P2515': u'costume_designer_(P2515)',
    u'P2516': u'Australian_Ramsar_site_ID_(P2516)',
    u'P2517': u'category_for_recipients_of_this_award_(P2517)',
    u'P2518': u'Scope.dk_film_ID_(P2518)',
    u'P2519': u'Scope.dk_person_ID_(P2519)',
    u'P2520': u'UNESCO_Biosphere_Reserve_url_(P2520)',
    u'P2521': u'female_form_of_label_(P2521)',
    u'P2522': u'victory_(P2522)',
    u'P2524': u'SEED_number_(P2524)',
    u'P2525': u'Ramsar_Sites_Information_Service_ID_(P2525)',
    u'P2526': u'National_Historic_Sites_of_Canada_ID_(P2526)',
    u'P2527': u'earthquake_magnitude_on_the_moment_magnitude_scale_(P2527)',
    u'P2528': u'earthquake_magnitude_on_the_Richter_magnitude_scale_(P2528)',
    u'P2529': u'ČSFD_film_ID_(P2529)',
    u'P2530': u'Box_Office_Mojo_franchise_ID_(P2530)',
    u'P2531': u'Box_Office_Mojo_studio_ID_(P2531)',
    u'P2532': u'lowest_atmospheric_pressure_(P2532)',
    u'P2533': u'WomenWriters_ID_(P2533)',
    u'P2534': u'defining_formula_(P2534)',
    u'P2535': u'Sandbox-Mathematical_expression_(P2535)',
    u'P2536': u'Sandbox-External_identifier_(P2536)',
    u'P2537': u'Free_Software_Directory_entry_(P2537)',
    u'P2538': u'Nationalmuseum_Sweden_artist_ID_(P2538)',
    u'P2539': u'Nationalmuseum_Sweden_artwork_ID_(P2539)',
    u'P2540': u'Aarne–Thompson–Uther_Tale_Type_Index_(P2540)',
    u'P2541': u'operating_area_(P2541)',
    u'P2542': u'acceptable_daily_intake_(P2542)',
    u'P2545': u'bowling_style_(P2545)',
    u'P2546': u'sidekick_of_(P2546)',
    u'P2547': u'perimeter_(P2547)',
    u'P2548': u'strand_orientation_(P2548)',
    u'P2549': u'Italian_Senate_of_the_Republic_ID_(P2549)',
    u'P2550': u'recording_or_performance_of_(P2550)',
    u'P2551': u'used_metre_(P2551)',
    u'P2552': u'quantitative_metrical_pattern_(P2552)',
    u'P2553': u'in_work_(P2553)',
    u'P2554': u'production_designer_(P2554)',
    u'P2555': u'fee_(P2555)',
    u'P2556': u'bore_(P2556)',
    u'P2557': u'stroke_(P2557)',
    u'P2558': u'autores.uy_database_id_(P2558)',
    u'P2559': u'Wikidata_usage_instructions_(P2559)',
    u'P2560': u'GPU_(P2560)',
    u'P2561': u'name_(P2561)',
    u'P2562': u'married_name_(P2562)',
    u'P2563': u'superhuman_feature_or_ability_(P2563)',
    u'P2564': u'Köppen_climate_classification_(P2564)',
    u'P2565': u'global-warming_potential_(P2565)',
    u'P2566': u'ECHA_InfoCard_ID_(P2566)',
    u'P2567': u'amended_by_(P2567)',
    u'P2568': u'repealed_by_(P2568)',
    u'P2571': u'uncertainty_corresponds_to_(P2571)',
    u'P2572': u'Twitter_hashtag_(P2572)',
    u'P2573': u'number_of_out_of_school_children_(P2573)',
    u'P2574': u'National-Football-Teams.com_player_ID_(P2574)',
    u'P2575': u'measures_(P2575)',
    u'P2576': u'UCSC_Genome_Browser_assembly_ID_(P2576)',
    u'P2577': u'admissible_rule_in_(P2577)',
    u'P2578': u'studies_(P2578)',
    u'P2579': u'studied_by_(P2579)',
    u'P2580': u'Baltisches_Biographisches_Lexikon_digital_ID_(P2580)',
    u'P2581': u'BabelNet_id_(P2581)',
    u'P2582': u'J._Paul_Getty_Museum_object_ID_(P2582)',
    u'P2583': u'distance_from_Earth_(P2583)',
    u'P2584': u'Australian_Wetlands_Code_(P2584)',
    u'P2585': u'INSEE_region_code_(P2585)',
    u'P2586': u'INSEE_department_code_(P2586)',
    u'P2587': u'has_phoneme_(P2587)',
    u'P2588': u'administrative_code_of_Indonesia_(P2588)',
    u'P2589': u'Statistics_Indonesia_ethnicity_code_(P2589)',
    u'P2590': u'BPS_language_code_(P2590)',
    u'P2591': u'grammatical_option_indicates_(P2591)',
    u'P2592': u'Québec_cultural_heritage_directory_people_identifier_(P2592)',
    u'P2593': u'Latvian_Olympic_Committee_athlete_ID_(P2593)',
    u'P2595': u'maximum_gradient_(P2595)',
    u'P2596': u'culture_(P2596)',
    u'P2597': u'Gram_staining_(P2597)',
    u'P2598': u'serial_number_(P2598)',
    u'P2599': u'block_size_(P2599)',
    u'P2600': u'Geni.com_profile_ID_(P2600)',
    u'P2601': u'Eurohockey.com_player_ID_(P2601)',
    u'P2602': u'HockeyDB.com_player_ID_(P2602)',
    u'P2603': u'Kinopoisk_film_ID_(P2603)',
    u'P2604': u'Kinopoisk_person_ID_(P2604)',
    u'P2605': u'ČSFD_person_ID_(P2605)',
    u'P2606': u'PlayStation_ID_(P2606)',
    u'P2607': u'BookBrainz_creator_ID_(P2607)',
    u'P2610': u'hickness_(P2610)',
    u'P2611': u'TED_speaker_ID_(P2611)',
    u'P2612': u'TED_topic_ID_(P2612)',
    u'P2613': u'TED_talk_ID_(P2613)',
    u'P2614': u'World_Heritage_criteria_(P2614)',
    u'P2618': u'SHOWA_ID_(P2618)',
    u'P2619': u'Hungarian_company_ID_(P2619)',
    u'P2620': u'ISO_15924_numeric_code_(P2620)',
    u'P2621': u'Site_of_Special_Scientific_Interest_(England)_ID_(P2621)',
    u'P2622': u'Companies_House_ID_(P2622)',
    u'P2623': u'MEK_ID_(P2623)',
    u'P2624': u'MetroLyrics_ID_(P2624)',
    u'P2625': u'PASE_ID_(P2625)',
    u'P2626': u'DNF_person_ID_(P2626)',
    u'P2627': u'ISO_9362_SWIFT/BIC_code_(P2627)',
    u'P2628': u'German_tax_authority_ID_(P2628)',
    u'P2629': u'BBFC_rating_(P2629)',
    u'P2630': u'cost_of_damage_(P2630)',
    u'P2631': u'Turner_Classic_Movies_film_ID_(P2631)',
    u'P2632': u'place_of_detention_(P2632)',
    u'P2633': u'geography_of_topic_(P2633)',
    u'P2634': u'model_(P2634)',
    u'P2635': u'number_of_parts_of_a_work_of_art_(P2635)',
    u'P2636': u'Minkultury_film_ID_(P2636)',
    u'P2637': u'RARS_rating_(P2637)',
    u'P2638': u'TV.com_ID_(P2638)',
    u'P2639': u'Filmportal_ID_(P2639)',
    u'P2640': u'Swimrankings.net_swimmer_ID_(P2640)',
    u'P2641': u'Davis_Cup_player_ID_(P2641)',
    u'P2642': u'Fed_Cup_player_ID_(P2642)',
    u'P2643': u'Carnegie_Classification_of_Institutions_of_Higher_Education_(P2643)',
    u'P2645': u'mean_lifetime_(P2645)',
    u'P2646': u'mirTarBase_ID_(P2646)',
    u'P2647': u'source_of_material_(P2647)',
    u'P2648': u'Cycling_Quotient_ID_men\'s_race_ID_(P2648)',
    u'P2649': u'Cycling_Quotient_men\'s_team_URL_(P2649)',
    u'P2650': u'interested_in_(P2650)',
    u'P2651': u'CRICOS_Provider_Code_(P2651)',
    u'P2652': u'partnership_with_(P2652)',
    u'P2655': u'Estyn_ID_(P2655)',
    u'P2657': u'EU_transparency_register_ID_(P2657)',
    u'P2658': u'Scoville_grade_(P2658)',
    u'P2659': u'opographic_isolation_(P2659)',
    u'P2660': u'opographic_prominence_(P2660)',
    u'P2661': u'arget_interest_rate_(P2661)',
    u'P2662': u'consumption_rate_per_capita_(P2662)',
    u'P2663': u'ier_1_capital_ratio_(CETI)_(P2663)',
    u'P2664': u'units_sold_(P2664)',
    u'P2665': u'alcohol_by_volume_(P2665)',
    u'P2666': u'Datahub_page_(P2666)',
    u'P2667': u'corresponding_template_(P2667)',
    u'P2668': u'stability_of_property_value_(P2668)',
    u'P2669': u'discontinued_date_(P2669)',
    u'P2670': u'has_parts_of_the_class_(P2670)',
    u'P2671': u'Google_Knowledge_Graph_identifier_(P2671)',
    u'P2672': u'SOATO_ID_(P2672)',
    u'P2673': u'next_crossing_upstream_(P2673)',
    u'P2674': u'next_crossing_downstream_(P2674)',
    u'P2675': u'reply_to_(P2675)',
    u'P2676': u'rating_certificate_ID_(P2676)',
    u'P2677': u'relative_position_within_image_(P2677)',
    u'P2678': u'Russiancinema.ru_film_ID_(P2678)',
    u'P2679': u'author_of_foreword_(P2679)',
    u'P2680': u'author_of_afterword_(P2680)',
    u'P2681': u'is_recto_of_(P2681)',
    u'P2682': u'is_verso_of_(P2682)',
    u'P2683': u'Bekker_Number_(P2683)',
    u'P2684': u'Kijkwijzer_rating_(P2684)',
    u'P2685': u'Basketball-Reference.com_NBA_player_ID_(P2685)',
    u'P2686': u'OpenSecrets_people_ID_(P2686)',
    u'P2687': u'NDL_JPNO_(P2687)',
    u'P2688': u'Box_Office_Mojo_person_ID_(P2688)',
    u'P2689': u'BARTOC_ID_(P2689)',
    u'P2694': u'ISU_figure_skater_ID_(P2694)',
    u'P2695': u'ype_locality_(P2695)',
    u'P2696': u'FIG_gymnast_licence_number_(P2696)',
    u'P2697': u'ESPNcricinfo.com_player_ID_(P2697)',
    u'P2698': u'CricketArchive_player_ID_(P2698)',
    u'P2699': u'URL_(P2699)',
    u'P2700': u'protocol_(P2700)',
    u'P2701': u'file_format_(P2701)',
    u'P2702': u'dataset_distribution_(P2702)',
    u'P2703': u'BFI_work_ID_(P2703)',
    u'P2704': u'EIDR_identifier_(P2704)',
    u'P2705': u'Karate_Records_ID_(P2705)',
    u'P2708': u'Cycling_Quotient_women\'s_race_ID_(P2708)',
    u'P2709': u'Cycling_Quotient_female_cyclist_ID_(P2709)',
    u'P2710': u'minimal_lethal_concentration_(P2710)',
    u'P2712': u'median_lethal_concentration_(P2712)',
    u'P2713': u'sectional_view_(P2713)',
    u'P2715': u'elected_in_(P2715)',
    u'P2716': u'collage_image_(P2716)',
    u'P2717': u'no-observed-adverse-effect_level_(P2717)',
    u'P2718': u'lowest-observed-adverse-effect_level_(P2718)',
    u'P2719': u'Hungarian-style_transcription_(P2719)',
    u'P2720': u'embed_URL_template_(P2720)',
    u'P2721': u'Encyclopaedia_Metallum_release_ID_(P2721)',
    u'P2722': u'Deezer_artist_ID_(P2722)',
    u'P2723': u'Deezer_album_ID_(P2723)',
    u'P2724': u'Deezer_track_ID_(P2724)',
    u'P2725': u'GOG_application_ID_(P2725)',
    u'P2726': u'UIPM_ID_(P2726)',
    u'P2727': u'United_World_Wrestling_ID_(P2727)',
    u'P2728': u'CageMatch_worker_ID_(P2728)',
    u'P2729': u'Badminton_World_Federation_ID_(P2729)',
    u'P2730': u'ISSF_ID_(P2730)',
    u'P2731': u'Projeto_Excelências_ID_(P2731)',
    u'P2732': u'Persée_author_ID_(P2732)',
    u'P2733': u'Persée_journal_ID_(P2733)',
    u'P2734': u'UNZ_author_identifier_(P2734)',
    u'P2735': u'UNZ_journal_identifier_(P2735)',
    u'P2736': u'Biographical_Directory_of_Federal_Judges_ID_(P2736)',
    u'P2737': u'union_of_(P2737)',
    u'P2738': u'disjoint_union_of_(P2738)',
    u'P2739': u'ypeface/font_(P2739)',
    u'P2740': u'ResearchGate_institute_ID_(P2740)',
    u'P2741': u'Tate_artist_identifier_(P2741)',
    u'P2742': u'Australian_Geological_Provinces_ID_(P2742)',
    u'P2743': u'his_zoological_name_is_coordinate_with_(P2743)',
    u'P2744': u'PASE_name_(P2744)',
    u'P2745': u'Dictionary_of_New_Zealand_Biography_(P2745)',
    u'P2746': u'production_statistics_(P2746)',
    u'P2747': u'Filmiroda_rating_(P2747)',
    u'P2748': u'PRONOM_file_format_identifier_(P2748)',
    u'P2749': u'PRONOM_software_identifier_(P2749)',
    u'P2750': u'Photographers\'_Identities_Catalog_ID_(P2750)',
    u'P2751': u'Roller_Coaster_Database_ID_(P2751)',
    u'P2752': u'New_Zealand_Organisms_Register_ID_(P2752)',
    u'P2753': u'Dictionary_of_Canadian_Biography_ID_(P2753)',
    u'P2754': u'production_date_(P2754)',
    u'P2755': u'exploitation_visa_number_(P2755)',
    u'P2756': u'EIRIN_film_rating_(P2756)',
    u'P2758': u'CNC_film_rating_(France)_(P2758)',
    u'P2759': u'AUSNUT_food_ID_(P2759)',
    u'P2760': u'NUTTAB_food_ID_(P2760)',
    u'P2761': u'Research_Papers_in_Economics_Series_handle_(P2761)',
    u'P2762': u'Skyscraper_Center_building_complex_ID_(P2762)',
    u'P2763': u'Danish_protected_area_ID_(P2763)',
    u'P2764': u'Wrestlingdata_person_ID_(P2764)',
    u'P2765': u'blue-style.com_ID_(P2765)',
    u'P2766': u'ISO_4063_process_number_(P2766)',
    u'P2767': u'JudoInside.com_ID_(P2767)',
    u'P2768': u'BNE_journal_ID_(P2768)',
    u'P2769': u'budget_(P2769)',
    u'P2770': u'source_of_income_(P2770)',
    u'P2771': u'D-U-N-S_(P2771)',
    u'P2772': u'FIS_alpine_skier_ID_(P2772)',
    u'P2773': u'FIS_cross-country_skier_ID_(P2773)',
    u'P2774': u'FIS_freestyle_skier_ID_(P2774)',
    u'P2775': u'FIS_ski_jumper_ID_(P2775)',
    u'P2776': u'FIS_Nordic_combined_skier_ID_(P2776)',
    u'P2777': u'FIS_snowboarder_ID_(P2777)',
    u'P2778': u'IAT_triathlete_ID_(P2778)',
    u'P2779': u'IAT_weightlifter_ID_(P2779)',
    u'P2780': u'IAT_diver_ID_(P2780)',
    u'P2781': u'race_time_(P2781)',
    u'P2782': u'Models.com_client_ID_(P2782)',
    u'P2783': u'Danish_listed_buildings_case_ID_(P2783)',
    u'P2784': u'Mercalli_intensity_scale_(P2784)',
    u'P2786': u'aerodrome_reference_point_(P2786)',
    u'P2787': u'longest_span_(P2787)',
    u'P2788': u'Czech_neighbourhood_ID_code_(P2788)',
    u'P2789': u'connects_with_(P2789)',
    u'P2790': u'net_tonnage_(P2790)',
    u'P2791': u'power_consumed_(P2791)',
    u'P2792': u'ASF_KID_Cave_Tag_Number_(P2792)',
    u'P2793': u'clearance_(P2793)',
    u'P2794': u'Index_Hepaticarum_ID_(P2794)',
    u'P2795': u'directions_(P2795)',
    u'P2796': u'3DMet_ID_(P2796)',
    u'P2797': u'sound_power_level_(P2797)',
    u'P2798': u'Loop_ID_(P2798)',
    u'P2799': u'BVMC_person_ID_(P2799)',
    u'P2800': u'Beach_Volleyball_Database_ID_(P2800)',
    u'P2801': u'FIVB_beach_volleyball_player_ID_(P2801)',
    u'P2802': u'fleet_or_registration_number_(P2802)',
    u'P2803': u'Wikidata_time_precision_(P2803)',
    u'P2804': u'World_Sailing_member_ID_(P2804)',
    u'P2805': u'Goratings_ID_(P2805)',
    u'P2806': u'vibration_(P2806)',
    u'P2807': u'molar_volume_(P2807)',
    u'P2808': u'wavelength_(P2808)',
    u'P2809': u'Australasian_Pollen_and_Spore_Atlas_Code_(P2809)',
    u'P2810': u'LPGA_Tour_ID_(P2810)',
    u'P2811': u'PGA_Tour_ID_(P2811)',
    u'P2812': u'MathWorld_identifier_(P2812)',
    u'P2813': u'mouthpiece_(P2813)',
    u'P2814': u'P-number_(P2814)',
    u'P2815': u'ESR_station_code_(P2815)',
    u'P2816': u'HowLongToBeat_ID_(P2816)',
    u'P2817': u'appears_in_the_heritage_monument_list_(P2817)',
    u'P2818': u'Sherdog_ID_(P2818)',
    u'P2819': u'Yandex.Music_album_ID_(P2819)',
    u'P2820': u'cardinality_of_this_set_(P2820)',
    u'P2821': u'by-product_(P2821)',
    u'P2822': u'by-product_of_(P2822)',
    u'P2823': u'Royal_Belgian_Football_Association_player_ID_(P2823)',
    u'P2824': u'Gazetteer_of_Planetary_Nomenclature_ID_(P2824)',
    u'P2825': u'via_(P2825)',
    u'P2826': u'Megogo_ID_(P2826)',
    u'P2827': u'flower_color_(P2827)',
    u'P2828': u'corporate_officer_(P2828)',
    u'P2829': u'Internet_Wrestling_Database_ID_(P2829)',
    u'P2830': u'Online_World_of_Wrestling_ID_(P2830)',
    u'P2831': u'otem_(P2831)',
    u'P2832': u'Joint_Electronics_Type_Designation_Automated_System_designation_(P2832)',
    u'P2833': u'ARKive_ID_(P2833)',
    u'P2834': u'individual_tax_rate_(P2834)',
    u'P2835': u'lowest_income_threshold_(P2835)',
    u'P2836': u'highest_income_threshold_(P2836)',
    u'P2837': u'Wikidata_month_number_(P2837)',
    u'P2838': u'professional_name_(Japan)_(P2838)',
    u'P2839': u'gait_(P2839)',
    u'P2840': u'NSC_number_(P2840)',
    u'P2841': u'age_of_onset_(P2841)',
    u'P2842': u'place_of_marriage_(P2842)',
    u'P2843': u'Benezit_ID_(P2843)',
    u'P2844': u'incidence_(P2844)',
    u'P2845': u'RAN_ID_(P2845)',
    u'P2846': u'wheelchair_accessibility_(P2846)',
    u'P2847': u'Google+_ID_(P2847)',
    u'P2848': u'Wi-Fi_(P2848)',
    u'P2849': u'produced_by_(P2849)',
    u'P2850': u'iTunes_artist_ID_(P2850)',
    u'P2851': u'payment_types_accepted_(P2851)',
    u'P2852': u'emergency_phone_number_(P2852)',
    u'P2853': u'electrical_plug_type_(P2853)',
    u'P2854': u'disease_burden_(P2854)',
    u'P2855': u'VAT-rate_(P2855)',
    u'P2856': u'EU_Surface_Water_Body_Code_(P2856)',
    u'P2857': u'WWE.com_superstar_ID_(P2857)',
    u'P2858': u'KLOV_ID_(P2858)',
    u'P2859': u'X-SAMPA_Code_(P2859)',
    u'P2860': u'cites_(P2860)',
    u'P2861': u'Leidse_Hoogleraren_ID_(P2861)',
    u'P2862': u'Catalogus_Professorum_Academiae_Rheno-Traiectinae_ID_(P2862)',
    u'P2863': u'Nederlandse_Molendatabase_ID_(P2863)',
    u'P2864': u'OpenCritic_ID_(P2864)',
    u'P2865': u'band_number_(P2865)',
    u'P2866': u'Hollandsche_Molen_ID_(P2866)',
    u'P2867': u'Molenecho\'s_ID_(P2867)',
    u'P2868': u'subject_has_role_(P2868)',
    u'P2869': u'record_or_record_progression_(P2869)',
    u'P2870': u'miRBase_pre-miRNA_ID_(P2870)',
    u'P2871': u'miRBase_mature_miRNA_ID_(P2871)',
    u'P2872': u'ourist_office_(P2872)',
    u'P2873': u'ime_in_space_(P2873)',
    u'P2874': u'PubChem_BioAssay_ID_(AID)_(P2874)',
    u'P2875': u'property_usage_tracking_category_(P2875)',
    u'P2876': u'ype_of_unit_for_this_property_(P2876)',
    u'P2877': u'SureChEMBL_ID_(P2877)',
    u'P2878': u'Minitel_code_(P2878)',
    u'P2879': u'Library_of_the_National_Congress_of_Argentina_ID_(P2879)',
    u'P2880': u'NIOSHTIC-2_ID_(P2880)',
    u'P2881': u'promoted_(P2881)',
    u'P2882': u'relegated_(P2882)',
    u'P2883': u'HKMDb_film_ID_(P2883)',
    u'P2884': u'mains_voltage_(P2884)',
    u'P2886': u'Shakeosphere_person_ID_(P2886)',
    u'P2887': u'reserve_number_(Canada)_(P2887)',
    u'P2888': u'exact_match_(P2888)',
    u'P2889': u'FamilySearch_person_ID_(P2889)',
    u'P2892': u'UMLS_CUI_(P2892)',
    u'P2893': u'Skype_username_(P2893)',
    u'P2894': u'day_of_week_(P2894)',
    u'P2895': u'maximum_sustained_winds_(P2895)',
    u'P2896': u'publication_interval_(P2896)',
    u'P2897': u'Eldoblaje_Movie_ID_(P2897)',
    u'P2898': u'Swedish_county_letter_(P2898)',
    u'P2899': u'minimum_age_(P2899)',
    u'P2900': u'fax_number_(P2900)',
    u'P2903': u'Molendatabase_verdwenen_molens_ID_(P2903)',
    u'P2904': u'Beer_Advocate_brewery_ID_(P2904)',
    u'P2905': u'RateBeer_brewery_ID_(P2905)',
    u'P2907': u'imezone_offset_(P2907)',
    u'P2908': u'SecondHandSongs_song_ID_(P2908)',
    u'P2909': u'SecondHandSongs_artist_ID_(P2909)',
    u'P2910': u'icon_(P2910)',
    u'P2911': u'ime_gap_(P2911)',
    u'P2912': u'distinctive_jersey_(P2912)',
    u'P2913': u'date_depicted_(P2913)',
    u'P2914': u'MSBI_person_ID_(P2914)',
    u'P2915': u'ECARTICO_person_ID_(P2915)',
    u'P2916': u'syntax_clarification_(P2916)',
    u'P2917': u'COAM_structure_ID_(P2917)',
    u'P2918': u'PO_Box_(P2918)',
    u'P2919': u'label_in_sign_language_(P2919)',
    u'P2922': u'month_of_the_year_(P2922)',
    u'P2923': u'focal_height_(P2923)',
    u'P2924': u'Great_Russian_Encyclopedia_Online_ID_(P2924)',
    u'P2925': u'domain_of_saint_or_deity_(P2925)',
    u'P2926': u'InterPro_ID_(P2926)',
    u'P2927': u'water_as_percent_of_area_(P2927)',
    u'P2928': u'memory_capacity_(P2928)',
    u'P2929': u'lighthouse_range_(P2929)',
    u'P2930': u'INSPIRE-HEP_author_ID_(P2930)',
    u'P2931': u'Encyclopedia_of_Triangle_Centers_identifier_(P2931)',
    u'P2935': u'connector_(P2935)',
    u'P2936': u'language_used_(P2936)',
    u'P2937': u'parliamentary_term_(P2937)',
    u'P2938': u'Pleiades_place_type_identifier_(P2938)',
    u'P2939': u'CageMatch_tag_team_ID_(P2939)',
    u'P2940': u'Catalogus_Professorum_Rostochiensium_ID_(P2940)',
    u'P2941': u'Munk\'s_Roll_ID_(P2941)',
    u'P2942': u'Dailymotion_channel_ID_(P2942)',
    u'P2943': u'warheroes.ru_ID_(P2943)',
    u'P2944': u'Plarr_ID_(P2944)',
    u'P2945': u'British_Book_Trade_Index_ID_(P2945)',
    u'P2946': u'BacDive_ID_(P2946)',
    u'P2948': u'Estonian_cultural_monument_ID_(P2948)',
    u'P2949': u'WikiTree_person_ID_(P2949)',
    u'P2950': u'Nomisma_ID_(P2950)',
    u'P2951': u'Cultural_heritage_database_in_Austria_ObjektID_(P2951)',
    u'P2952': u'boat_of_heritage_interest_ID_(P2952)',
    u'P2953': u'Estonian_Research_Portal_person_ID_(P2953)',
    u'P2954': u'Transferred_Account_Data_Interchange_Group_Code_(P2954)',
    u'P2955': u'point_of_penalty_(P2955)',
    u'P2956': u'NAIF_ID_(P2956)',
    u'P2957': u'hroughput_(P2957)',
    u'P2959': u'permanent_duplicated_item_(P2959)',
    u'P2960': u'archive_date_(P2960)',
    u'P2961': u'BVPH_ID_(P2961)',
    u'P2962': u'itle_of_chess_player_(P2962)',
    u'P2963': u'Goodreads_author_ID_(P2963)',
    u'P2964': u'right_to_vote_(P2964)',
    u'P2965': u'EU_River_Basin_District_code_(P2965)',
    u'P2966': u'National_Library_of_Wales_ID_(P2966)',
    u'P2967': u'Queensland_Heritage_Register_ID_(P2967)',
    u'P2968': u'QUDT_unit_ID_(P2968)',
    u'P2969': u'Goodreads_book_ID_(P2969)',
    u'P2970': u'Kinopolis_film_ID_(P2970)',
    u'P2971': u'GCatholic_church_ID_(P2971)',
    u'P2972': u'SNAP_ID_(P2972)',
    u'P2973': u'Spenserians_person_ID_(P2973)',
    u'P2974': u'habitat_(P2974)',
    u'P2975': u'host_(P2975)',
    u'P2976': u'patronym_or_matronym_(P2976)',
    u'P2977': u'LBT_person_ID_(P2977)',
    u'P2978': u'wheel_arrangement_(P2978)',
    u'P2979': u'maritime_identification_digits_(P2979)',
    u'P2980': u'ARLHS_Lighthouse_ID_(P2980)',
    u'P2981': u'UIC_alphabetical_country_code_(P2981)',
    u'P2982': u'UIC_numerical_country_code_(P2982)',
    u'P2983': u'UNDP_country_code_(P2983)',
    u'P2984': u'Snapchat_username_(P2984)',
    u'P2985': u'DSBE_ID_(P2985)',
    u'P2986': u'aircraft_registration_prefix_(P2986)',
    u'P2987': u'Taekwondo_Data_ID_(P2987)',
    u'P2988': u'GOST_7.67_cyrillic_(P2988)',
    u'P2989': u'has_grammatical_case_(P2989)',
    u'P2990': u'FIL_ID_(P2990)',
    u'P2991': u'IBSF_ID_(P2991)',
    u'P2992': u'software_quality_assurance_(P2992)',
    u'P2993': u'partition_coefficient_water/octanol_(P2993)',
    u'P2997': u'age_of_majority_(P2997)',
    u'P2998': u'age_of_candidacy_(P2998)',
    u'P2999': u'age_of_consent_(P2999)',
    u'P3000': u'marriageable_age_(P3000)',
    u'P3001': u'retirement_age_(P3001)',
    u'P3002': u'Untappd_brewery_ID_(P3002)',
    u'P3003': u'zanikleobce.cz_abandoned_objects_ID_(P3003)',
    u'P3004': u'Galiciana_work_ID_(P3004)',
    u'P3005': u'valid_in_place_(P3005)',
    u'P3006': u'Marine_Regions_Geographic_ID_(P3006)',
    u'P3007': u'Cadw_Monument_ID_(P3007)',
    u'P3008': u'Australian_Heritage_Database_Place_ID_(P3008)',
    u'P3009': u'Conservatoire_du_littoral_ID_(P3009)',
    u'P3010': u'World_Archery_athlete_ID_(P3010)',
    u'P3012': u'Standard_Geographical_Classification_code_(P3012)',
    u'P3013': u'surface_tension_(P3013)',
    u'P3014': u'laws_applied_(P3014)',
    u'P3015': u'backup_or_reserve_team_or_crew_(P3015)',
    u'P3016': u'French_national_research_structure_identifier_(P3016)',
    u'P3017': u'Rolling_Stone_artist_ID_(P3017)',
    u'P3018': u'located_in_protected_area_(P3018)',
    u'P3019': u'railway_signalling_system_(P3019)',
    u'P3020': u'residence_time_of_water_(P3020)',
    u'P3021': u'Iranica_ID_(P3021)',
    u'P3022': u'flag_bearer_(P3022)',
    u'P3023': u'Legacies_of_British_Slave-ownership_person_ID_(P3023)',
    u'P3024': u'ITU_letter_code_(P3024)',
    u'P3025': u'open_days_(P3025)',
    u'P3026': u'closed_on_(P3026)',
    u'P3027': u'open_period_from_(P3027)',
    u'P3028': u'open_period_to_(P3028)',
    u'P3029': u'UK_National_Archives_ID_(P3029)',
    u'P3030': u'sheet_music_(P3030)',
    u'P3031': u'EPPO_Code_(P3031)',
    u'P3032': u'adjacent_building_(P3032)',
    u'P3033': u'package_management_system_(P3033)',
    u'P3034': u'Indonesian_ethnicity_code_(P3034)',
    u'P3035': u'ISBN_publisher_prefix_(P3035)',
    u'P3036': u'precipitation_height_(P3036)',
    u'P3037': u'spatial_reference_system_(P3037)',
    u'P3038': u'IWM_memorial_ID_(P3038)',
    u'P3039': u'wheelbase_(P3039)',
    u'P3040': u'SoundCloud_ID_(P3040)',
    u'P3041': u'luminous_intensity_(P3041)',
    u'P3042': u'CageMatch_wrestling_stable_id_(P3042)',
    u'P3043': u'Scoresway_soccer_person_ID_(P3043)',
    u'P3044': u'College_Football_HoF_ID_(P3044)',
    u'P3045': u'HanCinema_person_ID_(P3045)',
    u'P3046': u'ForaDeJogo_player_ID_(P3046)',
    u'P3047': u'TheFinalBall_player_ID_(P3047)',
    u'P3048': u'racing-reference_driver_ID_(P3048)',
    u'P3049': u'Scottish_Football_Association_player_ID_(P3049)',
    u'P3050': u'FACR_player_ID_(P3050)',
    u'P3051': u'Kindred_Britain_ID_(P3051)',
    u'P3052': u'Bloomberg_person_ID_(P3052)',
    u'P3053': u'K_League_player_ID_(P3053)',
    u'P3054': u'Ontario_MPP_ID_(P3054)',
    u'P3055': u'NAQ_elected_person_ID_(P3055)',
    u'P3056': u'Turner_Classic_Movies_person_ID_(P3056)',
    u'P3057': u'Charity_Commission_no._(P3057)',
    u'P3058': u'Architectuurgids_architect_ID_(P3058)',
    u'P3059': u'Architectuurgids_building_ID_(P3059)',
    u'P3060': u'ButMoth_ID_(P3060)',
    u'P3061': u'Basisregistratie_Instellingen_number_(P3061)',
    u'P3063': u'gestation_period_(P3063)',
    u'P3064': u'LepIndex_ID_(P3064)',
    u'P3065': u'RERO_ID_(P3065)',
    u'P3066': u'GLAM_Identifier_(P3066)',
    u'P3067': u'GS1_country_code_(P3067)',
    u'P3068': u'WIPO_ST.3_(P3068)',
    u'P3069': u'UN_document_symbol_(P3069)',
    u'P3070': u'dynamic_viscosity_(P3070)',
    u'P3071': u'standard_molar_entropy_(P3071)',
    u'P3072': u'Storting_person_ID_(P3072)',
    u'P3073': u'CosIng_number_(P3073)',
    u'P3074': u'Grace\'s_Guide_ID_(P3074)',
    u'P3075': u'official_religion_(P3075)',
    u'P3076': u'Open_Beauty_Facts_category_ID_(P3076)',
    u'P3077': u'Cineplex_film_ID_(P3077)',
    u'P3078': u'standard_enthalpy_of_formation_(P3078)',
    u'P3080': u'game_artist_(P3080)',
    u'P3081': u'damaged_(P3081)',
    u'P3082': u'destroyed_(P3082)',
    u'P3083': u'SIMBAD_ID_(P3083)',
    u'P3085': u'qualifies_for_event_(P3085)',
    u'P3086': u'speed_limit_(P3086)',
    u'P3087': u'fiscal/tax_revenue_(P3087)',
    u'P3088': u'Catalogue_of_Life_in_Taiwan_ID_(P3088)',
    u'P3089': u'Flags_of_the_World_ID_(P3089)',
    u'P3090': u'flight_number_(P3090)',
    u'P3091': u'mount_(P3091)',
    u'P3092': u'film_crew_member_(P3092)',
    u'P3093': u'recovered_by_(P3093)',
    u'P3094': u'develops_from_(P3094)',
    u'P3095': u'practiced_by_(P3095)',
    u'P3096': u'KML_file_(P3096)',
    u'P3097': u'ISBN_identifier_group_(P3097)',
    u'P3098': u'ClinicalTrials.gov_Identifier_(P3098)',
    u'P3099': u'Internet_Bird_Collection_species_ID_(P3099)',
    u'P3100': u'Flora_of_Australia_ID_(P3100)',
    u'P3101': u'FloraBase_ID_(P3101)',
    u'P3102': u'Plantarium_ID_(P3102)',
    u'P3103': u'has_tense_(P3103)',
    u'P3104': u'Gares_&amp;_Connexions_ID_(P3104)',
    u'P3105': u'Tela_Botanica_ID_(P3105)',
    u'P3106': u'Guardian_topic_ID_(P3106)',
    u'P3107': u'LdiF_ID_(P3107)',
    u'P3108': u'Yelp_ID_(P3108)',
    u'P3109': u'Peakbagger_mountain_ID_(P3109)',
    u'P3110': u'ISzDb_film_ID_(P3110)',
    u'P3111': u'FEI_ID_(P3111)',
    u'P3112': u'DistroWatch_ID_(P3112)',
    u'P3113': u'does_not_have_part_(P3113)',
    u'P3114': u'ISzDb_person_ID_(P3114)',
    u'P3115': u'ISzDb_company_ID_(P3115)',
    u'P3116': u'ISzDb_dub_ID_(P3116)',
    u'P3117': u'DSSTOX_substance_identifier_(P3117)',
    u'P3118': u'OpenDomesday_settlement_ID_(P3118)',
    u'P3119': u'Code_for_China_Reservoir_Name_(P3119)',
    u'P3120': u'TOID_(P3120)',
    u'P3121': u'Epguides_ID_(P3121)',
    u'P3122': u'OpenDomesday_person_ID_(P3122)',
    u'P3123': u'Stanford_Encyclopedia_of_Philosophy_ID_(P3123)',
    u'P3124': u'Polish_scientist_ID_(P3124)',
    u'P3125': u'EDRPOU_code_(P3125)',
    u'P3126': u'ALCUIN_ID_(P3126)',
    u'P3127': u'Latindex_ID_(P3127)',
    u'P3128': u'CiNetMag_film_ID_(P3128)',
    u'P3129': u'cine.gr_film_ID_(P3129)',
    u'P3130': u'NSW_Flora_ID_(P3130)',
    u'P3131': u'Redalyc_journal_ID_(P3131)',
    u'P3132': u'last_line_(P3132)',
    u'P3133': u'NSZL_name_authority_ID_(P3133)',
    u'P3134': u'TripAdvisor_ID_(P3134)',
    u'P3135': u'elCinema_film_ID_(P3135)',
    u'P3136': u'elCinema_person_ID_(P3136)',
    u'P3137': u'parent_peak_(P3137)',
    u'P3138': u'OFDb_ID_(P3138)',
    u'P3139': u'SourehCinema_film_ID_(P3139)',
    u'P3140': u'SourehCinema_person_ID_(P3140)',
    u'P3141': u'EDb_film_ID_(P3141)',
    u'P3142': u'EDb_person_ID_(P3142)',
    u'P3143': u'elFilm_film_ID_(P3143)',
    u'P3144': u'elFilm_person_ID_(P3144)',
    u'P3145': u'Sratim_ID_(P3145)',
    u'P3146': u'CiNetMag_person_ID_(P3146)',
    u'P3147': u'United_States_Reports_ID_(P3147)',
    u'P3148': u'repeals_(P3148)',
    u'P3149': u'molecule_conformation_(P3149)',
    u'P3150': u'birthday_(P3150)',
    u'P3151': u'iNaturalist_taxon_ID_(P3151)',
    u'P3152': u'Findsmiley_ID_(P3152)',
    u'P3153': u'Crossref_funder_ID_(P3153)',
    u'P3154': u'Runeberg_author_ID_(P3154)',
    u'P3155': u'Runeberg_book_ID_(P3155)',
    u'P3156': u'Australian_Classification_(P3156)',
    u'P3157': u'event_distance_(P3157)',
    u'P3158': u'enclosure_(P3158)',
    u'P3159': u'UGentMemorialis_ID_(P3159)',
    u'P3160': u'Minnesota_legislator_ID_(P3160)',
    u'P3161': u'has_grammatical_mood_(P3161)',
    u'P3162': u'Rock_Hall_of_Fame_ID_(P3162)',
    u'P3163': u'Scottish_Charity_number_(P3163)',
    u'P3165': u'Horsetelex_ID_(P3165)',
    u'P3166': u'Webpedigrees_ID_(P3166)',
    u'P3167': u'Allbreedpedigree_ID_(P3167)',
    u'P3168': u'Sporthorse_data_ID_(P3168)',
    u'P3169': u'Harasire_ID_(P3169)',
    u'P3170': u'Cultural_Heritage_Armenia_ID_(P3170)',
    u'P3171': u'International_Olympic_Committee_athlete_ID_(P3171)',
    u'P3172': u'World_Bridge_Federation_ID_(P3172)',
    u'P3173': u'offers_view_on_(P3173)',
    u'P3174': u'art_director_(P3174)',
    u'P3175': u'Statoids_ID_(P3175)',
    u'P3176': u'uses_property_(P3176)',
    u'P3177': u'Patrimonio_Web_JCyL_ID_(P3177)',
    u'P3178': u'Zaragoza_monument_ID_(P3178)',
    u'P3179': u'erritory_overlaps_(P3179)',
    u'P3180': u'Visual_Novel_Database_ID_(P3180)',
    u'P3181': u'OpenCitations_bibliographic_resource_ID_(P3181)',
    u'P3182': u'FANTOIR_code_(P3182)',
    u'P3183': u'WSJ_topic_ID_(P3183)',
    u'P3184': u'Czech_National_Bibliography_book_ID_(P3184)',
    u'P3185': u'VKontakte_username_(P3185)',
    u'P3186': u'TAXREF_ID_(P3186)',
    u'P3187': u'Marburger_Professorenkatalog_ID_(P3187)',
    u'P3188': u'Nobel_prize_ID_(P3188)',
    u'P3189': u'innervated_by_(P3189)',
    u'P3190': u'innervates_(P3190)',
    u'P3191': u'IMIS_person_ID_(P3191)',
    u'P3192': u'Last.fm_music_ID_(P3192)',
    u'P3193': u'GS1_Manufacturer_code_(P3193)',
    u'P3194': u'INA_video_ID_(P3194)',
    u'P3195': u'league_points_system_(P3195)',
    u'P3196': u'USGS_earthquake_id_(P3196)',
    u'P3197': u'Federal_Heritage_Buildings_ID_(Canada)_(P3197)',
    u'P3198': u'JewishGen_Locality_ID_(P3198)',
    u'P3199': u'charity_number_(Isle_of_Man)_(P3199)',
    u'P3200': u'Northern_Ireland_charity_number_(P3200)',
    u'P3201': u'Medical_Dictionary_for_Regulatory_Activities_ID_(P3201)',
    u'P3202': u'UAI_code_(P3202)',
    u'P3203': u'Ciné-Ressources_film_ID_(P3203)',
    u'P3204': u'Ciné-Ressources_person_ID_(P3204)',
    u'P3205': u'patient_of_(P3205)',
    u'P3206': u'data.gouv.fr_ID_(P3206)',
    u'P3207': u'Vine_user_ID_(P3207)',
    u'P3208': u'New_General_Catalogue_ID_(P3208)',
    u'P3209': u'Heritage_Railway_Station_of_Canada_ID_(P3209)',
    u'P3211': u'Heritage_Lighthouse_of_Canada_ID_(P3211)',
    u'P3212': u'ISAN_(P3212)',
    u'P3213': u'Indian_census_area_code_(P3213)',
    u'P3215': u'FR_SIRET_number_(P3215)',
    u'P3216': u'ClassInd_rating_(P3216)',
    u'P3217': u'Dictionary_of_Swedish_National_Biography_(P3217)',
    u'P3218': u'Auñamendi_ID_(P3218)',
    u'P3219': u'Encyclopædia_Universalis_Online_ID_(P3219)',
    u'P3220': u'KvK_company_ID_(P3220)',
    u'P3221': u'NYT_topic_ID_(P3221)',
    u'P3222': u'NE.se_ID_(P3222)',
    u'P3223': u'Online_List_of_Lights_id_(P3223)',
    u'P3224': u'NAICS_code_(P3224)',
    u'P3225': u'Corporate_Number_(Japan)_(P3225)',
    u'P3226': u'HAS_member_ID_(P3226)',
    u'P3227': u'Cultural_Heritage_Kosovo_ID_(P3227)',
    u'P3228': u'Schläfli_symbol_(P3228)',
    u'P3229': u'South_Dakota_legislator_ID_(P3229)',
    u'P3230': u'SCAR_Composite_Gazetteer_place_ID_(P3230)',
    u'P3231': u'Mediaarts-DB_ID_(P3231)',
    u'P3232': u'PhilPapers_author_ID_(P3232)',
    u'P3233': u'PhilPapers_profile_(P3233)',
    u'P3234': u'Library_of_Congress_Cultural_Heritage_Organizations_(P3234)',
    u'P3235': u'PhilPapers_topic_(P3235)',
    u'P3236': u'PhilPapers_publication_ID_(P3236)',
    u'P3237': u'KU_Leuven_person_ID_(P3237)',
    u'P3238': u'runk_prefix_(P3238)',
    u'P3240': u'NBN_System_Key_(P3240)',
    u'P3241': u'Catholic_Encyclopedia_ID_(P3241)',
    u'P3242': u'SIC_code_(P3242)',
    u'P3243': u'OKVED_1.1_code_of_the_economic_activity_(P3243)',
    u'P3245': u'OKPD_code_of_the_good_or_service_(P3245)',
    u'P3246': u'OKVED_2_code_of_the_economic_activity_(P3246)',
    u'P3248': u'OKP_ID_of_the_good_or_service_(P3248)',
    u'P3250': u'OKPD2_code_of_the_good_or_service_(P3250)',
    u'P3251': u'minimum_viable_temperature_(P3251)',
    u'P3252': u'maximum_viable_temperature_(P3252)',
    u'P3253': u'optimum_viable_temperature_(P3253)',
    u'P3254': u'property_proposal_discussion_(P3254)',
    u'P3256': u'Cave_E-Cadastre_ID_(P3256)',
    u'P3257': u'Queensland_place_ID_(P3257)',
    u'P3258': u'LiveJournal_ID_(P3258)',
    u'P3259': u'intangible_cultural_heritage_status_(P3259)',
    u'P3260': u'points_awarded_(P3260)',
    u'P3261': u'anatomical_branch_of_(P3261)',
    u'P3262': u'has_anatomical_branch_(P3262)',
    u'P3263': u'base_(P3263)',
    u'P3264': u'radix_(P3264)',
    u'P3265': u'Myspace_ID_(P3265)',
    u'P3266': u'LocFDD_ID_(P3266)',
    u'P3267': u'Flickr_user_ID_(P3267)',
    u'P3268': u'PORT_organization_URL_(P3268)',
    u'P3269': u'Fotografen.nl_ID_(P3269)',
    u'P3270': u'compulsory_education_(minimum_age)_(P3270)',
    u'P3271': u'compulsory_education_(maximum_age)_(P3271)',
    u'P3272': u'Zeri_image_ID_(P3272)',
    u'P3273': u'Actorenregister_ID_(P3273)',
    u'P3274': u'content_deliverer_(P3274)',
    u'P3275': u'storyboard_artist_(P3275)',
    u'P3276': u'Angel_List_ID_(P3276)',
    u'P3277': u'KANTL_member_ID_(P3277)',
    u'P3279': u'statistical_leader_(P3279)',
    u'P3280': u'BanQ_author_ID_(P3280)',
    u'P3281': u'French_National_Assembly_Lobbyist_ID_(P3281)',
    u'P3283': u'Bandcamp_artist_ID_(P3283)',
    u'P3284': u'Yahoo!_Japan_Talent_Database_ID_(P3284)',
    u'P3285': u'Mathematics_Subject_Classification_ID_(P3285)',
    u'P3286': u'Squash_Info_ID_(P3286)',
    u'P3288': u'World_Spider_Catalog_ID_(P3288)',
    u'P3289': u'Cellosaurus_ID_(P3289)',
    u'P3290': u'biography_at_the_Landtag_of_Mecklenburg-Vorpommern_(P3290)',
    u'P3291': u'DocCheck_Flexikon_En_ID_(P3291)',
    u'P3292': u'DocCheck_Flexikon_De_ID_(P3292)',
    u'P3293': u'BALaT_image_ID_(P3293)',
    u'P3294': u'encoding_(P3294)',
    u'P3295': u'code_(P3295)',
    u'P3296': u'DRÚSOP_ID_(P3296)',
    u'P3297': u'Flemish_Parliament_person_ID_(P3297)',
    u'P3298': u'Belgian_Senate_person_ID_(P3298)',
    u'P3299': u'student_register_of_the_University_of_Helsinki_ID_(1640–1852)_(P3299)',
    u'P3300': u'musical_conductor_(P3300)',
    u'P3301': u'broadcast_by_(P3301)',
    u'P3302': u'Open_Media_Database_film_ID_(P3302)',
    u'P3303': u'hird-party_formatter_URL_(P3303)',
    u'P3304': u'NGS_pumping_station_ID_(P3304)',
    u'P3305': u'KINENOTE_person_ID_(P3305)',
    u'P3306': u'ICAA_rating_(P3306)',
    u'P3307': u'Galiciana_Author_ID_(P3307)',
    u'P3308': u'lib.reviews_ID_(P3308)',
    u'P3309': u'SummitPost_mountain_ID_(P3309)',
    u'P3310': u'muscle_action_(P3310)',
    u'P3311': u'plan_image_(P3311)',
    u'P3314': u'365chess_player_ID_(P3314)',
    u'P3315': u'chesstempo_ID_(P3315)',
    u'P3316': u'ICCF_player_ID_(P3316)',
    u'P3318': u'Patrimonio_Inmueble_de_Andalucía_ID_(P3318)',
    u'P3320': u'board_member_(P3320)',
    u'P3321': u'male_form_of_label_(P3321)',
    u'P3322': u'Vlinderstichting-ID_(P3322)',
    u'P3323': u'opponent_during_disputation_(P3323)',
    u'P3324': u'petit-patrimoine.com_ID_(P3324)',
    u'P3325': u'student_register_of_the_University_of_Helsinki_ID_(1853–1899)_(P3325)',
    u'P3326': u'World_Waterfall_Database_ID_(P3326)',
    u'P3327': u'Réserves_naturelles_de_France_ID_(P3327)',
    u'P3328': u'wurvoc.org_measure_ID_(P3328)',
    u'P3329': u'CIViC_variant_ID_(P3329)',
    u'P3330': u'Supermodels.nl_ID_(P3330)',
    u'P3331': u'HGVS_nomenclature_(P3331)',
    u'P3332': u'ACM_Digital_Library_citation_ID_(P3332)',
    u'P3333': u'ACM_Digital_Library_event_ID_(P3333)',
    u'P3335': u'hazard_on_site_(P3335)',
    u'P3337': u'generation_time_(P3337)',
    u'P3338': u'Encyclopedia_of_Surfing_ID_(P3338)',
    u'P3339': u'World_Surf_League_ID_(P3339)',
    u'P3340': u'Kvikmyndir_film_ID_(P3340)',
    u'P3341': u'Kvikmyndir_person_ID_(P3341)',
    u'P3342': u'significant_person_(P3342)',
    u'P3343': u'legislation.gov.uk_ID_(P3343)',
    u'P3344': u'Vote_Smart_ID_(P3344)',
    u'P3345': u'RxNorm_CUI_(P3345)',
    u'P3346': u'HKMDb_person_ID_(P3346)',
    u'P3347': u'PermID_(P3347)',
    u'P3348': u'National_Library_of_Greece_ID_(P3348)',
    u'P3349': u'designed_to_carry_(P3349)',
    u'P3350': u'WHO_international_non-proprietary_names_ID_(P3350)',
    u'P3351': u'Adult_Film_Database_person_ID_(P3351)',
    u'P3352': u'musipedia_tune_ID_(P3352)',
    u'P3353': u'National_Recreation_Trails_Database_ID_(P3353)',
    u'P3354': u'positive_therapeutic_predictor_(P3354)',
    u'P3355': u'negative_therapeutic_predictor_(P3355)',
    u'P3356': u'positive_diagnostic_predictor_(P3356)',
    u'P3357': u'negative_diagnostic_predictor_(P3357)',
    u'P3358': u'positive_prognostic_predictor_(P3358)',
    u'P3359': u'negative_prognostic_predictor_(P3359)',
    u'P3360': u'Nobel_Prize_People_Nomination_ID_(P3360)',
    u'P3361': u'PictoRight_ID_code_(P3361)',
    u'P3362': u'operating_income_(P3362)',
    u'P3363': u'Tennis_HoF_player_ID_(P3363)',
    u'P3364': u'stereoisomer_of_(P3364)',
    u'P3365': u'Treccani_ID_(P3365)',
    u'P3366': u'GECD_person_ID_(P3366)',
    u'P3367': u'GECD_film_ID_(P3367)',
    u'P3368': u'Prabook_ID_(P3368)',
    u'P3370': u'Géopatronyme_ID_(P3370)',
    u'P3371': u'Observatoire_du_Patrimoine_Religieux_ID_(P3371)',
    u'P3372': u'Auckland_Art_Gallery_artist_ID_(P3372)',
    u'P3373': u'sibling_(P3373)',
    u'P3374': u'endianness_(P3374)',
    u'P3375': u'GECD_Firmen-ID_(P3375)',
    u'P3376': u'Enterprise_number_(Belgium)_(P3376)',
    u'P3377': u'Bloomberg_private_company_ID_(P3377)',
    u'P3378': u'Merck_Index_reaction_ID_(P3378)',
    u'P3379': u'Model_Manual_ID_(P3379)',
    u'P3380': u'FAO_2007_genetic_resource_ID_(P3380)',
    u'P3381': u'File_Format_Wiki_page_ID_(P3381)',
    u'P3382': u'GeneDB_ID_(P3382)',
    u'P3383': u'film_poster_(P3383)',
    u'P3385': u'Japan_Sumo_Association_ID_(P3385)',
    u'P3386': u'French_Sculpture_Census_work_ID_(P3386)',
    u'P3387': u'minimum_frequency_of_audible_sound_(P3387)',
    u'P3388': u'LittleSis_people_ID_(P3388)',
    u'P3389': u'Royal_Swedish_Academy_of_Letters_member_ID_(P3389)',
    u'P3390': u'Consolidated_code_of_the_electronic_catalog_of_libraries_of_Belarus_(P3390)',
    u'P3391': u'Verkhovna_Rada_MP_ID_(P3391)',
    u'P3392': u'Surman_ID_(P3392)',
    u'P3393': u'LittleSis_organisation_ID_(P3393)',
    u'P3394': u'Finnish_Lake_ID_(P3394)',
    u'P3395': u'heart_rate_(P3395)',
    u'P3396': u'French_Catholic_Church_structure_ID_(P3396)',
    u'P3397': u'Guide_of_the_French_Church_person_ID_(P3397)',
    u'P3398': u'Butterflies_and_Moths_of_North_America_ID_(P3398)',
    u'P3399': u'JTWC_tropical_cyclone_ID_(P3399)',
    u'P3400': u'CORDIS_Project_ID_(P3400)',
    u'P3401': u'Wilderness.net_ID_(P3401)',
    u'P3402': u'CNC_film_rating_(Romania)_(P3402)',
    u'P3403': u'coextensive_with_(P3403)',
    u'P3404': u'The_Vogue_List_ID_(P3404)',
    u'P3405': u'Nederlands_Soortenregister_ID_(P3405)',
    u'P3406': u'Saccharomyces_Genome_Database_ID_(P3406)',
    u'P3407': u'Klosterdatenbank_ID_(P3407)',
    u'P3408': u'FINA_athlete_ID_(P3408)',
    u'P3409': u'Catalogus_Professorum_Lipsiensis_ID_(P3409)',
    u'P3410': u'Clergy_of_the_Church_of_England_database_ID_(P3410)',
    u'P3411': u'Saxon_Academy_of_Sciences_member_ID_(P3411)',
    u'P3412': u'Canadian_Environmental_Sustainability_Indicators_ID_(Protected_areas)_(P3412)',
    u'P3413': u'Leopoldina_member_ID_(P3413)',
    u'P3414': u'Yle_Areena_ID_(P3414)',
    u'P3415': u'start_period_(P3415)',
    u'P3416': u'end_period_(P3416)',
    u'P3417': u'Quora_topic_ID_(P3417)',
    u'P3418': u'Google_Play_Store_ID_(P3418)',
    u'P3419': u'Basic_Unit_of_Settlement_code_(Czech/Slovak)_(P3419)',
    u'P3420': u'Calflora_ID_(P3420)',
    u'P3421': u'Belvedere_artist_ID_(P3421)',
    u'P3422': u'INSEE_countries_and_foreign_territories_code_(P3422)',
    u'P3423': u'INSEE_arrondissement_code_(P3423)',
    u'P3424': u'Polish_cultural_heritage_register_number_(P3424)',
    u'P3425': u'Natura_2000_site_ID_(P3425)',
    u'P3426': u'ArbetSam_ID_(P3426)',
    u'P3427': u'AS.com_athlete_ID_(P3427)',
    u'P3428': u'INCAA_film_rating_(P3428)',
    u'P3429': u'Electronic_Enlightenment_ID_(P3429)',
    u'P3430': u'SNAC_Ark_ID_(P3430)',
    u'P3431': u'Publons_Publication_ID_(P3431)',
    u'P3432': u'parent_cell_line_(P3432)',
    u'P3433': u'biological_variant_of_(P3433)',
    u'P3434': u'ERIH_PLUS_ID_(P3434)',
    u'P3435': u'VGMdb_artist_ID_(P3435)',
    u'P3436': u'AELG_ID_(P3436)',
    u'P3437': u'people_or_cargo_transported_(P3437)',
    u'P3438': u'vehicle_normally_used_(P3438)',
    u'P3439': u'angular_resolution_(P3439)',
    u'P3440': u'ime_signature_(P3440)',
    u'P3441': u'FIFA_country_code_(P3441)',
    u'P3442': u'Debian_stable_package_(P3442)',
    u'P3443': u'Victorian_Heritage_Database_ID_(P3443)',
    u'P3444': u'eBird_ID_(P3444)',
    u'P3445': u'Cinema_of_Israel_ID_(P3445)',
    u'P3446': u'FIDAL_ID_(P3446)',
    u'P3447': u'mirrors_data_from_(P3447)',
    u'P3448': u'stepparent_(P3448)',
    u'P3449': u'NSW_Heritage_database_ID_(P3449)',
    u'P3450': u'sports_season_of_(P3450)',
    u'P3451': u'nighttime_view_(P3451)',
    u'P3452': u'inferred_from_(P3452)',
    u'P3453': u'IPI_base_code_(P3453)',
    u'P3454': u'Arch_package_(P3454)',
    u'P3455': u'CircleID_(P3455)',
    u'P3456': u'ATP_tennis_tournament_ID_(P3456)',
    u'P3457': u'case_fatality_rate_(P3457)',
    u'P3458': u'CNC_authorization_number_(P3458)',
    u'P3459': u'Euring_number_(P3459)',
    u'P3460': u'colonel-in-chief_(P3460)',
    u'P3461': u'designated_as_terrorist_by_(P3461)',
    u'P3462': u'FAMA_work_ID_(P3462)',
    u'P3463': u'Fedora_package_(P3463)',
    u'P3464': u'medicine_marketing_authorization_(P3464)',
    u'P3465': u'maximum_frequency_of_audible_sound_(P3465)',
    u'P3466': u'Israeli_CBS_municipal_ID_(P3466)',
    u'P3467': u'Inventario_Sculture_-_Polo_Museale_Fiorentino_(P3467)',
    u'P3468': u'National_Inventors_Hall_of_Fame_ID_(P3468)',
    u'P3469': u'WTA_tennis_tournament_ID_(P3469)',
    u'P3470': u'Woodland_Trust_wood_ID_(P3470)',
    u'P3471': u'WikiSkripta_ID_(P3471)',
    u'P3472': u'VICNAMES_Place_ID_(P3472)',
    u'P3473': u'Ubuntu_package_(P3473)',
    u'P3475': u'SANU_member_ID_(P3475)',
    u'P3476': u'PSA_World_Tour_ID_(P3476)',
    u'P3477': u'Nihon_Tarento_Meikan_ID_(P3477)',
    u'P3478': u'Songkick_artist_ID_(P3478)',
    u'P3479': u'Omni_topic_ID_(P3479)',
    u'P3480': u'base_Mémoire_reference_(P3480)',
    u'P3481': u'Parks_&amp;_Gardens_UK_Record_ID_(P3481)',
    u'P3482': u'Europeana_Fashion_creator_ID_(P3482)',
    u'P3483': u'VGMdb_album_ID_(P3483)',
    u'P3484': u'name_shares_origin_with_(P3484)',
    u'P3485': u'bite_force_quotient_(P3485)',
    u'P3486': u'normal_respiratory_rate_(P3486)',
    u'P3487': u'maximal_incubation_period_in_humans_(P3487)',
    u'P3488': u'minimal_incubation_period_in_humans_(P3488)',
    u'P3489': u'pregnancy_category_(P3489)',
    u'P3490': u'muscle_origin_(P3490)',
    u'P3491': u'muscle_insertion_(P3491)',
    u'P3492': u'basic_reproduction_number_(P3492)',
    u'P3493': u'legal_status_(medicine)_(P3493)',
    u'P3494': u'points_classification_(P3494)',
    u'P3495': u'FilmPolski.pl_ID_(P3495)',
    u'P3496': u'eams_classification_by_points_(P3496)',
    u'P3497': u'eams_classification_by_time_(P3497)',
    u'P3498': u'ZNIEFF_ID_(P3498)',
    u'P3499': u'Gentoo_package_(P3499)',
    u'P3500': u'Ringgold_ID_(P3500)',
    u'P3501': u'Catholic_rite_(P3501)',
    u'P3502': u'Ameblo_username_(P3502)',
    u'P3503': u'LombardiaBeniCulturali_building_ID_(P3503)',
    u'P3504': u'Florentine_Inventario_Palatina_art_ID_(P3504)',
    u'P3505': u'BoardGameGeek_designer_ID_(P3505)',
    u'P3506': u'Luding_designer_ID_(P3506)',
    u'P3507': u'Bivouac.com_mountain_ID_(P3507)',
    u'P3509': u'Dagens_Nyheter_topic_ID_(P3509)',
    u'P3511': u'VGMDb_organization_ID_(P3511)',
    u'P3512': u'means_of_locomotion_(P3512)',
    u'P3513': u'peakware_mountain_ID_(P3513)',
    u'P3514': u'U.S._National_Geodetic_Survey_ID_(P3514)',
    u'P3515': u'NPCA_ID_(P3515)',
    u'P3516': u'National_Park_Foundation_ID_(P3516)',
    u'P3517': u'Geographical_Names_Board_of_New_South_Wales_ID_(P3517)',
    u'P3518': u'Smithsonian_trinomial_(P3518)',
    u'P3519': u'Pfam_ID_(P3519)',
    u'P3520': u'databaseOlympics.com_athlete_ID_(P3520)',
    u'P3521': u'EuroTour_golf_player_ID_(P3521)',
    u'P3522': u'NHL.com_player_ID_(P3522)',
    u'P3523': u'Rfam_ID_(P3523)',
    u'P3524': u'Simple_Modular_Architecture_Research_Tool_ID_(P3524)',
    u'P3525': u'acb.com_ID_(P3525)',
    u'P3526': u'wisdenindia.com_ID_(P3526)',
    u'P3527': u'eurobasket.com_ID_(P3527)',
    u'P3528': u'Luding_game_ID_(P3528)',
    u'P3529': u'median_income_(P3529)',
    u'P3530': u'par_(P3530)',
    u'P3531': u'AZBilliards_ID_(P3531)',
    u'P3532': u'databaseFootball.com_ID_(P3532)',
    u'P3533': u'draftexpress.com_ID_(P3533)',
    u'P3534': u'Australian_Government_Organisations_Register_ID_(P3534)',
    u'P3535': u'JapanTour_golf_player_ID_(P3535)',
    u'P3536': u'euroleague.net_ID_(P3536)',
    u'P3537': u'FootballDatabase.eu_ID_(P3537)',
    u'P3538': u'Fussballdaten.de_ID_(P3538)',
    u'P3539': u'NFL.com_ID_(P3539)',
    u'P3541': u'MLB_ID_(P3541)',
    u'P3542': u'FIBA_player_ID_(P3542)',
    u'P3544': u'Te_Papa_person_ID_(P3544)',
    u'P3545': u'Theoi_Project_ID_(P3545)',
    u'P3546': u'AustralianFootball.com_ID_(P3546)',
    u'P3547': u'AFL_Tables_ID_(P3547)',
    u'P3548': u'Australian_Business_Number_(P3548)',
    u'P3549': u'Australian_Company_Number_(P3549)',
    u'P3550': u'Australian_Register_of_Therapeutic_Goods_ID_(P3550)',
    u'P3551': u'Australian_Registered_Body_Number_(P3551)',
    u'P3552': u'Australian_Registered_Scheme_Number_(P3552)',
    u'P3553': u'Zhihu_topic_ID_(P3553)',
    u'P3554': u'World_Series_of_Poker_ID_(P3554)',
    u'P3555': u'World_Guide_to_Covered_Bridges_ID_(P3555)',
    u'P3556': u'World_Curling_Tour_ID_(P3556)',
    u'P3557': u'World_Curling_Federation_ID_(P3557)',
    u'P3558': u'USCF_ID_(P3558)',
    u'P3559': u'maximum_size_or_capacity_(P3559)',
    u'P3560': u'College_Football_Data_Warehouse_ID_(P3560)',
    u'P3561': u'pro-football-reference_ID_(P3561)',
    u'P3562': u'Admiralty_number_(P3562)',
    u'P3563': u'NGA_Lighthouse_ID_(P3563)',
    u'P3564': u'Global_Poker_Index_ID_(P3564)',
    u'P3565': u'J.League_player_ID_(P3565)',
    u'P3566': u'Just_Sports_Stats_ID_(P3566)',
    u'P3567': u'HHOF.com_NHL_player_ID_(P3567)',
    u'P3568': u'OWGR_ID_(P3568)',
    u'P3569': u'Cultureel_Woordenboek_identifier_(P3569)',
    u'P3570': u'European_Case_Law_id_(P3570)',
    u'P3571': u'ESPN.com_MLB_player_ID_(P3571)',
    u'P3572': u'ESPNcricinfo_playing_ground_ID_(P3572)',
    u'P3573': u'European_Handball_Federation_ID_(P3573)',
    u'P3574': u'Fangraphs_ID_(P3574)',
    u'P3575': u'data_size_(P3575)',
    u'P3576': u'TLG_author_ID_(P3576)',
    u'P3577': u'Croatian_Football_Federation_player_ID_(P3577)',
    u'P3578': u'autologous_cell_line_(P3578)',
    u'P3579': u'Sina_Weibo_ID_(P3579)',
    u'P3580': u'SIPCA_code_(P3580)',
    u'P3581': u'Pro_Football_Hall_of_Fame_ID_(P3581)',
    u'P3582': u'SunshineTour_ID_(P3582)',
    u'P3583': u'Surfline_ID_(P3583)',
    u'P3584': u'ITRA_runner_ID_(P3584)',
    u'P3585': u'UltraSignup_runner_ID_(P3585)',
    u'P3586': u'CricketArchive_playing_ground_ID_(P3586)',
    u'P3587': u'Common_Vulnerabilities_and_Exposures_ID_(P3587)',
    u'P3588': u'WNBA_player_ID_(P3588)',
    u'P3589': u'GCD_series_ID_(P3589)',
    u'P3590': u'Relations_Ontology_ID_(P3590)',
    u'P3591': u'WCSPF_ID_(P3591)',
    u'P3592': u'Saros_cycle_of_eclipse_(P3592)',
    u'P3593': u'AFI_Catalog_of_Feature_Films_ID_(P3593)',
    u'P3594': u'Araneae_Spider_ID_(P3594)',
    u'P3595': u'BLF_article_ID_(P3595)',
    u'P3596': u'Danish_ancient_monument_ID_(P3596)',
    u'P3597': u'F-Droid_package_(P3597)',
    u'P3598': u'Hockey-reference_player_ID_(P3598)',
    u'P3599': u'archival_creator_authority_record_at_the_Archives_nationales_(P3599)',
    u'P3600': u'protected_heritage_site_in_Brussels_ID_(P3600)',
    u'P3601': u'MarineTraffic_Lighthouse_ID_(P3601)',
    u'P3602': u'candidacy_in_election_(P3602)',
    u'P3603': u'Minneapolis_Institute_of_Art_constituent_ID_(P3603)',
    u'P3604': u'ITU_triathlete_ID_(P3604)',
    u'P3605': u'90minut_player_ID_(P3605)',
    u'P3606': u'BOLD_Systems_taxon_ID_(P3606)',
    u'P3607': u'Booking.com_hotel_ID_(P3607)',
    u'P3608': u'EU_VAT_number_(P3608)',
    u'P3609': u'Recreation.gov_area_ID_(P3609)',
    u'P3610': u'fare_zone_(P3610)',
    u'P3611': u'Borden_Code_(P3611)',
    u'P3612': u'Le_Monde_diplomatique_subject_ID_(P3612)',
    u'P3613': u'Naturvårdsregistret_ID_(P3613)',
    u'P3614': u'DigDag_ID_(P3614)',
    u'P3615': u'Vision_of_Britain_unit_ID_(P3615)',
    u'P3616': u'Vision_of_Britain_place_ID_(P3616)',
    u'P3618': u'base_salary_(P3618)',
    u'P3619': u'ski-db.com_skier_ID_(P3619)',
    u'P3620': u'BWFbadminton.com_player_ID_(P3620)',
    u'P3621': u'Darts_Database_player_ID_(P3621)',
    u'P3622': u'rusbandy_player_ID_(P3622)',
    u'P3623': u'BadmintonLink_player_ID_(P3623)',
    u'P3624': u'CWE_ID_(P3624)',
    u'P3625': u'Kirshenbaum_code_(P3625)',
    u'P3626': u'Australian_Antarctic_Gazetteer_ID_(P3626)',
    u'P3627': u'Historical_Gazetteer_of_England\'s_Place_Names_ID_(P3627)',
    u'P3628': u'British_History_Online_VCH_ID_(P3628)',
    u'P3629': u'stated_age_at_event_(P3629)',
    u'P3630': u'Babelio_author_ID_(P3630)',
    u'P3631': u'Babelio_work_ID_(P3631)',
    u'P3632': u'British_Museum_thesaurus_ID_(P3632)',
    u'P3633': u'British_Museum_place_ID_(P3633)',
    u'P3634': u'The_Met_object_ID_(P3634)',
    u'P3635': u'Peakbagger_area_ID_(P3635)',
    u'P3636': u'PDB_ligand_ID_(P3636)',
    u'P3637': u'European_Medicines_Agency_product_number_(P3637)',
    u'P3638': u'Oorlogsmonument_ID_(P3638)',
    u'P3639': u'KEPN_ID_(P3639)',
    u'P3640': u'National_Drug_Code_(P3640)',
    u'P3641': u'Uniform_Type_Identifier_(P3641)',
    u'P3642': u'ARCHON_code_(P3642)',
    u'P3643': u'significant_environmental_impact_(P3643)',
    u'P3644': u'FFR_player_ID_(P3644)',
    u'P3645': u'All_Blacks_player_ID_(P3645)',
    u'P3646': u'Basketball_Hall_of_Fame_ID_(P3646)',
    u'P3647': u'NBA_player_ID_(P3647)',
    u'P3648': u'NatureServe_conservation_status_(P3648)',
    u'P3650': u'JMK_film_rating_(P3650)',
    u'P3651': u'American_Hockey_League_player_ID_(P3651)',
    u'P3652': u'Kontinental_Hockey_League_player_ID_(P3652)',
    u'P3653': u'ARRS_runner_ID_(P3653)',
    u'P3654': u'Chess.com_player_ID_(P3654)',
    u'P3655': u'BDFutbol_player_ID_(P3655)',
    u'P3656': u'Cross-tables.com_Scrabble_player_ID_(P3656)',
    u'P3657': u'Croatian_Football_Statistics_ID_(P3657)',
    u'P3658': u'DZFoot.com_player_ID_(P3658)',
    u'P3659': u'Estonian_Football_Association_player_ID_(P3659)',
    u'P3660': u'FootballFacts.ru_player_ID_(P3660)',
    u'P3661': u'ForaDeJogo_manager_ID_(P3661)',
    u'P3662': u'Football_Federation_of_Ukraine_player_ID_(P3662)',
    u'P3663': u'Fotbal_DNES_player_ID_(P3663)',
    u'P3664': u'Futsal_Planet_player_ID_(P3664)',
    u'P3665': u'L\'Équipe_football_player_ID_(P3665)',
    u'P3666': u'EPCR_player_ID_(P3666)',
    u'P3667': u'International_Weightlifting_Federation_ID_(P3667)',
    u'P3668': u'Sambafoot_player_ID_(P3668)',
    u'P3669': u'Swimming_Australia_swimmer_ID_(P3669)',
    u'P3670': u'Tennis_Archives_player_ID_(P3670)',
    u'P3671': u'USA_Gymnastics_athlete_ID_(P3671)',
    u'P3672': u'International_Orienteering_Federation_athlete_ID_(P3672)',
    u'P3673': u'film-documentaire.fr_film_ID_(P3673)',
    u'P3674': u'Mutopia_composer_ID_(P3674)',
    u'P3675': u'Hans_Christian_Andersen_Centre_work_ID_(P3675)',
    u'P3676': u'National_Bridge_Inventory_Number_(P3676)',
    u'P3677': u'Wereld_van_Oranje_player_ID_(P3677)',
    u'P3678': u'SA_Rugby_player_ID_(P3678)',
    u'P3679': u'stock_market_index_(P3679)',
    u'P3680': u'statement_supported_by_(P3680)',
    u'P3681': u'ESPNFC.com_player_ID_(P3681)',
    u'P3682': u'Australian_Olympic_Committee_athlete_ID_(P3682)',
    u'P3683': u'LFP_player_ID_(P3683)',
    u'P3684': u'DriverDB_driver_ID_(P3684)',
    u'P3685': u'ESPN.com_NBA_player_ID_(P3685)',
    u'P3686': u'ESPN.com_NFL_player_ID_(P3686)',
    u'P3687': u'ESPN.com_NHL_player_ID_(P3687)',
    u'P3689': u'ICF_canoer_ID_(P3689)',
    u'P3690': u'IFSC_climber_ID_(P3690)',
    u'P3691': u'ISHOF_swimmer_ID_(P3691)',
    u'P3692': u'NCAA_sports_team_ID_(P3692)',
    u'P3693': u'ShorttrackOnLine_speed_skater_ID_(P3693)',
    u'P3694': u'SpeedSkatingNews.info_speed_skater_ID_(P3694)',
    u'P3695': u'SpeedSkatingStats_speed_skater_ID_(P3695)',
    u'P3696': u'SRCBB_player_ID_(P3696)',
    u'P3697': u'SRCFB_player_ID_(P3697)',
    u'P3698': u'Tennis_Australia_player_ID_(P3698)',
    u'P3699': u'Transfermarkt_referee_ID_(P3699)',
    u'P3700': u'NHF_player_ID_(P3700)',
    u'P3701': u'incarnation_of_(P3701)',
    u'P3702': u'Catalogue_of_Illuminated_Manuscripts_ID_(P3702)',
    u'P3703': u'JMDb_person_or_company_ID_(P3703)',
    u'P3704': u'KMDb_film_ID_(P3704)',
    u'P3705': u'RITVA_Program_ID_(P3705)',
    u'P3706': u'RITVA_Person_ID_(P3706)',
    u'P3707': u'Gridabase_glacier_ID_(P3707)',
    u'P3708': u'PhDTree_person_ID_(P3708)',
    u'P3709': u'category_for_value_different_from_Wikidata_(P3709)',
    u'P3710': u'Jewish_Encyclopedia_Daat_ID_(P3710)',
    u'P3711': u'Vanderkrogt.net_Statues_ID_(P3711)',
    u'P3712': u'objective_of_project_or_mission_(P3712)',
    u'P3713': u'category_for_value_not_in_Wikidata_(P3713)',
    u'P3714': u'Recreation.gov_facility_ID_(P3714)',
    u'P3715': u'NISH_Hall_of_Fame_ID_(P3715)',
    u'P3716': u'social_classification_(P3716)',
    u'P3717': u'MEROPS_enzyme_ID_(P3717)',
    u'P3718': u'NCMEC_person_ID_(P3718)',
    u'P3719': u'regulated_by_(P3719)',
    u'P3720': u'GPnotebook_ID_(P3720)',
    u'P3721': u'public_key_fingerprint_(P3721)',
    u'P3722': u'Commons_maps_category_(P3722)',
    u'P3723': u'USCG_Lighthouse_ID_(P3723)',
    u'P3724': u'USHMM_Holocaust_Encyclopedia_ID_(P3724)',
    u'P3725': u'CEV_player_ID_(P3725)',
    u'P3726': u'Eu-football.info_player_ID_(P3726)',
    u'P3727': u'Serbia_municipality_ID_(P3727)',
    u'P3728': u'New_Zealand_Heritage_List_number_(P3728)',
    u'P3729': u'next_lower_rank_(P3729)',
    u'P3730': u'next_higher_rank_(P3730)',
    u'P3731': u'Serbia_cadastral_municipality_ID_(P3731)',
    u'P3732': u'PhilPapers_record_(P3732)',
    u'P3733': u'MOOMA_artist_ID_(P3733)',
    u'P3734': u'category_for_value_same_as_Wikidata_(P3734)',
    u'P3735': u'Yad_Vashem_Encyclopedia_of_the_Ghettos_ID_(P3735)',
    u'P3736': u'Eurovision_Song_Contest_song_ID_(P3736)',
    u'P3737': u'maximum_wavelength_of_sensitivity_(P3737)',
    u'P3738': u'minimum_wavelength_of_sensitivity_(P3738)',
    u'P3739': u'inflorescence_(P3739)',
    u'P3740': u'number_of_works_(P3740)',
    u'P3741': u'seed_dispersal_(P3741)',
    u'P3742': u'FIH_player_ID_(P3742)',
    u'P3743': u'ITU/ISO/IEC_object_identifier_(P3743)',
    u'P3744': u'number_of_subscribers_(P3744)',
    u'P3745': u'World_Rugby_Sevens_Series_ID_(P3745)',
    u'P3746': u'Wildflowers_of_Israel_ID_(P3746)',
    u'P3747': u'SSRN_author_ID_(P3747)',
    u'P3748': u'Israel_Football_Association_national_team_player_ID_(P3748)',
    u'P3749': u'Google_Maps_CID_(P3749)',
    u'P3750': u'KMDb_documentary_ID_(P3750)',
    u'P3751': u'Shoftim_BeIsrael_judge_ID_(P3751)',
    u'P3752': u'worst-case_performance_(P3752)',
    u'P3753': u'best-case_performance_(P3753)',
    u'P3754': u'average_performance_(P3754)',
    u'P3755': u'worst-case_space_complexity_(P3755)',
    u'P3756': u'best-case_space_complexity_(P3756)',
    u'P3757': u'average_space_complexity_(P3757)',
    u'P3758': u'DOCOMOMO_Ibérico_ID_(P3758)',
    u'P3759': u'SAHRA_heritage_site_ID_(P3759)',
    u'P3760': u'Iditarod_musher_ID_(P3760)',
    u'P3761': u'IPv4_routing_prefix_(P3761)',
    u'P3762': u'openMLOL_author_ID_(P3762)',
    u'P3763': u'MIMO_instrument_ID_(P3763)',
    u'P3764': u'pole_position_(P3764)',
    u'P3765': u'All-Athletics.com_ID_(P3765)',
    u'P3766': u'European_Athletics_ID_(P3766)',
    u'P3767': u'FFA_ID_(P3767)',
    u'P3768': u'Medieval_Libraries_of_Great_Britain_ID_(P3768)',
    u'P3769': u'It\'s_Rugby_ID_(P3769)',
    u'P3770': u'PeakFinder_ID_(P3770)',
    u'P3771': u'activator_of_(P3771)',
    u'P3772': u'agonist_of_(P3772)',
    u'P3773': u'antagonist_of_(P3773)',
    u'P3774': u'blocker_of_(P3774)',
    u'P3775': u'disrupting_agent_for_(P3775)',
    u'P3776': u'inhibitor_of_(P3776)',
    u'P3777': u'antisense_inhibitor_of_(P3777)',
    u'P3778': u'positive_allosteric_modulator_of_(P3778)',
    u'P3779': u'negative_allosteric_modulator_of_(P3779)',
    u'P3780': u'active_ingredient_in_(P3780)',
    u'P3781': u'has_active_ingredient_(P3781)',
    u'P3782': u'Artnet_artist_ID_(P3782)',
    u'P3783': u'Christie\'s_work_ID_(P3783)',
    u'P3784': u'CiteSeerX_article_ID_(P3784)',
    u'P3785': u'danskefilm_film_ID_(P3785)',
    u'P3786': u'danskefilm_person_ID_(P3786)',
    u'P3787': u'danskefilm_silent_film_ID_(P3787)',
    u'P3788': u'BNA_authority_ID_(P3788)',
    u'P3789': u'Telegram_username_(P3789)',
    u'P3790': u'AnimeCons.com_ID_(P3790)',
    u'P3791': u'Art_Renewal_Center_ID_(P3791)',
    u'P3792': u'rate_of_fire_(P3792)',
    u'P3793': u'IPv6_routing_prefix_(P3793)',
    u'P3794': u'Dictionary_of_Sydney_ID_(P3794)',
    u'P3795': u'Flora_of_Israel_plant_ID_(P3795)',
    u'P3796': u'Bureau_of_Meteorology_station_ID_(P3796)',
    u'P3797': u'autonomous_system_number_(P3797)',
    u'P3798': u'Star_Wars_Databank_ID_(P3798)',
    u'P3799': u'Safsal_player_ID_(P3799)',
    u'P3800': u'Safsal_coach_ID_(P3800)',
    u'P3801': u'INEGI_municipality_ID_(P3801)',
    u'P3802': u'Launchpad.net_project_ID_(P3802)',
    u'P3803': u'original_film_format_(P3803)',
    u'P3804': u'TV_Guide_show_ID_(P3804)',
    u'P3805': u'Tax-exempt_heritage_asset_ID_(P3805)',
    u'P3806': u'Mapa_place_ID_(P3806)',
    u'P3807': u'S2A3_Biographical_Database_ID_(P3807)',
    u'P3808': u'The_Numbers_movie_ID_(P3808)',
    u'P3809': u'YerelNET_district_ID_(P3809)',
    u'P3810': u'Parks.it_ID_(P3810)',
    u'P3811': u'Evidence_&amp;_Conclusion_Ontology_ID_(P3811)',
    u'P3812': u'Elle.fr_person_ID_(P3812)',
    u'P3813': u'Bivouac.com_pass_ID_(P3813)',
    u'P3814': u'BoF_person_ID_(P3814)',
    u'P3815': u'volcano_observatory_(P3815)',
    u'P3816': u'film_script_(P3816)',
    u'P3817': u'FI_WarSampo_person_ID_(P3817)',
    u'P3818': u'KMRB_film_rating_(P3818)',
    u'P3819': u'FI_WarSampo_army_unit_ID_(P3819)',
    u'P3820': u'Flanders_Arts_Institute_venue_ID_(P3820)',
    u'P3821': u'Bangla_Movie_Database_ID_(P3821)',
    u'P3822': u'rules_for_classification_(P3822)',
    u'P3823': u'Ethnologue_language_status_(P3823)',
    u'P3824': u'VTJ-PRT_building_ID_(P3824)',
    u'P3825': u'United_States_Statutes_at_Large_citation_(P3825)',
    u'P3826': u'Welsh_Rugby_Union_player_ID_(P3826)',
    u'P3827': u'JSTOR_topic_ID_(P3827)',
    u'P3828': u'wears_(P3828)',
    u'P3829': u'Publons_author_ID_(P3829)',
    u'P3830': u'CueTracker_player_ID_(P3830)',
    u'P3831': u'object_has_role_(P3831)',
    u'P3832': u'Europeana_Fashion_Vocabulary_ID_(P3832)',
    u'P3833': u'diaspora_(P3833)',
    u'P3834': u'RTC_film_rating_(P3834)',
    u'P3835': u'Mendeley_person_ID_(P3835)',
    u'P3836': u'Pinterest_username_(P3836)',
    u'P3837': u'United_States_Public_Law_(P3837)',
    u'P3838': u'Tab4u_song_ID_(P3838)',
    u'P3839': u'Tab4u_artist_ID_(P3839)',
    u'P3840': u'slope_rating_(P3840)',
    u'P3841': u'Human_Phenotype_Ontology_ID_(P3841)',
    u'P3842': u'located_in_present-day_administrative_territorial_entity_(P3842)',
    u'P3843': u'DLV_athlete_ID_(P3843)',
    u'P3844': u'Deutsche_Synchronkartei_film_ID_(P3844)',
    u'P3845': u'TV_Guide_person_ID_(P3845)',
    u'P3846': u'DBC_author_ID_(P3846)',
    u'P3847': u'Open_Library_subject_ID_(P3847)',
    u'P3848': u'Irish_Rugby_Football_Union_ID_(P3848)',
    u'P3849': u'LombardiaBeniCulturali_institution_ID_(P3849)',
    u'P3850': u'LombardiaBeniCulturali_toponym_ID_(P3850)',
    u'P3851': u'cinenacional.com_movie_ID_(P3851)',
    u'P3852': u'FlyBase_Gene_ID_(P3852)',
    u'P3853': u'Rat_Genome_Database_ID_(P3853)',
    u'P3854': u'Soundtrack_Collector_ID_(P3854)',
    u'P3855': u'LombardiaBeniCulturali_artwork_ID_(P3855)',
    u'P3856': u'Quebec_municipalities_geographical_code_(P3856)',
    u'P3857': u'cinenacional.com_person_ID_(P3857)',
    u'P3858': u'route_diagram_(P3858)',
    u'P3859': u'Environment_Ontology_ID_(P3859)',
    u'P3860': u'Wormbase_Gene_ID_(P3860)',
    u'P3861': u'iTunes_app_ID_(P3861)',
    u'P3862': u'MyDramaList_name_ID_(P3862)',
    u'P3863': u'Italian_Navy_Lighthouses_and_Beacons_ID_(P3863)',
    u'P3864': u'suicide_rate_(P3864)',
    u'P3865': u'ype_of_reference_(P3865)',
    u'P3866': u'LAWA_waterbody_ID_(P3866)',
    u'P3867': u'Israel_Chess_Federation_player_ID_(P3867)',
    u'P3868': u'MyDramaList_title_ID_(P3868)',
    u'P3869': u'IAFD_female_performer_ID_(P3869)',
    u'P3870': u'ZFIN_Gene_ID_(P3870)',
    u'P3871': u'ributary_orientation_(P3871)',
    u'P3872': u'patronage_(P3872)',
    u'P3874': u'Justia_Patents_inventor_ID_(P3874)',
    u'P3875': u'Justia_Patents_company_ID_(P3875)',
    u'P3876': u'category_for_alumni_of_educational_institution_(P3876)',
    u'P3877': u'HappyCow_restaurant_ID_(P3877)',
    u'P3878': u'Soundex_(P3878)',
    u'P3879': u'Cologne_phonetics_(P3879)',
    u'P3880': u'Caverphone_(P3880)',
    u'P3881': u'National_Track_&amp;_Field_Hall_of_Fame_athlete_ID_(P3881)',
    u'P3882': u'Tilastopaja_female_athlete_ID_(P3882)',
    u'P3883': u'Red_Bull_athlete_ID_(P3883)',
    u'P3884': u'Tilastopaja_male_athlete_ID_(P3884)',
    u'P3885': u'History_of_Modern_Biomedicine_ID_(P3885)',
    u'P3886': u'number_of_perpetrators_(P3886)',
    u'P3887': u'KVAB_member_ID_(P3887)',
    u'P3888': u'Boijmans_artist_ID_(P3888)',
    u'P3889': u'Swiss_Federal_Archives_ID_(P3889)',
    u'P3890': u'MetaboLights_Compound_ID_(P3890)',
    u'P3891': u'observing_time_available_(P3891)',
    u'P3892': u'PictoRight_ID-Droit_de_suite_(P3892)',
    u'P3893': u'public_domain_date_(P3893)',
    u'P3894': u'OSTI_article_ID_(P3894)',
    u'P3895': u'INAO_product_ID_(P3895)',
    u'P3896': u'geoshape_(P3896)',
    u'P3897': u'Ladies_European_Tour_ID_(P3897)',
    u'P3898': u'Hotels.com_hotel_ID_(P3898)',
    u'P3899': u'Medium_username_(P3899)',
    u'P3900': u'CONICET_person_ID_(P3900)',
    u'P3901': u'ADAGP_artist_ID_(P3901)',
    u'P3902': u'had_as_last_meal_(P3902)',
    u'P3903': u'column_(P3903)',
    u'P3904': u'VIVC_grape_variety_ID_(P3904)',
    u'P3905': u'GINCO_ID_(P3905)',
    u'P3906': u'Ishim_ID_(P3906)',
    u'P3907': u'LoJ_peak_ID_(P3907)',
    u'P3908': u'Reprezentacija_ID_(P3908)',
    u'P3909': u'last_words_(P3909)',
    u'P3910': u'Bollywood_Hungama_person_ID_(P3910)',
    u'P3911': u'STW_Thesaurus_for_Economics_ID_(P3911)',
    u'P3912': u'newspaper_format_(P3912)',
    u'P3913': u'MobyGames_developer_ID_(P3913)',
    u'P3914': u'GuideStar_Israel_organization_ID_(P3914)',
    u'P3915': u'Athletics_Australia_athlete_ID_(P3915)',
    u'P3916': u'UNESCO_Thesaurus_ID_(P3916)',
    u'P3917': u'IPA_number_order_(P3917)',
    u'P3918': u'Répertoire_national_des_associations_identifier_(P3918)',
    u'P3919': u'contributed_to_published_work_(P3919)',
    u'P3920': u'Canadian_Coastguard_Lighthouse_ID_(P3920)',
    u'P3921': u'Wikidata_SPARQL_query_equivalent_(P3921)',
    u'P3922': u'light_sector_(P3922)',
    u'P3923': u'Diamond_League_athlete_ID_(P3923)',
    u'P3924': u'Track_and_Field_Statistics_female_athlete_ID_(P3924)',
    u'P3925': u'Track_and_Field_Statistics_male_athlete_ID_(P3925)',
    u'P3926': u'USATF_athlete_ID_(P3926)',
    u'P3927': u'eWRC-results.com_racer_ID_(P3927)',
    u'P3928': u'MotoGP_racer_ID_(P3928)',
    u'P3929': u'V&amp;A_art_work_ID_(P3929)',
    u'P3930': u'Rallye-info.com_driver_or_co-driver_ID_(P3930)',
    u'P3931': u'copyright_owner_(P3931)',
    u'P3932': u'Digital_Valencian_Library_author_ID_(P3932)',
    u'P3933': u'Cinema_ID_(P3933)',
    u'P3934': u'face_value_(P3934)',
    u'P3935': u'Chamber_of_Deputies_of_Italy_storia_ID_(P3935)',
    u'P3936': u'NFF_person_profile_ID_(P3936)',
    u'P3937': u'Reactome_ID_(P3937)',
    u'P3938': u'named_by_(P3938)',
    u'P3939': u'ESTC_citation_number_(P3939)',
    u'P3940': u'OlimpBase_Chess_Olympiad_player_ID_(P3940)',
    u'P3941': u'Israel_Antiquities_Authority_ID_(P3941)',
    u'P3942': u'Bmx-results.com_rider_ID_(P3942)',
    u'P3943': u'Tumblr_ID_(P3943)',
    u'P3944': u'Old_Bailey_Proceedings_ID_(P3944)',
    u'P3945': u'RANM_member_ID_(P3945)',
    u'P3946': u'Directorio_Grierson_ID_(P3946)',
    u'P3948': u'MLL_player_ID_(P3948)',
    u'P3949': u'Juwra.com_ID_(P3949)',
    u'P3950': u'narrower_external_class_(P3950)',
    u'P3951': u'BioRxiv_ID_(P3951)',
    u'P3952': u'Stereo_Ve_Mono_artist_ID_(P3952)',
    u'P3953': u'ALPG_golfer_ID_(P3953)',
    u'P3954': u'Italian_Senate_ID_(P3954)',
    u'P3955': u'NLL_player_ID_(P3955)',
    u'P3956': u'National_Academy_of_Medicine_(France)_MemberID_(P3956)',
    u'P3957': u'RealGM_basketball_player_ID_(P3957)',
    u'P3958': u'ENARD_athlete_ID_(P3958)',
    u'P3959': u'NNL_work_ID_(P3959)',
    u'P3960': u'Base_biographique_AUTOR_ID_(P3960)',
    u'P3961': u'Unifrance_film_ID_(P3961)',
    u'P3962': u'Global_Trade_Item_Number_(P3962)',
    u'P3963': u'Clochers_de_France_ID_(P3963)',
    u'P3964': u'BDCYL_authority_ID_(P3964)',
    u'P3965': u'Bridgeman_artist_ID_(P3965)',
    u'P3966': u'programming_paradigm_(P3966)',
    u'P3967': u'final_event_(P3967)',
    u'P3968': u'CETS_number_(P3968)',
    u'P3969': u'signed_form_(P3969)',
    u'P3970': u'channel_number_(P3970)',
    u'P3971': u'PASE_Domesday_person_ID_(P3971)',
    u'P3972': u'PASE_Domesday_place_(P3972)',
    u'P3973': u'PIM_authority_ID_(P3973)',
    u'P3974': u'nature_preserve_in_North_Rhine-Westphalia_ID_(P3974)',
    u'P3975': u'secretary_general_(P3975)',
    u'P3976': u'BVMC_work_ID_(P3976)',
    u'P3977': u'Songkick_venue_ID_(P3977)',
    u'P3978': u'IECIC_2015_ID_(P3978)',
    u'P3979': u'Unifrance_company_ID_(P3979)',
    u'P3980': u'Unifrance_person_ID_(P3980)',
    u'P3981': u'Misjonsarkiv_person_ID_(P3981)',
    u'P3982': u'TA98_Latin_term_(P3982)',
    u'P3983': u'sports_league_level_(P3983)',
    u'P3984': u'subreddit_(P3984)',
    u'P3985': u'supports_programming_language_(P3985)',
    u'P3986': u'Sequence_Ontology_ID_(P3986)',
    u'P3987': u'SHARE_Catalogue_author_ID_(P3987)',
    u'P3988': u'National_Library_Board_Singapore_ID_(P3988)',
    u'P3989': u'members_have_occupation_(P3989)',
    u'P3990': u'BG_EKATTE_place_ID_(P3990)',
    u'P3991': u'Austrian_Textbook_ID_(P3991)',
    u'P3992': u'SSB_urban_settlement_number_(P3992)',
    u'P3993': u'Vitaskrá_ID_(P3993)',
    u'P3994': u'racon_signal_(P3994)',
    u'P3995': u'Filmweb.pl_ID_(P3995)',
    u'P3996': u'Bait_La_Zemer_Ha-Ivri_song_ID_(P3996)',
    u'P3997': u'Bait_La_Zemer_Ha-Ivri_artist_ID_(P3997)',
    u'P3998': u'Censo-Guía_archive_ID_(P3998)',
    u'P3999': u'date_of_official_closure_(P3999)',
    u'P4000': u'has_fruit_type_(P4000)',
    u'P4001': u'Latvian_Protected_Nature_Territory_URL_(P4001)',
    u'P4002': u'WFD_Ecological_status_(P4002)',
    u'P4003': u'official_Facebook_page_(P4003)',
    u'P4004': u'shield_image_(P4004)',
    u'P4005': u'Bavarikon_ID_(P4005)',
    u'P4006': u'overrules_(P4006)',
    u'P4007': u'DBS_ID_(P4007)',
    u'P4008': u'Early_Aviators_people_ID_(P4008)',
    u'P4009': u'RKY_national_built_heritage_environment_ID_(P4009)',
    u'P4010': u'GDP_(PPP)_(P4010)',
    u'P4011': u'Semantic_Scholar_paper_ID_(P4011)',
    u'P4012': u'Semantic_Scholar_author_ID_(P4012)',
    u'P4013': u'Giphy_username_(P4013)',
    u'P4014': u'Australian_Statistical_Geography_2011_ID_(P4014)',
    u'P4015': u'Vimeo_username_(P4015)',
    u'P4016': u'SlideShare_username_(P4016)',
    u'P4017': u'Ustream_username_(P4017)',
    u'P4018': u'The_Arabidopsis_Information_Resource_Accession_(P4018)',
    u'P4019': u'USL_player_ID_(P4019)',
    u'P4020': u'dimension_(P4020)',
    u'P4021': u'danskefilm_animated_film_ID_(P4021)',
    u'P4022': u'danskefilm_TV_series_ID_(P4022)',
    u'P4023': u'DFB_datacenter_player_ID_(P4023)',
    u'P4024': u'ADW_taxon_ID_(P4024)',
    u'P4025': u'Pinakothek_artist_ID_(P4025)',
    u'P4026': u'Cullum_number_(P4026)',
    u'P4027': u'National_Discography_of_Italian_Song_work_ID_(P4027)',
    u'P4028': u'Google_Scholar_paper_ID_(P4028)',
    u'P4029': u'Latvian_Protected_Nature_Territory_ID_(P4029)',
    u'P4030': u'PLU_Code_(P4030)',
    u'P4031': u'Hungarian_NGO_ID_(P4031)',
    u'P4032': u'reviewed_by_(P4032)',
    u'P4033': u'Mastodon_address_(P4033)',
    u'P4034': u'Shironet_artist_ID_(P4034)',
    u'P4035': u'Shironet_song_ID_(P4035)',
    u'P4036': u'field_of_view_(P4036)',
    u'P4037': u'South_Australian_Heritage_Register_Database_ID_(P4037)',
    u'P4038': u'Danish_List_of_Lights_and_Fog_signals_ID_(P4038)',
    u'P4040': u'Rock.com.ar_artist_ID_(P4040)',
    u'P4041': u'Rock.com.ar_album_ID_(P4041)',
    u'P4042': u'ESBL_athlete_ID_(P4042)',
    u'P4043': u'emulates_(P4043)',
    u'P4044': u'herapeutic_area_(P4044)',
    u'P4045': u'Sandbox-Tabular_data_(P4045)',
    u'P4046': u'SIMC_place_ID_(P4046)',
    u'P4047': u'Sandbox-Geographic_shape_(P4047)',
    u'P4048': u'J.League_manager_ID_(P4048)',
    u'P4050': u'EspritBleu_athlete_ID_(P4050)',
    u'P4051': u'Ukrainian_regulations_base_ID_(P4051)',
    u'P4052': u'Academia.edu_institutional_ID_(P4052)',
    u'P4053': u'Deutsche_Olympiamannschaft_athlete_ID_(P4053)',
    u'P4054': u'Canadian_Olympic_Committee_athlete_ID_(P4054)',
    u'P4055': u'Norwegian_List_of_Lights_ID_(P4055)',
    u'P4056': u'New_Zealand_Olympic_Committee_athlete_ID_(P4056)',
    u'P4057': u'Irish_Sites_and_Monuments_Record_ID_(P4057)',
    u'P4058': u'FINESS_medical_facility_ID_(P4058)',
    u'P4059': u'Irish_National_Monument_ID_(P4059)',
    u'P4060': u'Brazilian_Olympic_Committee_athlete_ID_(P4060)',
    u'P4061': u'LTOK_athlete_ID_(P4061)',
    u'P4062': u'ČOV_athlete_ID_(P4062)',
    u'P4063': u'United_States_Olympic_Committee_athlete_ID_(P4063)',
    u'P4065': u'COA_athlete_ID_(P4065)',
    u'P4066': u'MOB_athlete_ID_(P4066)',
    u'P4067': u'COSR_athlete_ID_(P4067)',
    u'P4068': u'CIS_Chinese_Athletes_Database_ID_(P4068)',
    u'P4069': u'CONI_athlete_ID_(P4069)',
    u'P4070': u'identifier_shared_with_(P4070)',
    u'P4071': u'Zemereshet_artist_ID_(P4071)',
    u'P4072': u'Zemereshet_song_ID_(P4072)',
    u'P4073': u'Wikia_wiki_ID_(P4073)',
    u'P4074': u'FFN_swimmer_ID_(P4074)',
    u'P4075': u'Czech_Monument_Catalogue_Number_(P4075)',
    u'P4076': u'WorldSBK.com_racer_identifier_(P4076)',
    u'P4077': u'Pizmonet_ID_(P4077)',
    u'P4078': u'SKIP_code_(P4078)',
    u'P4079': u'Theatres_Trust_Database_ID_(P4079)',
    u'P4080': u'number_of_houses_(P4080)',
    u'P4081': u'BHL_creator_ID_(P4081)',
    u'P4082': u'image_captured_with_(P4082)',
    u'P4083': u'NPS_unit_ID_(P4083)',
    u'P4084': u'MyAnimeList_people_ID_(P4084)',
    u'P4085': u'MyAnimeList_character_ID_(P4085)',
    u'P4086': u'MyAnimeList_anime_ID_(P4086)',
    u'P4087': u'MyAnimeList_manga_ID_(P4087)',
    u'P4088': u'Irish_National_Inventory_of_Architectural_Heritage_ID_(P4088)',
    u'P4089': u'Global_Terrorism_Database_ID_(P4089)',
    u'P4090': u'Biodiversity_Repository_ID_(P4090)',
    u'P4091': u'Irish_Grid_Reference_(P4091)',
    u'P4092': u'checksum_(P4092)',
    u'P4093': u'Australian_Statistical_Geography_2016_ID_(P4093)',
    u'P4094': u'Australian_Standard_Geographic_Classification_2006_ID_(P4094)',
    u'P4095': u'Principal_Galaxies_Catalogue_ID_(P4095)',
    u'P4096': u'RePEc_institute_ID_(P4096)',
    u'P4097': u'MuseScore_ID_(P4097)',
    u'P4098': u'BVMC_place_id_(P4098)',
    u'P4099': u'metrically_compatible_typeface_(P4099)',
    u'P4100': u'parliamentary_group_(P4100)',
    u'P4101': u'dissertation_submitted_to_(P4101)',
    u'P4102': u'Atlas_of_Hillforts_ID_(P4102)',
    u'P4103': u'assets_under_management_(P4103)',
    u'P4104': u'Carnegie_Hall_agent_ID_(P4104)',
    u'P4105': u'EGF_rating_(P4105)',
    u'P4106': u'Finnish_archaeological_heritage_ID_(P4106)',
    u'P4107': u'Framalibre_ID_(P4107)',
    u'P4108': u'Gedbas_genealogy_person_ID_(P4108)',
    u'P4109': u'URN-NBN_(P4109)',
    u'P4110': u'Crunchyroll_ID_(P4110)',
    u'P4111': u'danskefilm_TV_Christmas_calendar_(P4111)',
    u'P4112': u'danskfilmogtv_person_(P4112)',
    u'P4113': u'FRED_time-series_ID_(P4113)',
    u'P4114': u'ADK_member_ID_(P4114)',
    u'P4115': u'INSPIRE_ID_(P4115)',
    u'P4116': u'JewAge_person_ID_(P4116)',
    u'P4117': u'National_Record_of_the_Historic_Environment_ID_(P4117)',
    u'P4118': u'NLS-FI_Geographic_Name_ID_(P4118)',
    u'P4119': u'NLS_Geographic_Names_Place_ID_(P4119)',
    u'P4120': u'Ontario_Heritage_Act_Register_ID_(P4120)',
    u'P4121': u'openAIP_ID_(P4121)',
    u'P4122': u'PalDat_plant_ID_(P4122)',
    u'P4123': u'French_National_Assembly_ID_(P4123)',
    u'P4124': u'Who\'s_Who_in_France_biography_ID_(P4124)',
    u'P4125': u'Titan_ID_(P4125)',
    u'P4126': u'ESEC_person_ID_(P4126)',
    u'P4127': u'EmbassyPages.com_ID_(P4127)',
    u'P4128': u'NPSN_Indonesian_school_ID_(P4128)',
    u'P4129': u'Cinema_Treasures_ID_(P4129)',
    u'P4130': u'USHMM_person_ID_(P4130)',
    u'P4131': u'annual_energy_output_(P4131)',
    u'P4132': u'linguistic_typology_(P4132)',
    u'P4133': u'Patrimonioculturale-ER_ID_(P4133)',
    u'P4135': u'maximum_age_(P4135)',
    u'P4136': u'WIGOS_station_ID_(P4136)',
    u'P4137': u'muzzle_velocity_(P4137)',
    u'P4138': u'Treasury_of_Lives_ID_(P4138)',
    u'P4139': u'National_Assembly_of_Nigeria_ID_(P4139)',
    u'P4140': u'energy_storage_capacity_(P4140)',
    u'P4141': u'Gatehouse_Gazetteer_place_ID_(P4141)',
    u'P4142': u'RIWAQ_Registry_of_Historic_Buildings_in_Palestine_ID_(P4142)',
    u'P4143': u'Finnish_List_of_Lights_ID_(P4143)',
    u'P4144': u'Atheneum_artwork_ID_(P4144)',
    u'P4145': u'Atheneum_person_ID_(P4145)',
    u'P4146': u'Atheneum_museum_ID_(P4146)',
    u'P4147': u'conjugate_acid_(P4147)',
    u'P4149': u'conjugate_base_(P4149)',
    u'P4150': u'weather_history_(P4150)',
    u'P4151': u'game_mechanics_(P4151)',
    u'P4152': u'file_format_identification_pattern_(P4152)',
    u'P4153': u'offset_(P4153)',
    u'P4154': u'National_Forest_Foundation_ID_(P4154)',
    u'P4155': u'separator_(P4155)',
    u'P4156': u'Czech_Registration_ID_(P4156)',
    u'P4157': u'MEG_ID_(P4157)',
    u'P4158': u'autores.ar_id_(P4158)',
    u'P4159': u'WeRelate_person_ID_(P4159)',
    u'P4160': u'Michelin_Restaurants_ID_(P4160)',
    u'P4161': u'Michelin_Voyages_ID_(P4161)',
    u'P4162': u'AUR_package_(P4162)',
    u'P4163': u'magnification_(P4163)',
    u'P4164': u'National_Baseball_Hall_of_Fame_and_Museum_ID_(P4164)',
    u'P4165': u'CODECS_ID_(P4165)',
    u'P4166': u'Georgian_National_Register_of_Monuments_ID_(P4166)',
    u'P4167': u'Dagens_Næringsliv_topic_ID_(P4167)',
    u'P4168': u'IEDB_Epitope_ID_(P4168)',
    u'P4169': u'YCBA_agent_ID_(P4169)',
    u'P4170': u'eParks_unit_ID_(P4170)',
    u'P4171': u'World_Heritage_Tentative_List_ID_(P4171)',
    u'P4172': u'America\'s_Byways_road_ID_(P4172)',
    u'P4173': u'Instagram_location_ID_(P4173)',
    u'P4174': u'Wikimedia_username_(P4174)',
    u'P4175': u'Patreon_ID_(P4175)',
    u'P4176': u'effective_firing_range_(P4176)',
    u'P4177': u'Finnish_National_Gallery_artist_ID_(P4177)',
    u'P4178': u'Beazley_Archive_Pottery_Database_ID_(P4178)',
    u'P4179': u'abular_population_(P4179)',
    u'P4180': u'GujLit_Person_ID_(P4180)',
    u'P4181': u'GujLit_Book_ID_(P4181)',
    u'P4182': u'National_Natural_Landmarks_site_ID_(P4182)',
    u'P4183': u'angle_from_vertical_(P4183)',
    u'P4184': u'slope_(P4184)',
    u'P4185': u'has_attribute_(P4185)',
    u'P4186': u'Australian_Women\'s_Register_ID_(P4186)',
    u'P4187': u'Wylie_transliteration_(P4187)',
    u'P4188': u'Tibetan_pinyin_(P4188)',
    u'P4189': u'THL_Simplified_Phonetic_Transcription_(P4189)',
    u'P4190': u'Rivers.gov_protected_area_ID_(P4190)',
    u'P4191': u'Alljudo_athlete_ID_(P4191)',
    u'P4192': u'LNH_player_ID_(P4192)',
    u'P4193': u'Familypedia_person_ID_(P4193)',
    u'P4194': u'CNPS_ID_(P4194)',
    u'P4195': u'category_for_employees_of_the_organization_(P4195)',
    u'P4196': u'cytogenetic_location_(P4196)',
    u'P4197': u'Graphic_character_set_global_ID_(P4197)',
    u'P4198': u'Google_Play_Music_artist_ID_(P4198)',
    u'P4199': u'Google_Play_Music_album_ID_(P4199)',
    u'P4200': u'Christie\'s_creator_id_(P4200)',
    u'P4201': u'PagesJaunes_ID_(P4201)',
    u'P4202': u'WFD_Chemical_status_(P4202)',
    u'P4203': u'ROARMAP_ID_(P4203)',
    u'P4204': u'The_Times_of_India_topic_ID_(P4204)',
    u'P4206': u'FOIH_person_ID_(P4206)',
    u'P4207': u'Italian_National_Earthquake_Center_ID_(P4207)',
    u'P4208': u'Billboard_artist_ID_(P4208)',
    u'P4209': u'danskfilmogtv_title_ID_(P4209)',
    u'P4210': u'Bashkir_encyclopedia_(Bashkir_version)_ID_(P4210)',
    u'P4211': u'Bashkir_encyclopedia_(Russian_version)_ID_(P4211)',
    u'P4212': u'PACTOLS_thesaurus_ID_(P4212)',
    u'P4213': u'Unicode_hex_codepoint_(P4213)',
    u'P4214': u'highest_observed_lifespan_(P4214)',
    u'P4215': u'nLab_ID_(P4215)',
    u'P4216': u'LNB_Pro_A_player_ID_(P4216)',
    u'P4217': u'UK_Electoral_Commission_ID_(P4217)',
    u'P4218': u'shelf_life_(P4218)',
    u'P4219': u'Tyrolean_Art_Cadastre_inventory_ID_(P4219)',
    u'P4220': u'order_of_battle_(P4220)',
    u'P4221': u'National_Criminal_Justice_ID_(P4221)',
    u'P4222': u'United_Nations_Treaty_Series_Volume_Number_(P4222)',
    u'P4223': u'Enciclopedia_Italiana_ID_(P4223)',
    u'P4224': u'category_contains_(P4224)',
    u'P4225': u'IPA_Braille_(P4225)',
    u'P4226': u'Cyworld_ID_(P4226)',
    u'P4227': u'Indonesian_Small_Islands_Directory_ID_(P4227)',
    u'P4228': u'Encyclopedia_of_Australian_Science_ID_(P4228)',
    u'P4229': u'ICD-10-CM_(P4229)',
    u'P4230': u'Sefaria_ID_(P4230)',
    u'P4231': u'United_Nations_Treaty_Series_Registration_Number_(P4231)',
    u'P4232': u'Figshare_author_ID_(P4232)',
    u'P4233': u'PatientLikeMe_condition_ID_(P4233)',
    u'P4235': u'PatientLikeMe_treatment_ID_(P4235)',
    u'P4236': u'PatientLikeMe_symptom_ID_(P4236)',
    u'P4237': u'Heritage_Building_in_Finland_ID_(P4237)',
    u'P4238': u'webcam_page_URL_(P4238)',
    u'P4239': u'vocalized_name_(P4239)',
    u'P4240': u'regular_expression_syntax_(P4240)',
    u'P4241': u'refine_date_(P4241)',
    u'P4242': u'maximal_rate_of_climb_(P4242)',
    u'P4243': u'candidate_number_(P4243)',
    u'P4244': u'Bavarian_monument_authority_ID_(P4244)',
    u'P4245': u'cultural_heritage_monument_in_Serbia_ID_(P4245)',
    u'P4246': u'LfD-HB_object_ID_(P4246)',
    u'P4247': u'Tagesschau_election_ID_(P4247)',
    u'P4248': u'The_Black_Book_ID_(P4248)',
    u'P4249': u'Vincoli_in_Rete_ID_(P4249)',
    u'P4250': u'defined_daily_dose_(P4250)',
    u'P4251': u'TSE_number_(P4251)',
    u'P4252': u'All-Russian_Mathematical_Portal_ID_(P4252)',
    u'P4253': u'number_of_constituencies_(P4253)',
    u'P4254': u'Banglapedia_(Bengali_version)_ID_(P4254)',
    u'P4255': u'Banglapedia_(English_version)_ID_(P4255)',
    u'P4256': u'Spanish_"Boletín_Oficial_del_Estado"_ID_(P4256)',
    u'P4257': u'National_Museums_of_Japan_e-museum_ID_(P4257)',
    u'P4258': u'Gallica_ID_(P4258)',
    u'P4259': u'Lega_Pallavolo_Serie_A_Femminile_player_ID_(P4259)',
    u'P4260': u'NPB_player_ID_(P4260)',
    u'P4261': u'FFVoile_sailor_ID_(P4261)',
    u'P4262': u'Footoféminin.fr_player_ID_(P4262)',
    u'P4263': u'THW_Kiel_player_ID_(P4263)',
    u'P4264': u'LinkedIn_company_ID_(P4264)',
    u'P4265': u'Reddit_user_name_(P4265)',
    u'P4266': u'Bavarian_geotope_ID_(P4266)',
    u'P4267': u'Landtag_of_Liechtenstein_ID_(P4267)',
    u'P4268': u'half_maximal_inhibitory_concentration_(IC50)_(P4268)',
    u'P4269': u'half_maximal_effective_concentration_(P4269)',
    u'P4270': u'Daum_TV_series_ID_(P4270)',
    u'P4271': u'rating_(P4271)',
    u'P4272': u'DPLA_subject_ID_(P4272)',
    u'P4273': u'New_York_City_Landmarks_Preservation_Commission_ID_(P4273)',
    u'P4274': u'Tunisian_geographic_code_(P4274)',
    u'P4275': u'Japanese_Database_of_National_important_cultural_properties_(P4275)',
    u'P4276': u'Cinémathèque_québécoise_work_identifier_(P4276)',
    u'P4277': u'Daum_movie_ID_(P4277)',
    u'P4278': u'FCI_rider_ID_(P4278)',
    u'P4279': u'hydrological_order_number_(P4279)',
    u'P4280': u'International_Standard_Text_Code_(P4280)',
    u'P4281': u'LNV_player_ID_(P4281)',
    u'P4282': u'LUMIERE_film_ID_(P4282)',
    u'P4283': u'LUMIERE_director_ID_(P4283)',
    u'P4284': u'Nominis_given_name_ID_(P4284)',
    u'P4285': u'Theses.fr_person_ID_(P4285)',
    u'P4286': u'Nominis_saint_ID_(P4286)',
    u'P4287': u'Riigikogu_ID_(P4287)',
    u'P4288': u'UK_National_Fruit_Collection_ID_(P4288)',
    u'P4289': u'LFH_player_ID_(P4289)',
    u'P4290': u'official_app_(P4290)',
    u'P4291': u'panorama_view_(P4291)',
    u'P4292': u'possessed_by_spirit_(P4292)',
    u'P4293': u'PM20_folder_ID_(P4293)',
    u'P4294': u'PROSPERO_ID_(P4294)',
    u'P4295': u'readership_(P4295)',
    u'P4296': u'stellar_rotational_velocity_(P4296)',
    u'P4297': u'MIL-STD-2525_Symbol_Identification_Code_(P4297)',
    u'P4298': u'VBL_player_ID_(P4298)',
    u'P4299': u'Wheelchair_ITF_player_ID_(P4299)',
    u'P4300': u'YouTube_Playlist_ID_(P4300)',
    u'P4301': u'PfaF_id_(P4301)',
    u'P4302': u'KNAU_ID_(P4302)',
    u'P4303': u'Lega_Pallavolo_Serie_A_player_ID_(P4303)',
    u'P4304': u'FOIH_materials_ID_(P4304)',
    u'P4305': u'FOIH_styles_and_cultures_ID_(P4305)',
    u'P4306': u'FOIH_periods_ID_(P4306)',
    u'P4307': u'FOIH_heritage_types_ID_(P4307)',
    u'P4308': u'FOIH_value_types_ID_(P4308)',
    u'P4309': u'FOIH_event_types_ID_(P4309)',
    u'P4310': u'FOIH_decree_types_ID_(P4310)',
    u'P4311': u'FOIH_taxon_ID_(P4311)',
    u'P4312': u'camera_setup_(P4312)',
    u'P4313': u'musée_de_Bretagne_identifiant_collections_ID_(P4313)',
    u'P4314': u'SpeedskatingResults.com_speed_skater_ID_(P4314)',
    u'P4315': u'SwimSwam_ID_(P4315)',
    u'P4316': u'kinship_equivalent_in_SPARQL_at_Wikidata_(P4316)',
    u'P4317': u'GARD_rare_disease_ID_(P4317)',
    u'P4318': u'Scottish_Sports_HoF_athlete_ID_(P4318)',
    u'P4319': u'Elite_Prospects_staff_ID_(P4319)',
    u'P4320': u'mountains_classification_(P4320)',
    u'P4321': u'best_combative_classification_(P4321)',
    u'P4322': u'best_sprinter_classification_(P4322)',
    u'P4323': u'young_rider_classification_(P4323)',
    u'P4324': u'combination_classification_(P4324)',
    u'P4325': u'APA_phoneme_code_(P4325)',
    u'P4326': u'BFI-Filmography_person_ID_(P4326)',
    u'P4327': u'BHL_bibliography_ID_(P4327)',
    u'P4328': u'Brueckenweb_ID_(P4328)',
    u'P4329': u'Wikimedia_template_that_populates_this_category_(P4329)',
    u'P4330': u'contains_(P4330)',
    u'P4331': u'Department_of_Education_and_Skills_roll_number_(P4331)',
    u'P4332': u'Deutsche_Synchronkartei_actor-ID_(P4332)',
    u'P4333': u'GenBank_Assembly_accession_(P4333)',
    u'P4334': u'Heritage_Conservation_District_of_Ontario_ID_(P4334)',
    u'P4335': u'IDESCAT_territorial_code_in_Catalonia_(P4335)',
    u'P4336': u'Lloyd\'s_Register_Ship_ID_(P4336)',
    u'P4337': u'digitised_page_from_Lloyd\'s_Register_of_Ships_from_1930_to_1945_(P4337)',
    u'P4338': u'LOINC_ID_(P4338)',
    u'P4339': u'Radio_Radicale_organization_ID_(P4339)',
    u'P4340': u'PubsHistory_pub_ID_(P4340)',
    u'P4341': u'synodic_period_(P4341)',
    u'P4342': u'Store_norske_leksikon_ID_(P4342)',
    u'P4343': u'WBPLN_author_ID_(P4343)',
    u'P4344': u'QEdu_ID_(P4344)',
    u'P4345': u'excavation_director_(P4345)',
    u'P4346': u'T4T35_megalith_ID_(P4346)',
    u'P4347': u'Florida_Sports_Hall_of_Fame_athlete_ID_(P4347)',
    u'P4348': u'Lotsawa_House_Tibetan_author_ID_(P4348)',
    u'P4349': u'Lotsawa_House_Indian_author_ID_(P4349)',
    u'P4350': u'salinity_(P4350)',
    u'P4351': u'Cravo_Albin_artist_ID_(P4351)',
    u'P4352': u'BillionGraves_cemetery_ID_(P4352)',
    u'P4353': u'nominated_by_(P4353)',
    u'P4354': u'search_formatter_URL_(P4354)',
    u'P4355': u'PROSITE_documentation_ID_(P4355)',
    u'P4356': u'Megalithic_Portal_ID_(P4356)',
    u'P4357': u'Musikverket_person_ID_(P4357)',
    u'P4358': u'Dutch_lost_building_register_ID_(P4358)',
    u'P4359': u'gravsted.dk_ID_(P4359)',
    u'P4360': u'Monumentos_de_São_Paulo_ID_(P4360)',
    u'P4361': u'ExecutedToday_ID_(P4361)',
    u'P4362': u'ASHOF_athlete_ID_(P4362)',
    u'P4363': u'DSMHOF_athlete_ID_(P4363)',
    u'P4364': u'Georgia_Sports_Hall_of_Fame_ID_(P4364)',
    u'P4365': u'Hawai‘i_Sports_Hall_of_Fame_ID_(P4365)',
    u'P4366': u'Kansas_Sports_Hall_of_Fame_ID_(P4366)',
    u'P4367': u'Louisiana_Sports_Hall_of_Fame_ID_(P4367)',
    u'P4368': u'Mississippi_Sports_Hall_of_Fame_ID_(P4368)',
    u'P4369': u'Cairn_author_ID_(P4369)',
    u'P4370': u'KBO_hitter_ID_(P4370)',
    u'P4371': u'KBO_pitcher_ID_(P4371)',
    u'P4372': u'Infopatrimônio_ID_(P4372)',
    u'P4373': u'National_Trust_Collections_ID_(P4373)',
    u'P4374': u'New_Mexico_Sports_Hall_of_Fame_ID_(P4374)',
    u'P4375': u'North_Carolina_Sports_Hall_of_Fame_ID_(P4375)',
    u'P4376': u'Oklahoma_Sports_Hall_of_Fame_ID_(P4376)',
    u'P4377': u'Women\'s_Sports_Foundation_ID_(P4377)',
    u'P4379': u'youth_wing_(P4379)',
    u'P4380': u'Sandrart.net_artwork_ID_(P4380)',
    u'P4381': u'Soccerdonna_player_ID_(P4381)',
    u'P4382': u'LFB_player_ID_(P4382)',
    u'P4383': u'Tennessee_Sports_Hall_of_Fame_athlete_ID_(P4383)',
    u'P4384': u'DAMIT_asteroid_ID_(P4384)',
    u'P4385': u'Missouri_Sports_Hall_of_Fame_ID_(P4385)',
    u'P4386': u'Virginia_Sports_Hall_of_Fame_ID_(P4386)',
    u'P4387': u'update_method_(P4387)',
    u'P4388': u'Mappy_place_ID_(P4388)',
    u'P4389': u'Science_Museum_people_ID_(P4389)',
    u'P4390': u'mapping_relation_type_(P4390)',
    u'P4391': u'TheSports.org_athlete_ID_(P4391)',
    u'P4392': u'BC_Sports_Hall_of_Fame_athlete_ID_(P4392)',
    u'P4393': u'Anvisa_drug_ID_(P4393)',
    u'P4394': u'NeuroNames_ID_(plain_mode)_(P4394)',
    u'P4395': u'BrainInfo_ID_(hierarchical)_(P4395)',
    u'P4396': u'Alberta_Sports_Hall_of_Fame_and_Museum_athlete_ID_(P4396)',
    u'P4397': u'International_Paralympic_Committee_athlete_ID_(P4397)',
    u'P4398': u'Canada\'s_Sports_Hall_of_Fame_athlete_ID_(P4398)',
    u'P4399': u'Enciclopédia_Itaú_Cultural_ID_(P4399)',
    u'P4400': u'Memória_Globo_(P4400)',
    u'P4401': u'Museus.br_ID_(P4401)',
    u'P4402': u'Women\'s_Collegiate_Tennis_Hall_of_Fame_ID_(P4402)',
    u'P4403': u'caliber_(P4403)',
    u'P4404': u'MusicBrainz_recording_ID_(P4404)',
    u'P4405': u'NLBPA_ID_(P4405)',
    u'P4406': u'Nova_Scotia_Sport_Hall_of_Fame_ID_(P4406)',
    u'P4407': u'Opera_Vivra_singer_ID_(P4407)',
    u'P4408': u'sports.ru_player_ID_(P4408)',
    u'P4409': u'Baseball_Almanac_ID_(P4409)',
    u'P4410': u'Women\'s_Basketball_Hall_of_Fame_ID_(P4410)',
    u'P4411': u'Quora_username_(P4411)',
    u'P4412': u'Ontario_Sports_Hall_of_Fame_athlete_ID_(P4412)',
    u'P4413': u'Manitoba_Sports_Hall_of_Fame_athlete_ID_(P4413)',
    u'P4414': u'New_Brunswick_Sports_Hall_of_Fame_athlete_ID_(P4414)',
    u'P4415': u'Sport_Australia_Hall_of_Fame_inductee_ID_(P4415)',
    u'P4416': u'Panthéon_des_sports_du_Québec_ID_(P4416)',
    u'P4417': u'rfpl.org_player_ID_(P4417)',
    u'P4418': u'New_Zealand_Sports_Hall_of_Fame_ID_(P4418)',
    u'P4419': u'Videolectures_ID_(P4419)',
    u'P4421': u'Sportbox.ru_ID_(P4421)',
    u'P4422': u'U.S._Ski_and_Snowboard_Hall_of_Fame_athlete_ID_(P4422)',
    u'P4423': u'Portuguese_lighthouse_ID_(P4423)',
    u'P4424': u'mandates_(P4424)',
    u'P4425': u'mtDNA_haplogroup_(P4425)',
    u'P4426': u'Y-DNA_Haplogroup_(P4426)',
    u'P4427': u'GACS_ID_(P4427)',
    u'P4428': u'implementation_of_(P4428)',
    u'P4429': u'Pro14_player_ID_(P4429)',
    u'P4430': u'New_York_City_Parks_Monument_ID_(P4430)',
    u'P4431': u'Google_Doodle_(P4431)',
    u'P4432': u'AKL_Online_Artist_ID_(P4432)',
    u'P4433': u'Indian_Foundation_for_Butterflies_ID_(P4433)',
    u'P4434': u'LesBiographies.com_ID_(P4434)',
    u'P4435': u'snap_package_(P4435)',
    u'P4436': u'The_Coptic_Library_ID_(P4436)',
    u'P4437': u'FPB_rating_(P4437)',
    u'P4438': u'BFI_Film_and_TV_ID_(P4438)',
    u'P4439': u'MNCARS_artist_ID_(P4439)',
    u'P4440': u'Biblioteca_Nacional_de_México_ID_(P4440)',
    u'P4441': u'hydraulic_head_(P4441)',
    u'P4442': u'mean_age_(P4442)',
    u'P4443': u'card_network_(P4443)',
    u'P4444': u'reward_(P4444)',
    u'P4445': u'grace_period_(P4445)',
    u'P4446': u'reward_program_(P4446)',
    u'P4447': u'Minimum_Spend_Bonus_(P4447)',
    u'P4448': u'cash_back_(P4448)',
    u'P4449': u'Shirat_Nashim_person_ID_(P4449)',
    u'P4450': u'HAL_author_ID_(P4450)',
    u'P4451': u'Scoresway_handball_person_ID_(P4451)',
    u'P4452': u'Thibaudeau_classification_(P4452)',
    u'P4453': u'Argentine_Senate_member_ID_(P4453)',
    u'P4454': u'Argentine_Chamber_of_Deputies_ID_(P4454)',
    u'P4455': u'Coinage_of_the_Roman_Republic_Online_ID_(P4455)',
    u'P4456': u'Internet_Off-Broadway_Database_ID_(P4456)',
    u'P4457': u'Discography_of_American_Historical_Recordings_ID_(P4457)',
    u'P4458': u'Dutch_Cemetery_in_Chinsurah_person_ID_(P4458)',
    u'P4459': u'Dictionary_of_Spanish_Biography_ID_(P4459)',
    u'P4460': u'Kaitai_Struct_format_gallery_ID_(P4460)',
    u'P4461': u'World_Golf_Hall_of_Fame_player_ID_(P4461)',
    u'P4462': u'Canadian_Baseball_Hall_of_Fame_inductee_ID_(P4462)',
    u'P4463': u'Canadian_Ski_Hall_of_Fame_inductee_ID_(P4463)',
    u'P4464': u'Rugby_League_Project_player_ID_(P4464)',
    u'P4465': u'Ecole_des_chartes_thesis_abstract_ID_(P4465)',
    u'P4466': u'Unified_Astronomy_Thesaurus_ID_(P4466)',
    u'P4467': u'Twitch_game_ID_(P4467)',
    u'P4468': u'Mountain_Bike_Hall_of_Fame_inductee_ID_(P4468)',
    u'P4469': u'IGHOF_athlete_ID_(P4469)',
    u'P4470': u'British_Bobsleigh_&amp;_Skeleton_Association_ID_(P4470)',
    u'P4471': u'Rush_Parliamentary_Archive_ID_(P4471)',
    u'P4472': u'Czech_Jockey_Club_horse_ID_(P4472)',
    u'P4473': u'Melon_artist_ID_(P4473)',
    u'P4474': u'IBHOF_boxer_ID_(P4474)',
    u'P4475': u'European_Fencing_Confederation_athlete_ID_(P4475)',
    u'P4476': u'Australian_Baseball_League_player_ID_(P4476)',
    u'P4477': u'Humble_Store_ID_(P4477)',
    u'P4478': u'Survey_of_Scottish_Witchcraft_-_Accused_witch_ID_(P4478)',
    u'P4479': u'INDUCKS_character_ID_(P4479)',
    u'P4480': u'INDUCKS_publication_ID_(P4480)',
    u'P4481': u'INDUCKS_creator_ID_(P4481)',
    u'P4482': u'INDUCKS_publisher_ID_(P4482)',
    u'P4483': u'INDUCKS_story_ID_(P4483)',
    u'P4484': u'INDUCKS_issue_ID_(P4484)',
    u'P4485': u'INDUCKS_miniseries_ID_(P4485)',
    u'P4486': u'Google_Play_developer_ID_(P4486)',
    u'P4487': u'Camera_Decision_ID_(P4487)',
    u'P4488': u'COAM_architect_ID_(P4488)',
    u'P4489': u'HOC_athlete_ID_(P4489)',
    u'P4490': u'Team_GB_athlete_ID_(P4490)',
    u'P4491': u'Isidore_ID_(P4491)',
    u'P4493': u'Barbarian_F.C._ID_(P4493)',
    u'P4494': u'Barbarian_R.C._ID_(P4494)',
    u'P4495': u'Xenopus_Anatomical_Ontology_ID_(P4495)',
    u'P4496': u'NACE_code_(P4496)',
    u'P4497': u'Microsoft_Store_artist_ID_(P4497)',
    u'P4498': u'World_Snooker_athlete_ID_(P4498)',
    u'P4500': u'family_relationship_degree_(P4500)',
    u'P4501': u'albedo_(P4501)',
    u'P4502': u'Snooker.org_ID_(P4502)',
    u'P4503': u'Wimbledon_player_ID_(P4503)',
    u'P4504': u'IWRP_athlete_ID_(P4504)',
    u'P4505': u'IAFD_male_performer_ID_(P4505)',
    u'P4506': u'Formal_Public_Identifier_(P4506)',
    u'P4507': u'California_Sports_Hall_of_Fame_athlete_ID_(P4507)',
    u'P4508': u'CycleBase_cyclist_ID_(P4508)',
    u'P4509': u'National_Monuments_of_Namibia_Site_Reference_(P4509)',
    u'P4510': u'describes_a_project_that_uses_(P4510)',
    u'P4511': u'vertical_depth_(P4511)',
    u'P4512': u'Scottish_Cemetery_Kolkata_person_ID_(P4512)',
    u'P4513': u'Africultures_movie_ID_(P4513)',
    u'P4514': u'Africultures_person_ID_(P4514)',
    u'P4515': u'Prisma_ID_(P4515)',
    u'P4516': u'Mémoire_du_cyclisme_cyclist_ID_(P4516)',
    u'P4517': u'ctext_work_ID_(P4517)',
    u'P4518': u'Melon_album_ID_(P4518)',
    u'P4519': u'payload_mass_(P4519)',
    u'P4520': u'SUNCAT_ID_(P4520)',
    u'P4521': u'Radio_Radicale_person_ID_(P4521)',
    u'P4522': u'Alaska_Sports_Hall_of_Fame_athlete_ID_(P4522)',
    u'P4523': u'Vermont_Sports_Hall_of_Fame_athlete_ID_(P4523)',
    u'P4524': u'Survey_of_Scottish_Witchcraft_-_Person_ID_(P4524)',
    u'P4525': u'MuIS_ID_(P4525)',
    u'P4526': u'ABA_bird_ID_(P4526)',
    u'P4527': u'UK_Parliament_thesaurus_ID_(P4527)',
    u'P4528': u'REGINE_water_system_number_(P4528)',
    u'P4529': u'Douban_Movie_ID_(P4529)',
    u'P4530': u'Bangladesh_administrative_division_code_(P4530)',
    u'P4531': u'ChinesePosters_artist_ID_(P4531)',
    u'P4532': u'Survey_of_Scottish_Witchcraft_-_Trial_ID_(P4532)',
    u'P4533': u'Czech_street_ID_(P4533)',
    u'P4534': u'EUTA_person_ID_(P4534)',
    u'P4535': u'EUTA_theatre_ID_(P4535)',
    u'P4536': u'EThOS_thesis_ID_(P4536)',
    u'P4537': u'Spider_Ontology_ID_(P4537)',
    u'P4538': u'Snooker_Database_player_ID_(P4538)',
    u'P4539': u'Collective_Biographies_of_Women_ID_(P4539)',
    u'P4540': u'IBTrACS_cyclone_ID_(P4540)',
    u'P4541': u'HOO_athlete_ID_(P4541)',
    u'P4542': u'World_of_O_athlete_ID_(P4542)',
    u'P4543': u'has_listed_ingredient_(P4543)',
    u'P4544': u'Tennis_Temple_player_ID_(P4544)',
    u'P4545': u'sexually_homologous_with_(P4545)',
    u'P4546': u'Gymn_Forum_athlete_ID_(P4546)',
    u'P4547': u'OKS_athlete_ID_(P4547)',
    u'P4548': u'CGF_athlete_ID_(P4548)',
    u'P4549': u'ARLIMA_ID_(P4549)',
    u'P4550': u'CNRS_research_group_ID_(P4550)',
    u'P4551': u'Everyone_Remembered_ID_(P4551)',
    u'P4552': u'mountain_range_(P4552)',
    u'P4553': u'RA_Collections_ID_(P4553)',
    u'P4554': u'ICRC_PoW_ID_(P4554)',
    u'P4555': u'Canada_Games_ID_(P4555)',
    u'P4556': u'FAI_ID_(P4556)',
    u'P4557': u'Indiana_Basketball_Hall_of_Fame_ID_(P4557)',
    u'P4558': u'SNCZI-IPE_dam_ID_(P4558)',
    u'P4559': u'IJF_ID_(P4559)',
    u'P4560': u'National_Collegiate_Basketball_Hall_of_Fame_ID_(P4560)',
    u'P4561': u'Basketball-Reference.com_WNBA_ID_(P4561)',
    u'P4562': u'Japanese_Baseball_Hall_of_Fame_ID_(P4562)',
    u'P4563': u'Art_Museum_of_Estonia_artist_ID_(P4563)',
    u'P4564': u'Art_Museum_of_Estonia_artwork_ID_(P4564)',
    u'P4565': u'electoral_district_number_(P4565)',
    u'P4566': u'awarded_for_period_(P4566)',
    u'P4567': u'BTO_five-letter_code_(P4567)',
    u'P4568': u'SNCZI-IPE_reservoir_ID_(P4568)',
    u'P4569': u'BPH_journal_ID_(P4569)',
    u'P4570': u'Wikidata_project_(P4570)',
    u'P4571': u'West_Australian_Football_League_player_ID_(P4571)',
    u'P4572': u'SS_KL_Auschwitz_Garrison_ID_(P4572)',
    u'P4573': u'Merchant_Category_Code_(P4573)',
    u'P4574': u'Norwegian_historical_register_of_persons_ID_(P4574)',
    u'P4575': u'HTML_entity_(P4575)',
    u'P4576': u'Tidal_artist_ID_(P4576)',
    u'P4577': u'Tidal_album_ID_(P4577)',
    u'P4578': u'Tidal_track_ID_(P4578)',
    u'P4579': u'Tidal_video_ID_(P4579)',
    u'P4580': u'Berlinische_Galerie_artist_ID_(P4580)',
    u'P4581': u'Städel_Museum_artist_ID_(P4581)',
    u'P4582': u'Kulturelles_Erbe_Köln_object_ID_(P4582)',
    u'P4583': u'U.S._Gymnastics_Hall_of_Fame_athlete_ID_(P4583)',
    u'P4584': u'first_appearance_(P4584)',
    u'P4585': u'Accademia_della_Crusca_ID_(P4585)',
    u'P4586': u'ype_foundry_(P4586)',
    u'P4587': u'Argentinian_Historic_Heritage_ID_(P4587)',
    u'P4588': u'IWGA_athlete_ID_(P4588)',
    u'P4589': u'Dreadnought_Project_page_(P4589)',
    u'P4590': u'Atomic_Heritage_Foundation_ID_(P4590)',
    u'P4591': u'National_Inventory_of_Canadian_Military_Memorials_ID_(P4591)',
    u'P4592': u'Mountain_Project_ID_(P4592)',
    u'P4593': u'CPE_athlete_ID_(P4593)',
    u'P4594': u'arXiv_author_ID_(P4594)',
    u'P4595': u'post_town_(P4595)',
    u'P4596': u'NIOSH_Publication_Number_(P4596)',
    u'P4597': u'FAPESP_institution_ID_(P4597)',
    u'P4598': u'FAPESP_researcher_ID_(P4598)',
    u'P4599': u'monomer_of_(P4599)',
    u'P4600': u'polymer_of_(P4600)',
    u'P4601': u'Lives_of_WWI_ID_(P4601)',
    u'P4602': u'date_of_burial_or_cremation_(P4602)',
    u'P4603': u'Microsoft_Store_album_ID_(P4603)',
    u'P4604': u'World_Rugby_Hall_of_Fame_ID_(P4604)',
    u'P4605': u'South_Dakota_Sports_Hall_of_Fame_ID_(P4605)',
    u'P4606': u'National_Film_Board_of_Canada_movie_ID_(P4606)',
    u'P4607': u'Swedish_Musical_Heritage_composer_ID_(P4607)',
    u'P4608': u'scenographer_(P4608)',
    u'P4609': u'Queensland_Australian_Football_Hall_of_Fame_inductee_ID_(P4609)',
    u'P4610': u'Art_Institute_of_Chicago_artwork_ID_(P4610)',
    u'P4611': u'LACMA_ID_(P4611)',
    u'P4612': u'LKL_player_ID_(P4612)',
    u'P4613': u'Encyclopedia_of_Modern_Ukraine_ID_(P4613)',
    u'P4614': u'watershed_(P4614)',
    u'P4615': u'WAFL_FootyFacts_ID_(P4615)',
    u'P4616': u'UniProt_journal_ID_(P4616)',
    u'P4617': u'DVV_player_ID_(P4617)',
    u'P4618': u'AUSNUT_2011–13_Food_Group_ID_(P4618)',
    u'P4619': u'National_Library_of_Brazil_ID_(P4619)',
    u'P4620': u'Merkelstiftung_person_ID_(P4620)',
    u'P4621': u'Württembergische_Kirchengeschichte_person_ID_(P4621)',
    u'P4622': u'rophy_awarded_(P4622)',
    u'P4623': u'South_Australian_Football_Hall_of_Fame_inductee_ID_(P4623)',
    u'P4624': u'squadron_embarked_(P4624)',
    u'P4625': u'Museum_of_Fine_Arts,_Boston_object_ID_(P4625)',
    u'P4626': u'identity_of_object_in_context_(P4626)',
    u'P4627': u'EK_number_(P4627)',
    u'P4628': u'ICTV_virus_genome_composition_(P4628)',
    u'P4629': u'Online_Books_Page_author_ID_(P4629)',
    u'P4630': u'DORIS_ID_(P4630)',
    u'P4631': u'Iowa_Sports_Hall_of_Fame_ID_(P4631)',
    u'P4632': u'Bechdel_Test_Movie_List_ID_(P4632)',
    u'P4633': u'name_of_the_character_role_(P4633)',
    u'P4634': u'premiere_type_(P4634)',
    u'P4635': u'Indian_Financial_System_Code_(P4635)',
    u'P4636': u'Sherpa_Juliet_ID_(P4636)',
    u'P4637': u'FoodEx2_code_(P4637)',
    u'P4638': u'The_Peerage_person_ID_(P4638)',
    u'P4639': u'Estonian_Football_Association_team_ID_(P4639)',
    u'P4640': u'photosphere_image_(P4640)',
    u'P4641': u'Welsh_Chapels_ID_(P4641)',
    u'P4642': u'Colorado_Sports_Hall_of_Fame_ID_(P4642)',
    u'P4643': u'Philadelphia_Museum_of_Art_ID_(P4643)',
    u'P4644': u'ESCO_skill_ID_(P4644)',
    u'P4645': u'RCR_number_(P4645)',
    u'P4646': u'representation_of_(P4646)',
    u'P4647': u'location_of_first_performance_(P4647)',
    u'P4649': u'identity_of_subject_in_context_(P4649)',
    u'P4650': u'BSD_Portal_athlete_ID_(P4650)',
    u'P4651': u'Welsh_assembly_ID_(P4651)',
    u'P4652': u'ESCO_Occupation_ID_(P4652)',
    u'P4653': u'partition_table_type_(P4653)',
    u'P4654': u'partition_type_identifier_(P4654)',
    u'P4655': u'Flathub_ID_(P4655)',
    u'P4656': u'Wikimedia_import_URL_(P4656)',
    u'P4657': u'The_Numbers_person_ID_(P4657)',
    u'P4658': u'Coflein_ID_(P4658)',
    u'P4659': u'Musée_d\'Orsay_artwork_ID_(P4659)',
    u'P4660': u'CPDOC_ID_(P4660)',
    u'P4661': u'reservoir_created_(P4661)',
    u'P4662': u'SAN_ID_(P4662)',
    u'P4663': u'DACS_ID_(P4663)',
    u'P4664': u'Wiki_Aves_bird_ID_(P4664)',
    u'P4665': u'Cinemagia_film_ID_(P4665)',
    u'P4666': u'Cinemagia_actor_ID_(P4666)',
    u'P4667': u'Buenos_Aires_legislator_ID_(P4667)',
    u'P4668': u'USA_Gymnastics_Hall_of_Fame_athlete_ID_(P4668)',
    u'P4669': u'abular_software_version_(P4669)',
    u'P4670': u'Sjukvårdsrådgivningen_Category_ID_(P4670)',
    u'P4671': u'Hall_of_Light_Amiga_database_ID_(P4671)',
    u'P4672': u'EMLO_location_ID_(P4672)',
    u'P4673': u'Museum_of_Fine_Arts,_Houston_object_ID_(P4673)',
    u'P4674': u'Indianapolis_Museum_of_Art_artwork_ID_(P4674)',
    u'P4675': u'appears_in_the_form_of_(P4675)',
    u'P4676': u'MAVISE_TV_channel_ID_(P4676)',
    u'P4677': u'MAVISE_on-demand_audiovisual_service_ID_(P4677)',
    u'P4678': u'MAVISE_competent_authority_ID_(P4678)',
    u'P4679': u'MAVISE_company_ID_(P4679)',
    u'P4680': u'constraint_scope_(P4680)',
    u'P4681': u'Eldoblaje_dubbing_actor_ID_(P4681)',
    u'P4682': u'Eldoblaje_original_actor_ID_(P4682)',
    u'P4683': u'National_Gallery_of_Art_artwork_ID_(P4683)',
    u'P4684': u'National_Gallery_of_Victoria_artwork_ID_(P4684)',
    u'P4685': u'Nintendo_Game_Store_ID_(P4685)',
    u'P4686': u'Carnegie_Museum_of_Art_ID_(P4686)',
    u'P4687': u'Dictionnaire_des_peintres_belges_ID_(P4687)',
    u'P4688': u'geomorphological_unit_(P4688)',
    u'P4689': u'Environmental_Register_code_(Estonia)_(P4689)',
    u'P4690': u'Oireachtas_member_ID_(P4690)',
    u'P4691': u'OpenSecrets_organization_ID_(P4691)',
    u'P4692': u'American_Art_Collaborative_object_ID_(P4692)',
    u'P4693': u'Argentine_deputy_ID_(P4693)',
    u'P4694': u'Arquivo_Arq_ID_(P4694)',
    u'P4695': u'GEMS_Code_(P4695)',
    u'P4696': u'CIQUAL2017_ID_(P4696)',
    u'P4697': u'Historic_Place_Names_of_Wales_ID_(P4697)',
    u'P4698': u'Conseil_de_Presse_Luxembourg_journalist_ID_(P4698)',
    u'P4699': u'Canal-U_channel_ID_(P4699)',
    u'P4700': u'Cairn_journal_ID_(P4700)',
    u'P4701': u'Google_Arts_&amp;_Culture_asset_ID_(P4701)',
    u'P4702': u'Google_Arts_&amp;_Culture_partner_ID_(P4702)',
    u'P4703': u'HATVP_ID_(P4703)',
    u'P4704': u'Smithsonian_American_Art_Museum_ID_(P4704)',
    u'P4705': u'World_of_Spectrum_ID_(P4705)',
    u'P4706': u'Ent\'revues_ID_(P4706)',
    u'P4707': u'Rugby_Canada_ID_(P4707)',
    u'P4708': u'VOGRIPA_ID_(P4708)',
    u'P4709': u'Barnes_Foundation_ID_(P4709)',
    u'P4710': u'Guardiana_ID_(P4710)',
    u'P4711': u'CHGIS_ID_(P4711)',
    u'P4712': u'Minneapolis_Institute_of_Art_artwork_ID_(P4712)',
    u'P4713': u'Walters_Art_Museum_ID_(P4713)',
    u'P4714': u'itle_page_number_(P4714)',
    u'P4715': u'Guide_to_North_American_Birds_ID_(P4715)',
    u'P4717': u'Académie française member ID'
}
